@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2004-09-03
+ *  Updated   : 2008-04-23
  *  Notes     :
  *
- * Copyright (c) 1997-2004 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2008 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,6 +45,9 @@
 #ifndef RFC2046_H
 #	include <rfc2046.h>
 #endif /* !RFC2046_H */
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
 
 /*
  * The prototypes bracketed by DECL_xxxx ifdef's are used to get moderately
@@ -75,6 +78,9 @@
 #ifdef DECL_CALLOC
 	extern void *calloc(size_t, size_t);
 #endif /* DECL_CALLOC */
+#ifdef DECL_FCHMOD
+	extern int fchmod(int, mode_t);
+#endif /* DECL_FCHMOD */
 #ifdef DECL_FCLOSE
 	extern int fclose(FILE *);
 #endif /* DECL_FCLOSE */
@@ -229,12 +235,18 @@
 #ifdef DECL_STRNCASECMP
 	extern int strncasecmp(const char *, const char *, size_t);
 #endif /* DECL_STRNCASECMP */
+#ifdef DECL_STRSEP
+	extern char *strsep(char **, const char *);
+#endif /* DECL_STRSEP */
 #ifdef DECL_STRTOL
 	extern long strtol(const char *, char **, int);
 #endif /* DECL_STRTOL */
 #ifdef DECL_SYSTEM
 	extern int system(const char *);
 #endif /* DECL_SYSTEM */
+#ifdef DECL_TMPFILE
+	extern  FILE *tmpfile(void);
+#endif /* DECL_TMPFILE */
 #ifdef DECL_TGETENT
 	extern int tgetent(char *, char *);
 #endif /* DECL_TGETENT */
@@ -257,7 +269,7 @@
 	extern int tigetnum(char *);
 #endif /* DECL_TGETNUM */
 #ifdef DECL_TIGETSTR
-	extern char *tigetstr(char *, char **);
+	extern char *tigetstr(char *);
 #endif /* DECL_TIGETSTR */
 #ifdef DECL_TIME
 	extern time_t time(time_t *);
@@ -325,9 +337,9 @@ extern char **news_headers_to_display_array;
 extern char **news_headers_to_not_display_array;
 extern char *OPT_CHAR_list[];
 extern char *OPT_STRING_list[];
-extern char *ch_post_process;
 extern char *nntp_server;
 extern char active_times_file[PATH_LEN];
+extern char article_name[PATH_LEN];
 extern char bug_addr[LEN];
 extern char bug_nntpserver1[PATH_LEN];
 extern char bug_nntpserver2[PATH_LEN];
@@ -385,8 +397,11 @@ extern const char base64_alphabet[64];
 extern constext *content_encodings[];
 extern constext *content_types[];
 extern constext *txt_attrs[];
-extern constext *txt_colors[];
+#ifdef HAVE_COLOR
+	extern constext *txt_colors[];
+#endif /* HAVE_COLOR */
 extern constext *txt_confirm_choices[];
+extern constext *txt_goto_next_unread_options[];
 extern constext *txt_hide_uue_type[];
 extern constext *txt_interactive_mailers[];
 extern constext *txt_kill_level_type[];
@@ -431,6 +446,7 @@ extern constext txt_art_posted[];
 extern constext txt_art_rejected[];
 extern constext txt_art_thread_regex_tag[];
 extern constext txt_art_unavailable[];
+extern constext txt_art_x_of_n[];
 extern constext txt_article_cancelled[];
 extern constext txt_article_plural[];
 extern constext txt_article_reposted[];
@@ -473,9 +489,9 @@ extern constext txt_cannot_create[];
 extern constext txt_cannot_create_uniq_name[];
 extern constext txt_cannot_find_base_art[];
 extern constext txt_cannot_get_nntp_server_name[];
-#if defined(M_UNIX) && !defined(USE_TERMINFO)
+#if !defined(USE_CURSES) && defined(M_UNIX) && !defined(USE_TERMINFO)
 	extern constext txt_cannot_get_term_entry[];
-#endif /* M_UNIX && !USE_TERMINFO */
+#endif /* !USE_CURSES && M_UNIX && !USE_TERMINFO */
 extern constext txt_cannot_open[];
 extern constext txt_cannot_open_for_saving[];
 extern constext txt_cannot_post[];
@@ -491,12 +507,14 @@ extern constext txt_caughtup[];
 extern constext txt_check_article[];
 extern constext txt_checking_for_news[];
 extern constext txt_checking_new_groups[];
-#ifndef HAVE_LIBUU
+#if !defined(HAVE_LIBUU) && defined(M_UNIX) && defined(HAVE_SUM) && !defined(DONT_HAVE_PIPING)
 	extern constext txt_checksum_of_file[];
-#endif /* !HAVE_LIBUU */
+#endif /* !HAVE_LIBUU && M_UNIX && HAVE SUM && !DONT_HAVE_PIPING */
 extern constext txt_choose_post_process_type[];
-extern constext txt_color_off[];
-extern constext txt_color_on[];
+#ifdef HAVE_COLOR
+	extern constext txt_color_off[];
+	extern constext txt_color_on[];
+#endif /* HAVE_COLOR */
 extern constext txt_command_failed[];
 extern constext txt_confirm_select_on_exit[];
 #ifdef NNTP_ABLE
@@ -692,6 +710,7 @@ extern constext txt_help_filter_text[];
 extern constext txt_help_filter_text_type[];
 extern constext txt_help_filter_time[];
 extern constext txt_help_global_article_range[];
+extern constext txt_help_global_edit_filter[];
 extern constext txt_help_global_esc[];
 extern constext txt_help_global_help[];
 extern constext txt_help_global_last_art[];
@@ -716,6 +735,8 @@ extern constext txt_help_global_quit_tin[];
 extern constext txt_help_global_redraw_screen[];
 extern constext txt_help_global_save[];
 extern constext txt_help_global_auto_save[];
+extern constext txt_help_global_scroll_down[];
+extern constext txt_help_global_scroll_up[];
 extern constext txt_help_global_search_auth_backwards[];
 extern constext txt_help_global_search_auth_forwards[];
 extern constext txt_help_global_search_body[];
@@ -815,6 +836,9 @@ extern constext txt_info_postponed[];
 extern constext txt_info_x_conversion_note[];
 extern constext txt_invalid_from[];
 extern constext txt_invalid_sender[];
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern constext txt_invalid_multibyte_sequence[];
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 extern constext txt_inverse_off[];
 extern constext txt_inverse_on[];
 extern constext txt_is_mailbox[];
@@ -822,10 +846,7 @@ extern constext txt_is_tex_encoded[];
 extern constext txt_keymap_missing_key[];
 extern constext txt_keymap_invalid_key[];
 extern constext txt_keymap_invalid_name[];
-#ifdef DEBUG
-	extern constext txt_keymap_redef[];
-#endif /* DEBUG */
-extern constext txt_keymap_conflict[];
+extern constext txt_keymap_upgraded[];
 extern constext txt_kill_from[];
 extern constext txt_kill_lines[];
 extern constext txt_kill_menu[];
@@ -869,7 +890,9 @@ extern constext txt_mini_thread_1[];
 extern constext txt_mini_thread_2[];
 extern constext txt_more[];
 extern constext txt_moving[];
+extern constext txt_msgid_line_last[];
 extern constext txt_msgid_line_only[];
+extern constext txt_msgid_refs_line[];
 extern constext txt_name[];
 extern constext txt_newsgroup[];
 extern constext txt_newsgroup_plural[];
@@ -901,11 +924,13 @@ extern constext txt_no_responses[];
 extern constext txt_no_resps_in_thread[];
 extern constext txt_no_search_string[];
 extern constext txt_no_subject[];
-extern constext txt_no_term_clear_eol[];
-extern constext txt_no_term_clear_eos[];
-extern constext txt_no_term_clearscreen[];
-extern constext txt_no_term_cursor_motion[];
-extern constext txt_no_term_set[];
+#ifndef USE_CURSES
+	extern constext txt_no_term_clear_eol[];
+	extern constext txt_no_term_clear_eos[];
+	extern constext txt_no_term_clearscreen[];
+	extern constext txt_no_term_cursor_motion[];
+	extern constext txt_no_term_set[];
+#endif /* !USE_CURSES */
 extern constext txt_no_viewer_found[];
 extern constext txt_no_xover_support[];
 extern constext txt_not_exist[];
@@ -956,7 +981,6 @@ extern constext txt_quoted_printable[];
 extern constext txt_range_invalid[];
 extern constext txt_read_abort[];
 extern constext txt_read_exit[];
-extern constext txt_read_resp[];
 extern constext txt_reading_article[];
 extern constext txt_reading_arts[];
 extern constext txt_reading_attributes_file[];
@@ -972,6 +996,7 @@ extern constext txt_reading_newsrc[];
 #ifdef NNTP_ABLE
 	extern constext txt_reconnect_to_news_server[];
 #endif /* NNTP_ABLE */
+extern constext txt_refs_line_only[];
 #ifdef HAVE_GETTIMEOFDAY
 	extern constext txt_remaining[];
 #endif /* HAVE_GETTIMEOFDAY */
@@ -984,7 +1009,6 @@ extern constext txt_repost_group[];
 extern constext txt_reset_newsrc[];
 extern constext txt_resp_redirect[];
 extern constext txt_resp_to_poster[];
-extern constext txt_resp_x_of_n[];
 extern constext txt_return_key[];
 extern constext txt_save[];
 extern constext txt_save_attachment[];
@@ -999,7 +1023,9 @@ extern constext txt_saved_to[];
 extern constext txt_saved_to_range[];
 extern constext txt_saving[];
 extern constext txt_screen_init_failed[];
-extern constext txt_screen_too_small[];
+#ifndef USE_CURSES
+	extern constext txt_screen_too_small[];
+#endif /* !USE_CURSES */
 extern constext txt_screen_too_small_exiting[];
 extern constext txt_search_backwards[];
 extern constext txt_search_body[];
@@ -1041,12 +1067,15 @@ extern constext txt_supersede_group[];
 extern constext txt_superseding_art[];
 extern constext txt_suspended_message[];
 extern constext txt_tab[];
+extern constext txt_tex[];
 extern constext txt_there_is_no_news[];
 extern constext txt_thread_upper[];
 extern constext txt_thread_com[];
 extern constext txt_thread_marked_as_deselected[];
 extern constext txt_thread_marked_as_selected[];
-extern constext txt_thread_plural[];
+#if 0 /* unused */
+	extern constext txt_thread_plural[];
+#endif /* 0 */
 extern constext txt_thread_range[];
 extern constext txt_thread_singular[];
 extern constext txt_thread_x_of_n[];
@@ -1120,6 +1149,7 @@ extern constext txt_usage_verbose[];
 extern constext txt_usage_version[];
 extern constext txt_useful_without_batch_mode[];
 extern constext txt_useful_with_batch_mode[];
+extern constext txt_useless_combination[];
 extern constext txt_use_mime[];
 extern constext txt_value_out_of_range[];
 extern constext txt_view_attachment[];
@@ -1156,8 +1186,6 @@ extern constext txt_show_unread[];
 extern constext txt_yes[];
 extern constext txt_you_have_mail[];
 
-extern constext *xover_cmd;
-
 #ifndef DISABLE_PRINTING
 	extern constext txt_print[];
 	extern constext txt_printing[];
@@ -1201,18 +1229,19 @@ extern int cCOLS;
 extern int cLINES;
 extern int curr_line;
 extern int debug;
-extern int i_key_search_last;
 extern int input_context;
 extern int iso2asc_supported;
 extern int last_resp;
 extern int max_active;
 extern int max_art;
+extern int max_local_attributes;
 extern int max_newnews;
 extern int max_save;
+extern int need_resize;
 extern int num_headers_to_display;
 extern int num_headers_to_not_display;
-extern int need_resize;
 extern int num_active;
+extern int num_local_attributes;
 extern int num_newnews;
 extern int num_of_tagged_arts;
 extern int num_save;
@@ -1237,6 +1266,8 @@ extern struct regex_cache strip_re_regex;
 extern struct regex_cache strip_was_regex;
 extern struct regex_cache uubegin_regex;
 extern struct regex_cache uubody_regex;
+extern struct regex_cache verbatim_begin_regex;
+extern struct regex_cache verbatim_end_regex;
 extern struct regex_cache url_regex;
 extern struct regex_cache mail_regex;
 extern struct regex_cache news_regex;
@@ -1252,6 +1283,7 @@ extern struct regex_cache strokes_regex;
 #endif /* HAVE_COLOR */
 
 extern struct t_article *arts;
+extern struct t_attribute *local_attributes;
 extern struct t_attribute glob_attributes;
 extern struct t_config tinrc;
 extern struct t_filters glob_filter;
@@ -1260,6 +1292,7 @@ extern struct t_group *curr_group;
 extern struct t_newnews *newnews;
 extern struct t_option option_table[];
 extern struct t_save *save;
+extern struct t_capabilities nntp_caps;
 
 extern t_bool *OPT_ON_OFF_list[];
 extern t_bool can_post;
@@ -1267,6 +1300,7 @@ extern t_bool check_for_new_newsgroups;
 extern t_bool cmd_line;
 extern t_bool created_rcdir;
 extern t_bool dangerous_signal_exit; /* TRUE if SIGHUP, SIGTERM, SIGUSR1 */
+extern t_bool did_reconnect;
 extern t_bool disable_gnksa_domain_check;
 extern t_bool disable_sender;
 extern t_bool force_no_post;
@@ -1289,6 +1323,9 @@ extern t_bool show_subject;
 extern t_bool batch_mode;
 extern t_bool verbose;
 extern t_bool xref_supported;
+extern t_bool expensive_over_parse;
+
+extern t_function last_search;
 
 extern t_menu selmenu;
 extern t_menu grpmenu;
@@ -1296,6 +1333,8 @@ extern t_menu *currmenu;
 
 extern t_openartinfo pgart;
 
+extern struct t_overview_fmt *ofmt;
+
 enum {
 	HIST_OTHER = 0,
 	HIST_ART_SEARCH,
@@ -1460,7 +1499,9 @@ extern struct opttxt txt_batch_save;
 extern struct opttxt txt_beginner_level;
 extern struct opttxt txt_cache_overview_files;
 extern struct opttxt txt_catchup_read_groups;
-extern struct opttxt txt_color_options;
+#ifdef HAVE_COLOR
+	extern struct opttxt txt_color_options;
+#endif /* HAVE_COLOR */
 extern struct opttxt txt_confirm_choice;
 extern struct opttxt txt_date_format;
 extern struct opttxt txt_display_options;
@@ -1472,6 +1513,7 @@ extern struct opttxt txt_filtering_options;
 extern struct opttxt txt_force_screen_redraw;
 extern struct opttxt txt_getart_limit;
 extern struct opttxt txt_getart_limit_options;
+extern struct opttxt txt_goto_next_unread;
 extern struct opttxt txt_group_catchup_on_exit;
 extern struct opttxt txt_groupname_max_length;
 extern struct opttxt txt_hide_uue;
@@ -1493,12 +1535,17 @@ extern struct opttxt txt_mono_markstar;
 extern struct opttxt txt_mono_markdash;
 extern struct opttxt txt_mono_markslash;
 extern struct opttxt txt_mono_markstroke;
-extern struct opttxt txt_mm_charset;
+#ifndef CHARSET_CONVERSION
+	extern struct opttxt txt_mm_charset;
+#else
+#	ifdef NO_LOCALE
+		extern struct opttxt txt_mm_local_charset;
+#	endif /* NO_LOCALE */
+#endif /* CHARSET_CONVERSION */
 extern struct opttxt txt_metamail_prog;
 extern struct opttxt txt_news_headers_to_display;
 extern struct opttxt txt_news_headers_to_not_display;
 extern struct opttxt txt_news_quote_format;
-extern struct opttxt txt_pgdn_goto_next;
 extern struct opttxt txt_pos_first_unread;
 extern struct opttxt txt_post_8bit_header;
 extern struct opttxt txt_post_mime_encoding;
@@ -1506,8 +1553,10 @@ extern struct opttxt txt_post_process;
 extern struct opttxt txt_post_process_view;
 extern struct opttxt txt_posted_articles_file;
 extern struct opttxt txt_posting_options;
-extern struct opttxt txt_print_header;
-extern struct opttxt txt_printer;
+#ifndef DISABLE_PRINTING
+	extern struct opttxt txt_print_header;
+	extern struct opttxt txt_printer;
+#endif /* !DISABLE_PRINTING */
 extern struct opttxt txt_process_only_unread;
 extern struct opttxt txt_prompt_followupto;
 extern struct opttxt txt_quote_chars;
@@ -1533,7 +1582,6 @@ extern struct opttxt txt_signature_repost;
 extern struct opttxt txt_slashes_regex;
 extern struct opttxt txt_sort_article_type;
 extern struct opttxt txt_sort_threads_type;
-extern struct opttxt txt_space_goto_next_unread;
 extern struct opttxt txt_spamtrap_warning_addresses;
 extern struct opttxt txt_stars_regex;
 extern struct opttxt txt_start_editor_offset;
@@ -1543,9 +1591,9 @@ extern struct opttxt txt_strip_newsrc;
 extern struct opttxt txt_strip_re_regex;
 extern struct opttxt txt_strip_was_regex;
 extern struct opttxt txt_strokes_regex;
-extern struct opttxt txt_tab_goto_next_unread;
 extern struct opttxt txt_tex2iso_conv;
 extern struct opttxt txt_thread_articles;
+extern struct opttxt txt_thread_perc;
 extern struct opttxt txt_thread_catchup_on_exit;
 extern struct opttxt txt_thread_score;
 extern struct opttxt txt_underscores_regex;
@@ -1553,6 +1601,8 @@ extern struct opttxt txt_unlink_article;
 extern struct opttxt txt_url_handler;
 extern struct opttxt txt_url_highlight;
 extern struct opttxt txt_use_mouse;
+extern struct opttxt txt_verbatim_begin_regex;
+extern struct opttxt txt_verbatim_end_regex;
 extern struct opttxt txt_wildcard;
 extern struct opttxt txt_word_highlight;
 extern struct opttxt txt_word_h_display_marks;
