@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2004-09-05
+ *  Updated   : 2008-04-23
  *  Notes     :
  *
- * Copyright (c) 1997-2004 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2008 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,13 @@
 #ifndef PROTO_H
 #	define PROTO_H 1
 
+#ifndef KEYMAP_H
+#	include "keymap.h"
+#endif /* !KEYMAP_H */
+#ifndef OPTIONS_MENU_H
+#	include "options_menu.h"
+#endif /* !OPTIONS_MENU_H */
+
 /* This fixes ambiguities on platforms that don't distinguish extern case */
 #ifdef CASE_PROBLEM
 #	define Raw tin_raw
@@ -78,20 +85,19 @@ extern void write_attributes_file(const char *file);
 
 /* charset.c */
 extern char *convert_to_printable(char *buf);
-extern char *convert_body2printable(char* buf);
 extern t_bool is_art_tex_encoded(FILE *fp);
-extern void convert_iso2asc(char *iso, char **asc_buffer, int *max_line_len, int t);
+extern void convert_iso2asc(char *iso, char **asc_buffer, size_t *max_line_len, int t);
 extern void convert_tex2iso(char *from, char *to);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern wchar_t *wconvert_to_printable(wchar_t *wbuf);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* color.c */
+extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
 #ifdef HAVE_COLOR
 	extern void bcol(int color);
 	extern void fcol(int color);
 #endif /* HAVE_COLOR */
-extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
 
 /* config.c */
 extern char **ulBuildArgv(char *cmd, int *new_argc);
@@ -110,29 +116,18 @@ extern void write_config_file(char *file);
 /* cook.c */
 extern const char *get_filename(t_param *ptr);
 extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int tabs, int hide_uue);
-extern t_bool expand_ctrl_chars(char **line, int *length, size_t cook_width);
+extern t_bool expand_ctrl_chars(char **line, size_t *length, size_t cook_width);
 
 /* curses.c */
 extern OUTC_RETTYPE outchar(OUTC_ARGS);
 extern int InitScreen(void);
 extern int RawState(void);
 extern int ReadCh(void);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && !defined(USE_CURSES)
-	extern wint_t ReadWch(void);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE && !USE_CURSES */
 extern int get_arrow_key(int prech);
 extern int get_termcaps(void);
-extern void ClearScreen(void);
-extern void CleartoEOLN(void);
-extern void CleartoEOS(void);
-extern void ScrollScreen(int lines_to_scroll);
-extern void SetScrollRegion(int topline, int bottomline);
 extern void EndInverse(void);
 extern void EndWin(void);
 extern void InitWin(void);
-#ifndef USE_CURSES
-	extern void MoveCursor(int row, int col);
-#endif /* USE_CURSES */
 extern void Raw(int state);
 extern void StartInverse(void);
 extern void cursoroff(void);
@@ -144,6 +139,17 @@ extern void set_xclick_off(void);
 extern void set_xclick_on(void);
 extern void setup_screen(void);
 extern void word_highlight_string(int row, int col, int size, int color);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && !defined(USE_CURSES)
+	extern wint_t ReadWch(void);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE && !USE_CURSES */
+#ifndef USE_CURSES
+	extern void ClearScreen(void);
+	extern void CleartoEOLN(void);
+	extern void CleartoEOS(void);
+	extern void MoveCursor(int row, int col);
+	extern void ScrollScreen(int lines_to_scroll);
+	extern void SetScrollRegion(int topline, int bottomline);
+#endif /* USE_CURSES */
 #if 0
 	extern void ToggleInverse(void);
 #endif /* 0 */
@@ -151,20 +157,18 @@ extern void word_highlight_string(int row, int col, int size, int color);
 /* debug.c */
 #ifdef DEBUG
 	extern void debug_delete_files(void);
-	extern void debug_nntp(const char *func, const char *line);
+	extern void debug_print_file(const char *fname, const char *fmt, ...);
 	extern void debug_print_active(void);
 	extern void debug_print_arts(void);
+	extern void debug_print_bitmap(struct t_group *group, struct t_article *art);
+	extern void debug_print_comment(const char *comment);
 	extern void debug_print_filters(void);
 	extern void debug_print_header(struct t_article *s);
 	extern void debug_print_malloc(int is_malloc, const char *xfile, int line, size_t size);
+#	ifdef NNTP_ABLE
+		extern void debug_print_nntp_extensions(void);
+#	endif /* NNTP_ABLE */
 #endif /* DEBUG */
-#ifdef DEBUG_NEWSRC
-	extern void debug_print_newsrc(struct t_newsrc *NewSrc, FILE *fp);
-#endif /* DEBUG_NEWSRC */
-#if defined(DEBUG) || defined(DEBUG_NEWSRC)
-	extern void debug_print_bitmap(struct t_group *group, struct t_article *art);
-	extern void debug_print_comment(const char *comment);
-#endif /* DEBUG || DEBUG_NEWSRC */
 
 /* envarg.c */
 extern void envargs(int *Pargc, char ***Pargv, const char *envstr);
@@ -174,8 +178,8 @@ extern void feed_articles(int function, int level, struct t_group *group, int re
 
 /* filter.c */
 extern t_bool filter_articles(struct t_group *group);
-extern t_bool filter_menu(int type, struct t_group *group, struct t_article *art);
-extern t_bool quick_filter(int type, struct t_group *group, struct t_article *art);
+extern t_bool filter_menu(t_function type, struct t_group *group, struct t_article *art);
+extern t_bool quick_filter(t_function type, struct t_group *group, struct t_article *art);
 extern t_bool quick_filter_select_posted_art(struct t_group *group, const char *subj, const char *a_message_id);
 extern t_bool read_filter_file(const char *file);
 extern void free_filter_array(struct t_filters *ptr);
@@ -197,6 +201,8 @@ extern void top_of_list(void);
 extern void end_of_list(void);
 extern void move_to_item(int n);
 extern void prompt_item_num(int ch, const char *prompt);
+extern void scroll_down(void);
+extern void scroll_up(void);
 extern void set_first_screen_item(void);
 
 /* group.c */
@@ -238,13 +244,12 @@ extern void postinit_regexp(void);
 #endif /* HAVE_COLOR */
 
 /* joinpath.c */
-extern void joinpath(char *result, const char *dir, const char *file);
+extern void joinpath(char *result, size_t result_size, const char *dir, const char *file);
 
 /* keymap.c */
-extern char *printascii(char *buf, int ch);
 extern t_bool read_keymap_file(void);
-extern void build_keymaps(void);
 extern void free_keymaps(void);
+extern void setup_default_keys(void);
 
 /* langinfo.c */
 #ifndef NO_LOCALE
@@ -253,9 +258,9 @@ extern void free_keymaps(void);
 
 /* list.c */
 extern char *random_organization(char *in_org);
-extern int find_group_index(const char *group);
+extern int find_group_index(const char *group, t_bool ignore_case);
 extern struct t_group *group_add(const char *group);
-extern struct t_group *group_find(const char *group_name);
+extern struct t_group *group_find(const char *group_name, t_bool ignore_case);
 extern unsigned long hash_groupname(const char *group);
 extern void group_rehash(t_bool yanked_out);
 extern void init_group_hash(void);
@@ -290,6 +295,7 @@ extern void giveup(void);
 /* memory.c */
 extern void expand_active(void);
 extern void expand_art(void);
+extern void expand_local_attributes(void);
 extern void expand_newnews(void);
 extern void expand_save(void);
 extern void init_alloc(void);
@@ -350,9 +356,9 @@ extern void draw_mark_selected(int i);
 extern void draw_percent_mark(long cur_num, long max_num);
 extern void get_author(t_bool thread, struct t_article *art, char *str, size_t len);
 extern void get_cwd(char *buf);
-extern void make_base_group_path(const char *base_dir, const char *group_name, char *group_path);
+extern void make_base_group_path(const char *base_dir, const char *group_name, char *group_path, size_t group_path_len);
 extern void make_group_path(const char *name, char *path);
-extern void process_charsets(char **line, int *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
+extern void process_charsets(char **line, size_t *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
 extern void read_input_history_file(void);
 extern void rename_file(const char *old_filename, const char *new_filename);
 extern void show_inverse_video_status(void);
@@ -398,36 +404,34 @@ extern void subscribe(struct t_group *group, int sub_state, t_bool get_info);
 extern void thd_mark_read(struct t_group *group, long thread);
 extern void thd_mark_unread(struct t_group *group, long thread);
 extern void set_default_bitmap(struct t_group *group);
-#ifdef DEBUG_NEWSRC
-	extern void newsrc_test_harness(void);
-#endif /* DEBUG_NEWSRC */
 
 /* nntplib.c */
-#ifdef NNTP_ABLE
-	extern FILE *nntp_command(const char *, int, char *, size_t);
-#endif /* NNTP_ABLE */
 extern FILE *get_nntp_fp(FILE *fp);
 extern FILE *get_nntp_wr_fp(FILE *fp);
 extern char *getserverbyfile(const char *file);
-extern char *get_server(char *string, int size);
-extern int get_respcode(char *, size_t);
-extern int get_only_respcode(char *, size_t);
 extern int nntp_open(void);
 extern void nntp_close(void);
-extern void put_server(const char *string);
-extern void u_put_server(const char *string);
+#ifdef NNTP_ABLE
+	extern FILE *nntp_command(const char *, int, char *, size_t);
+	extern char *get_server(char *string, int size);
+	extern int get_respcode(char *, size_t);
+	extern int get_only_respcode(char *, size_t);
+	extern int new_nntp_command(const char *command, int success, char *message, size_t mlen);
+	extern void put_server(const char *string);
+	extern void u_put_server(const char *string);
+#endif /* NNTP_ABLE */
 
 /* nrctbl.c */
-extern int get_newsrcname(char *newsrc_name, const char *nntpserver_name);
-extern void get_nntpserver(char *nntpserver_name, char *nick_name);
+extern int get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const char *nntpserver_name);
+extern void get_nntpserver(char *nntpserver_name, size_t nntpserver_name_len, char *nick_name);
 
 /* options_menu.c */
-extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, int option);
-extern int change_config_file(struct t_group *group);
-extern int option_row(int option);
-extern t_bool option_is_visible(int option);
+extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, enum option_enum option);
+extern void change_config_file(struct t_group *group);
+extern int option_row(enum option_enum option);
+extern t_bool option_is_visible(enum option_enum option);
 extern void check_score_defaults(void);
-extern void refresh_config_page(int act_option);
+extern void refresh_config_page(enum option_enum act_option);
 extern void show_menu_help(const char *help_message);
 
 /* page.c */
@@ -473,7 +477,6 @@ extern t_bool user_posted_messages(void);
 extern void init_postinfo(void);
 extern void quick_post_article(t_bool postponed_only);
 #ifdef USE_CANLOCK
-	extern const char *build_cankey(const char *messageid, const char *secret);
 	extern const char *build_canlock(const char *messageid, const char *secret);
 	extern char *get_secret(void);
 #endif /* USE_CANLOCK */
@@ -482,15 +485,15 @@ extern void quick_post_article(t_bool postponed_only);
 extern char *prompt_string_default(const char *prompt, char *def, const char *failtext, int history);
 extern char *sized_message(char **result, const char *format, const char *subject);
 extern int prompt_num(int ch, const char *prompt);
-extern int prompt_yn(int line, const char *prompt, t_bool default_answer);
+extern int prompt_yn(const char *prompt, t_bool default_answer);
 extern int prompt_msgid(void);
 extern t_bool prompt_default_string(const char *prompt, char *buf, int buf_len, char *default_prompt, int which_hist);
 extern t_bool prompt_menu_string(int line, const char *prompt, char *var);
-extern t_bool prompt_option_char(int option);
-extern t_bool prompt_option_list(int option);
-extern t_bool prompt_option_num(int option);
-extern t_bool prompt_option_on_off(int option);
-extern t_bool prompt_option_string(int option);
+extern t_bool prompt_option_char(enum option_enum option);
+extern t_bool prompt_option_list(enum option_enum option);
+extern t_bool prompt_option_num(enum option_enum option);
+extern t_bool prompt_option_on_off(enum option_enum option);
+extern t_bool prompt_option_string(enum option_enum option);
 extern t_bool prompt_string(const char *prompt, char *buf, int which_hist);
 extern void prompt_continue(void);
 extern void prompt_slk_redraw(void);
@@ -520,6 +523,8 @@ extern t_mailcap *get_mailcap_entry(t_part *part, const char *path);
 extern void free_mailcap(t_mailcap *tmailcap);
 
 /* rfc2045.c */
+extern int read_decoded_base64_line(FILE *file, char **line, size_t *max_line_len, const int max_lines_to_read, char **rest);
+extern int read_decoded_qp_line(FILE *file, char **line, size_t *max_line_len, const int max_lines_to_read);
 extern void rfc1521_encode(char *line, FILE *f, int e);
 
 /* rfc2046.c */
@@ -547,13 +552,13 @@ extern void compose_mail_text_plain(const char *filename, struct t_group *group)
 /* save.c */
 extern int check_start_save_any_news(int function, t_bool catchup);
 extern t_bool create_path(const char *path);
-extern t_bool expand_save_filename(char *outpath, const char *path);
-extern t_bool post_process_files(int proc_type_ch, t_bool auto_delete);
+extern t_bool post_process_files(t_function proc_type_type, t_bool auto_delete);
 extern t_bool save_and_process_art(t_openartinfo *artinfo, struct t_article *artptr, t_bool is_mailbox, const char *inpath, int max, t_bool post_process);
 extern void decode_save_mime(t_openartinfo *art, t_bool postproc);
 extern void print_art_seperator_line(FILE *fp, t_bool is_mailbox);
 
 /* screen.c */
+extern char *fmt_message(const char *fmt, va_list ap);
 extern void center_line(int line, t_bool inverse, const char *str);
 extern void clear_message(void);
 extern void draw_arrow_mark(int line);
@@ -569,15 +574,16 @@ extern void stow_cursor(void);
 extern void wait_message(unsigned int sdelay, const char *fmt, ...);
 
 /* search.c */
+extern enum option_enum search_config(t_bool forward, t_bool repeat, enum option_enum current, enum option_enum last);
 extern int get_search_vectors(int *start, int *end);
-extern int search(int key, int current_art, t_bool forward, t_bool repeat);
+extern int search(t_function func, int current_art, t_bool repeat);
 extern int search_active(t_bool forward, t_bool repeat);
 extern int search_article(t_bool forward, t_bool repeat, int start_line, int lines, t_lineinfo *line, int reveal_ctrl_l_lines, FILE *fp);
-extern int search_config(t_bool forward, t_bool repeat, int current, int last);
 extern int search_body(struct t_group *group, int current_art, t_bool repeat);
+extern void reset_srch_offsets(void);
 
 /* select.c */
-extern int add_my_group(const char *group, t_bool add);
+extern int add_my_group(const char *group, t_bool add, t_bool ignore_case);
 extern int choose_new_group(void);
 extern int skip_newgroups(void);
 extern void selection_page(int start_groupnum, int num_cmd_line_groups);
@@ -588,7 +594,7 @@ extern void toggle_my_groups(const char *group);
 extern void msg_write_signature(FILE *fp, t_bool flag, struct t_group *thisgroup);
 
 /* signal.c */
-extern RETSIGTYPE(*sigdisp (int sig, RETSIGTYPE (*func)(SIG_ARGS))) (SIG_ARGS);
+extern RETSIGTYPE(*sigdisp (int signum, RETSIGTYPE (*func)(SIG_ARGS))) (SIG_ARGS);
 extern t_bool set_win_size(int *num_lines, int *num_cols);
 extern void allow_resize(t_bool allow);
 extern void handle_resize(t_bool repaint);
@@ -604,15 +610,19 @@ extern char *eat_tab(char *s);
 extern char *fmt_string(const char *fmt, ...);
 extern char *my_strdup(const char *str);
 extern char *str_trim(char *string);
+extern char *strunc(const char *message, int len);
 extern char *tin_ltoa(long value, int digits);
 extern char *tin_strtok(char *str, const char *delim);
-#if !defined(HAVE_STRCASESTR) || defined(DECL_STRCASESTR)
-	extern const char *strcasestr(const char *haystack, const char *needle);
-#endif /* !HAVE_STRCASESTR || DECL_STRCASESTR */
 extern int sh_format(char *dst, size_t len, const char *fmt, ...);
 extern size_t mystrcat(char **t, const char *s);
 extern void my_strncpy(char *p, const char *q, size_t n);
 extern void str_lwr(char *str);
+#if !defined(HAVE_STRCASESTR) || defined(DECL_STRCASESTR)
+	extern const char *strcasestr(const char *haystack, const char *needle);
+#endif /* !HAVE_STRCASESTR || DECL_STRCASESTR */
+#if !defined(HAVE_STRSEP) || defined(DECL_SEP)
+	extern char *strsep(char **stringp, const char *delim);
+#endif /* !HAVE_STRSEP || DECL_STRSEP */
 #ifndef HAVE_STRPBRK
 	extern char *strpbrk(char *str1, char *str2);
 #endif /* !HAVE_STRPBRK */
@@ -646,10 +656,8 @@ extern void str_lwr(char *str);
 	extern char *wchar_t2char(const wchar_t *wstr);
 	extern wchar_t *char2wchar_t(const char *str);
 	extern wchar_t *wcspart(const wchar_t *wstr, int columns, t_bool pad);
-	extern wchar_t *wstrunc(const wchar_t *wmessage, wchar_t *wbuf, size_t wbuf_len, int len);
-	extern wchar_t *my_wcsdup(const wchar_t *wstr);
+	extern wchar_t *wstrunc(const wchar_t *wmessage, int len);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-extern char *strunc(const char *message, char *buf, size_t buf_len, int len);
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	extern char *render_bidi(const char *str, t_bool *is_rtl);
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
@@ -696,8 +704,8 @@ extern int thread_page(struct t_group *group, int respnum, int thread_depth, t_p
 extern void fixup_thread(int respnum, t_bool redraw);
 
 /* version.c */
-extern int check_upgrade(char *line, const char *skip, const char *version);
-extern void upgrade_prompt_quit(int reason, const char *file);
+extern enum rc_state check_upgrade(char *line, const char *skip, const char *version);
+extern void upgrade_prompt_quit(enum rc_state reason, const char *file);
 
 /* wildmat.c */
 extern t_bool wildmat(const char *text, char *p, t_bool icase);
