@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-08-20
+ *  Updated   : 2008-03-26
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2008 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,9 +47,7 @@
 #		include	<autoconf.h>	/* FIXME: normally we use 'config.h' */
 #	endif /* !TIN_AUTOCONF_H */
 #else
-#	ifndef HAVE_CONFDEFS_H
-#		include	"oldconfig.h"
-#	endif /* !HAVE_CONFDEFS_H */
+#	error "configure run missing"
 #endif /* HAVE_CONFIG_H */
 
 
@@ -78,7 +76,7 @@
 
 #define N_(Str) Str
 
-#ifdef ENABLE_NLS
+#if defined(ENABLE_NLS) && !defined(__BUILD__)
 #	include <libintl.h>
 #	define _(Text)	gettext(Text)
 #else
@@ -87,7 +85,7 @@
 #	undef textdomain
 #	define textdomain(Domain) /* empty */
 #	define _(Text) Text
-#endif /* ENABLE_NLS */
+#endif /* ENABLE_NLS && !__BUILD__ */
 
 #ifndef LOCALEDIR
 #	define LOCALEDIR "/usr/share/locale"
@@ -104,62 +102,6 @@ enum icontext { cNone, cGetline, cPromptSLK };
 enum resizer { cNo, cYes, cRedraw };
 enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
-#ifdef VMS
-#	ifdef __DECC
-#		include <unixio.h>
-#	else
-#		ifndef __VMS_VER	/* assume old types.h */
-			typedef unsigned short mode_t;
-#			undef HAVE_STRFTIME
-#		endif /* !__VMS_VER */
-#		include <stdio.h>
-#	endif /* __DECC */
-#	ifdef SOCKETSHR_TCP
-#		include <socketshr.h>
-#		include <unistd.h>
-#		ifndef SOCKETSHR_HAVE_DUP
-#			define dup
-#		endif /* !SOCKETSHR_HAVE_DUP */
-#		ifndef SOCKETSHR_HAVE_FERROR
-#			define ferror(a) (0)
-#		endif /* !SOCKETSHR_HAVE_FERROR */
-#	endif /* SOCKETSHR_TCP */
-#	include <curses.h>
-#	include <stat.h>
-#	undef HAVE_SELECT
-#	define XHDR_XREF	/* enable crosspost support */
-#	define CASE_PROBLEM
-#	define HAVE_ERRNO_H
-#	define NNTP_ONLY
-#	define NNTP_INEWS
-#	define DONT_HAVE_PIPING
-#	define NO_SHELL_ESCAPE
-#	define USE_CLEARSCREEN
-#	ifndef MM_CHARSET
-#		define MM_CHARSET "ISO-8859-1"
-#	endif /* !MM_CHARSET */
-	/*
-	 * Apparently this means fileops=create if not already there - no idea
-	 * why this should be needed. Standard fopen() implies this in arg 2
-	 */
-	extern char *get_uaf_fullname();
-#	ifdef MULTINET
-#		include "MULTINET_ROOT:[MULTINET.INCLUDE.SYS]TYPES.H"
-#		include "MULTINET_ROOT:[MULTINET.INCLUDE.SYS]TIME.H"
-#	else
-#		include "select.h"
-#		ifdef VAXC
-#			include "vmstimval.h"
-#		endif /* VAXC */
-#		include <types.h>
-#	endif /* !MULTINET */
-#	define FOPEN_OPTS		, "fop=cif"
-#	define JOINPATH		joindir
-#else
-#	define FOPEN_OPTS
-#	define JOINPATH		joinpath
-#endif /* VMS */
-
 #include <stdio.h>
 #ifdef HAVE_ERRNO_H
 #	include	<errno.h>
@@ -204,9 +146,9 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	include <sys/times.h>
 #endif /* HAVE_SYS_TIMES_H */
 
-#ifdef HAVE_LIBC_H
+#if defined(HAVE_LIBC_H) && defined(__NeXT__)
 #	include <libc.h>
-#endif /* HAVE_LIBC_H */
+#endif /* HAVE_LIBC_H && __NeXT__ */
 
 #ifdef HAVE_UNISTD_H
 #	include <unistd.h>
@@ -346,32 +288,21 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 /*
  * Directory handling code
  */
-#ifdef HAVE_CONFIG_H
-#	ifdef HAVE_DIRENT_H
-#		include <dirent.h>
-#		define DIR_BUF	struct dirent
-#	else
-#		ifdef HAVE_SYS_DIR_H
-#			include <sys/dir.h>
-#		endif /* HAVE_SYS_DIR_H */
-#		ifdef HAVE_SYS_NDIR_H
-#			include <sys/ndir.h>
-#		endif /* HAVE_SYS_NDIR_H */
-#		define DIR_BUF	struct direct
-#	endif /* HAVE_DIRENT_H */
+#ifdef HAVE_DIRENT_H
+#	include <dirent.h>
 #else
-#	ifdef M_XENIX
+#	ifdef HAVE_SYS_NDIR_H
 #		include <sys/ndir.h>
-#		define DIR_BUF	struct direct
-#	endif /* M_XENIX */
-#	ifdef VMS
-#		include "ndir.h"
-#		define DIR_BUF	struct direct
-#	endif /* VMS */
-#endif /* !HAVE_CONFIG_H */
+#	endif /* HAVE_SYS_NDIR_H */
+#	ifdef HAVE_SYS_DIR_H
+#		include <sys/dir.h>
+#	endif /* HAVE_SYS_DIR_H */
+#	if HAVE_NDIR_H
+#		include <ndir.h>
+#	endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
 
 #ifndef DIR_BUF
-#	include <dirent.h>
 #	define DIR_BUF	struct dirent
 #endif /* !DIR_BUF */
 
@@ -421,6 +352,12 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	endif /* HAVE_PGP && HAVE_GPG */
 #endif /* HAVE_PGP || HAVE_PGPK || HAVE_GPG */
 
+/*
+ * slrnface requires some things
+ */
+#if defined(HAVE_SLRNFACE) && defined(HAVE_MKFIFO) && defined(HAVE_FORK) && defined(HAVE_EXECLP) && defined(HAVE_WAITPID) && !defined(DONT_HAVE_PIPING) && !defined(X_DISPLAY_MISSING)
+#	define XFACE_ABLE
+#endif /* HAVE_SLRNFACE && HAVE_MKFIFO && HAVE_FORK && HAVE_EXECLP && HAVE_WAITPID && !DONT_HAVE_PIPING && !X_DISPLAY_MISSING */
 
 /*
  * Setup support for reading from NNTP
@@ -454,39 +391,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  */
 #define VT_ESCAPE_TIMEOUT	1
 
-/*
- * Specify News spool & control directories if not running NNTP_ONLY
- * (on machines who can run configure this is not needed)
- */
-#ifndef HAVE_CONFIG_H
-#	ifndef NNTP_ONLY
-#		ifndef SPOOLDIR
-#			ifdef VMS
-#				define SPOOLDIR	"NEWSSPOOL:[000000]"
-#			else
-#				define SPOOLDIR	"/var/spool/news"
-#			endif /* VMS */
-#		endif /* !SPOOLDIR */
-#		ifndef NEWSLIBDIR
-#			ifdef VMS
-#				define NEWSLIBDIR	"NEWSLIB:[000000]"
-#			else
-#				define NEWSLIBDIR	"/usr/lib/news"
-#			endif /* VMS */
-#		endif /* !NEWSLIBDIR */
-#		ifndef NOVROOTDIR
-#			define NOVROOTDIR	SPOOLDIR
-#		endif /* !NOVROOTDIR */
-#		ifndef INEWSDIR
-#			define INEWSDIR	NEWSLIBDIR
-#		endif /* INEWSDIR */
-#	else
-#		undef	SPOOLDIR
-#		undef	NEWSLIBDIR
-#		undef	NOVROOTDIR
-#	endif /* !NNTP_ONLY */
-#endif /* !HAVE_CONFIG_H */
-
 /*
  * Determine machine configuration for external programs & directories
  */
@@ -534,15 +438,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	ifdef linux
 #		define DEFAULT_PRINTER	"/usr/bin/lpr"
 #	endif /* linux */
-#	ifdef VMS
-#		define DEFAULT_EDITOR	"EDIT/TPU"
-#		define DEFAULT_MAILBOX	"SYS$LOGIN:"
-#		define DEFAULT_MAILER	"MAIL"
-#		define MAILER_FORMAT	"MAIL /SUBJECT=\"%S\" %F MX%%\"%T\""
-#		define DEFAULT_POSTER	"inews %s."
-#		define DEFAULT_PRINTER	"PRINT/DELETE"
-#		define DEFAULT_UNSHAR	"unshar %s."
-#	endif /* VMS */
 #	ifdef QNX42
 #		ifndef DEFAULT_EDITOR
 #			define DEFAULT_EDITOR	"/bin/vedit"
@@ -592,7 +487,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	define PATH_ISPELL	"ispell"
 #endif /* !PATH_ISPELL */
 
-#ifndef PATH_METAMAIL	/* unly unset if !HAVE_METAMAIL */
+#ifndef PATH_METAMAIL	/* only unset if !HAVE_METAMAIL */
 #	define PATH_METAMAIL	"metamail"
 #endif /* !PATH_METAMAIL */
 #define METAMAIL_CMD		PATH_METAMAIL" -e -p -m \"tin\""
@@ -641,19 +536,14 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	define DEFAULT_SAVE_NUM	30
 #endif /* SMALL_MEMORY_MACHINE */
 #define DEFAULT_ACTIVE_NUM	1800
+#define DEFAULT_LOCAL_ATTRIBUTES_NUM	8
 #define DEFAULT_NEWNEWS_NUM	5
+#define DEFAULT_MAPKEYS_NUM 100	/* ~remappable keys per level (avoid massiv reallocs) */
 
-#ifdef VMS
-#	define RCDIR	"TIN"
-#	define INDEX_MAILDIR	"MAILIDX"
-#	define INDEX_NEWSDIR	"INDEX"
-#	define INDEX_SAVEDIR	"SAVE"
-#else
-#	define RCDIR	".tin"
-#	define INDEX_MAILDIR	".mail"
-#	define INDEX_NEWSDIR	".news"
-#	define INDEX_SAVEDIR	".save"
-#endif /* VMS */
+#define RCDIR	".tin"
+#define INDEX_MAILDIR	".mail"
+#define INDEX_NEWSDIR	".news"
+#define INDEX_SAVEDIR	".save"
 
 #define ACTIVE_FILE	"active"
 #define ACTIVE_MAIL_FILE	"active.mail"
@@ -664,7 +554,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define SERVERCONFIG_FILE	"serverrc"
 #define DEFAULT_MAILDIR	"Mail"
 #define DEFAULT_SAVEDIR	"News"
-#define DEFAULT_URL_HANDLER "url_handler.sh"
+#define DEFAULT_URL_HANDLER "url_handler.pl"
 /* Prefixes saved attachments with no set filename */
 #define SAVEFILE_PREFIX		"unknown"
 
@@ -701,11 +591,11 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 /* case sensitive && ^-anchored */
 #define DEFAULT_STRIP_RE_REGEX	"(?:R[eE](?:\\^\\d+|\\[\\d\\])?|A[wW]|Odp|Sv):\\s"
 /* case sensitive */
-#define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):.*\\)\\s*$"
+#define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l\\xb3]o):.*\\)\\s*$"
 #define DEFAULT_U8_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l\\x{0142}]o):.*\\)\\s*$"
 /*
  * overkill regexp for balanced '()':
- * #define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):(?:(?:[^)(])*(?:\\([^)(]*\\))*)+\\)\\s*$"
+ * #define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l\xb3]o):(?:(?:[^)(])*(?:\\([^)(]*\\))*)+\\)\\s*$"
  */
 
 /* case sensitive & ^-anchored */
@@ -716,10 +606,13 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 /* case sensitive & ^-anchored */
 #define SHAR_REGEX	"\\#(?:!\\s?(?:/usr)?/bin/sh|\\s?(?i)this\\sis\\sa\\sshell\\sarchive)"
 
+/* slrn verbatim marks, case sensitive & ^-anchored */
+#define DEFAULT_VERBATIM_BEGIN_REGEX	"#v+"
+#define DEFAULT_VERBATIM_END_REGEX	"#v-"
 
 /*
  * URL related regexs:
- * add TELNET & WAIS (RFC 1738), IMAP (RFC 2192), NFS (RFC 2224)
+ * add TELNET (RFC 4248), WAIS (RFC 4156), IMAP (RFC 2192), NFS (RFC 2224)
  *     LDAP (RFC 2255), POP (RFC 2384)
  * add IPv6 (RFC 2732, RFC 2373) support
  */
@@ -751,7 +644,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  * case insensitive
  */
 #if 1 /* complex */
-#  	define NEWS_REGEX "\\b(?:s?news|nntp):(?:(?:(?://(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?|xn--[^\\W_](?:-(?!-)|[^\\W_]){1,57}[^\\W_])\\.)+[a-z]{2,6}\\.?|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?(?(?=[/])[^()\\^\\[\\]{}\\|\\x00-\\x1f\\x7f\\s\"<>'\\\\:,;]+|$))|[^\\^\\[\\]{}\\|\\x00-\\x1f\\x7f\\s<>\"():,;\\\\'/]+)\\b"
+#	define NEWS_REGEX "\\b(?:s?news|nntp):(?:(?:(?://(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?|xn--[^\\W_](?:-(?!-)|[^\\W_]){1,57}[^\\W_])\\.)+[a-z]{2,6}\\.?|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?(?(?=[/])[^()\\^\\[\\]{}\\|\\x00-\\x1f\\x7f\\s\"<>'\\\\:,;]+|$))|[^\\^\\[\\]{}\\|\\x00-\\x1f\\x7f\\s<>\"():,;\\\\'/]+)\\b"
 #else
 #	if 1 /* less complex */
 #		define NEWS_REGEX	"\\b(?:s?news|nntp):[^\\s\\*@>]+(?(?=[@])[^\\s\\*@<>()\",/]+|[^\\s\\*<>()\":,/]+)\\b"
@@ -769,7 +662,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 #define FILTER_FILE	"filter"
 /* editor offset for filter-file; TODO: doesn't match for german filter-file */
-#define FILTER_FILE_OFFSET	27
+#define FILTER_FILE_OFFSET	26
 #define INPUT_HISTORY_FILE	".inputhistory"
 #ifdef HAVE_MH_MAIL_HANDLING
 #	define MAILGROUPS_FILE	"mailgroups"
@@ -787,14 +680,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define POSTPONED_FILE	"postponed.articles"
 #define SUBSCRIPTIONS_FILE	"subscriptions"
 #define NEWSGROUPS_FILE	"newsgroups"
-
-
-#ifdef USE_INN_NNTPLIB
-#	include <libinn.h> /* TODO: add configure chek for it */
-#	define _CONF_FROMHOST	"fromhost"
-#	define _CONF_ORGANIZATION	"organization"
-#	define _CONF_SERVER	"server"
-#endif /* USE_INN_NNTPLIB */
+#define KEYMAP_FILE	"keymap"
 
 #define SIGDASHES "-- \n"
 
@@ -874,11 +760,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  * HEADER_LEN  = max. size of a news/mail header-line
  * NEWSRC_LINE =
  */
-#if defined(VMS)
-#	define PATH_LEN	256
-#	define NAME_LEN	14
-#	define LEN	512
-#endif /* VMS */
 
 #ifdef M_UNIX
 #	ifdef PATH_MAX
@@ -981,6 +862,13 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define TINRC_CONFIRM_TO_QUIT	(tinrc.confirm_choice == 3 || tinrc.confirm_choice == 4 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 #define TINRC_CONFIRM_SELECT	(tinrc.confirm_choice == 2 || tinrc.confirm_choice == 5 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 
+/*
+ * defines for tinrc.goto_next_unread
+ */
+#define NUM_GOTO_NEXT_UNREAD	4
+#define GOTO_NEXT_UNREAD_PGDN	1
+#define GOTO_NEXT_UNREAD_TAB	2
+
 /*
  * MIME Encodings
  */
@@ -1023,9 +911,9 @@ enum {
 /* Line number (starting at 0) of 1st non-header data on the screen */
 /* ie, size of header */
 #define INDEX_TOP	2
-#define INDEX2LNUM(i)	(INDEX_TOP + (i) % NOTESLINES)
+#define INDEX2LNUM(i)	(INDEX_TOP + (i) - currmenu->first)
 #ifndef USE_CURSES
-#	define INDEX2SNUM(i)	((i) % NOTESLINES)
+#	define INDEX2SNUM(i)	((i) - currmenu->first)
 #endif /* !USE_CURSES */
 
 #define GROUP_MATCH(s1, pat, case)		(wildmat(s1, pat, case))
@@ -1037,7 +925,7 @@ enum {
 #define IGNORE_ART_THREAD(i)	(arts[i].thread != ART_UNTHREADED || (tinrc.kill_level == KILL_NOTHREAD && arts[i].killed))
 
 /*
- * Is this part text/plain ?
+ * Is this part text/plain?
  */
 #define IS_PLAINTEXT(x) \
 			(x->type == TYPE_TEXT && strcasecmp("plain", x->subtype) == 0)
@@ -1120,12 +1008,6 @@ enum {
 #define FEED_AUTOSAVE	5
 #define FEED_REPOST		6
 
-#if 0
-#	define DEBUG_IO(x)	fprintf x
-#else
-#	define DEBUG_IO(x)	/* nothing */
-#endif /* 0 */
-
 
 /*
  * Threading strategies available
@@ -1135,8 +1017,11 @@ enum {
 #define THREAD_REFS		2
 #define THREAD_BOTH		3
 #define THREAD_MULTI		4
+#define THREAD_PERC		5
 
-#define THREAD_MAX		THREAD_MULTI
+#define THREAD_MAX		THREAD_PERC
+
+#define THREAD_PERC_DEFAULT	75
 
 /*
  * Values for show_author
@@ -1201,12 +1086,8 @@ enum {
 #define SORT_THREADS_BY_NOTHING			0
 #define SORT_THREADS_BY_SCORE_DESCEND	1
 #define SORT_THREADS_BY_SCORE_ASCEND	2
-
-/*
- * Search keys
- */
-#define SEARCH_SUBJ		1
-#define SEARCH_AUTH		2
+#define SORT_THREADS_BY_LAST_POSTING_DATE_DESCEND	3
+#define SORT_THREADS_BY_LAST_POSTING_DATE_ASCEND	4
 
 /*
  * Different values of strip_bogus - the ways to handle bogus groups
@@ -1293,12 +1174,15 @@ enum {
 #	define EXIT_FAILURE	1	/* Failing exit status */
 #endif /* !EXIT_FAILURE */
 
-#define NNTP_ERROR_EXIT	2
+#define NEWS_AVAIL_EXIT 2
+#define NNTP_ERROR_EXIT	3
 
 /*
  * Assertion verifier
  */
-#undef assert
+#ifdef assert
+#	undef assert
+#endif /* assert */
 #ifdef CPP_DOES_EXPAND
 #	define assert(p)	if(! (p)) asfail(__FILE__, __LINE__, #p); else (void)0;
 #else
@@ -1307,15 +1191,11 @@ enum {
 
 #define ESC	27
 
-
 /*
- * return codes for change_config_file ()
+ * filter entrys expire after DEFAULT_FILTER_DAYS
  */
-#define NO_FILTERING		0
-#define FILTERING		1
 #define DEFAULT_FILTER_DAYS		28
 
-
 /*
  * art.thread
  */
@@ -1323,7 +1203,7 @@ enum {
 #define ART_EXPIRED		-2
 
 /*
- * Where does this belong ?? It is overloaded
+ * Where does this belong?? It is overloaded
  */
 #define ART_NORMAL		-1
 
@@ -1361,9 +1241,6 @@ enum {
 /*
  * filter_type used in struct t_filter
  */
-#define FILTER_KILL		0
-#define FILTER_SELECT		1
-
 #define SCORE_MAX		10000
 
 #define FILTER_SUBJ_CASE_SENSITIVE		0
@@ -1421,7 +1298,6 @@ enum {
 /*
  * used in checking article header before posting
  */
-#define NGLIMIT		20	/* Max. num. of crossposted groups before warning */
 #define MAX_COL		78	/* Max. line length before issuing a warning */
 #define MAX_SIG_LINES	4	/* Max. num. of signature lines before warning */
 
@@ -1463,7 +1339,7 @@ struct t_msgid {
  * struct t_article - article header
  *
  * article.thread:
- *  the next article in thread
+ * the next article in thread
  *	-1  (ART_UNTHREADED) article exists but is not (yet) threaded
  *	-2  (ART_EXPIRED) article has expired (wasn't found in search of spool
  *	    directory for the group)
@@ -1505,52 +1381,55 @@ struct t_article {
  * struct t_attribute - configurable attributes on a per group basis
  */
 struct t_attribute {
+	char *scope;				/* scope for these group attributes */
 	char *maildir;				/* mail dir if other than ~/Mail */
 	char *savedir;				/* save dir if other than ~/News */
-	char *savefile;			/* save articles to specified file */
+	char *savefile;				/* save articles to specified file */
 	char *sigfile;				/* sig file if other than ~/.Sig */
 	char *organization;			/* organization name */
 	char *followup_to;			/* where posts should be redirected */
-	char *quick_kill_scope;			/* quick filter kill scope */
-	char *quick_select_scope;		/* quick filter select scope */
+	char *quick_kill_scope;		/* quick filter kill scope */
+	char *quick_select_scope;	/* quick filter select scope */
 	char *mailing_list;			/* mail list email address */
 	char *x_headers;			/* extra headers for message header */
-	char *x_body;				/* bolierplate text for message body */
-	char *from;				/* from line */
-	char *news_quote_format;		/* another way to begin a posting format */
+	char *x_body;				/* boilerplate text for message body */
+	char *from;					/* from line */
+	char *news_quote_format;	/* another way to begin a posting format */
 	char *quote_chars;			/* string to precede quoted text on each line */
 	char *mime_types_to_save;	/* MIME content major/minors we want to save */
 #ifdef HAVE_ISPELL
-	char *ispell;			/* path to ispell and options */
+	char *ispell;				/* path to ispell and options */
 #endif /* HAVE_ISPELL */
 	unsigned global:1;			/* global/group specific */
-	unsigned quick_kill_header:3;		/* quick filter kill header */
-	unsigned quick_kill_expire:1;		/* quick filter kill limited/unlimited time */
+	unsigned quick_kill_header:3;	/* quick filter kill header */
+	unsigned quick_kill_expire:1;	/* quick filter kill limited/unlimited time */
 	unsigned quick_kill_case:1;		/* quick filter kill case sensitive? */
-	unsigned quick_select_header:3;		/* quick filter select header */
-	unsigned quick_select_expire:1;		/* quick filter select limited/unlimited time */
-	unsigned quick_select_case:1;		/* quick filter select case sensitive? */
+	unsigned quick_select_header:3;	/* quick filter select header */
+	unsigned quick_select_expire:1;	/* quick filter select limited/unlimited time */
+	unsigned quick_select_case:1;	/* quick filter select case sensitive? */
 	unsigned auto_select:1;			/* 0=show all unread, 1='X' just hot arts */
 	unsigned auto_save:1;			/* 0=none, 1=save */
 	unsigned batch_save:1;			/* 0=none, 1=save -S/mail -M */
-	unsigned delete_tmp_files:1;		/* 0=leave, 1=delete */
-	unsigned show_only_unread:1;		/* 0=all, 1=only unread */
-	unsigned thread_arts:3;			/* 0=unthread, 1=subject, 2=refs, 3=both, 4=multipart*/
+	unsigned delete_tmp_files:1;	/* 0=leave, 1=delete */
+	unsigned show_only_unread:1;	/* 0=all, 1=only unread */
+	unsigned thread_arts:3;			/* 0=unthread, 1=subject, 2=refs, 3=both, 4=multipart, 5=percentage */
+	unsigned thread_perc:7;			/* percentage threading threshold */
 	unsigned show_author:2;			/* 0=none, 1=name, 2=addr, 3=both */
 	unsigned show_info:2;			/* 0=none, 1=lines, 2=score, 3=both */
 	unsigned sort_art_type:4;		/* 0=none, 1=subj descend, 2=subj ascend,
 						   3=from descend, 4=from ascend,
 						   5=date descend, 6=date ascend,
 						   7=score descend, 8=score ascend */
-	unsigned sort_threads_type:2;	/* 0=none, 1=score descend, 2=score ascend */
-	unsigned int post_proc_type:2;		/* 0=none, 1=shar, 2=uudecode */
-	unsigned int x_comment_to:1;		/* insert X-Comment-To: in Followup */
-	unsigned int tex2iso_conv:1;		/* Convert TeX2ISO */
-	unsigned int mime_forward:1;		/* forward articles as attachment or inline */
-	char *fcc;							/* Fcc folder for mail */
+	unsigned sort_threads_type:3;	/* 0=none, 1=score descend, 2=score ascend,
+						   3=last posting date descend, 4=last posting date ascend */
+	unsigned int post_proc_type:2;	/* 0=none, 1=shar, 2=uudecode */
+	unsigned int x_comment_to:1;	/* insert X-Comment-To: in Followup */
+	unsigned int tex2iso_conv:1;	/* Convert TeX2ISO */
+	unsigned int mime_forward:1;	/* forward articles as attachment or inline */
+	char *fcc;						/* Fcc folder for mail */
 #ifdef CHARSET_CONVERSION
-	int mm_network_charset;				/* network charset */
-	char *undeclared_charset;			/* charset of articles without MIME charset declaration */
+	int mm_network_charset;			/* network charset */
+	char *undeclared_charset;		/* charset of articles without MIME charset declaration */
 #endif /* CHARSET_CONVERSION */
 };
 
@@ -1561,7 +1440,7 @@ struct t_attribute {
  * struct t_newsrc - newsrc related info.
  */
 struct t_newsrc {
-	t_bool present:1;		/* update newsrc ? */
+	t_bool present:1;		/* update newsrc? */
 	long num_unread;		/* unread articles in group */
 	long xmax;			/* newsrc max */
 	long xmin;			/* newsrc min */
@@ -1576,7 +1455,7 @@ struct t_newsrc {
  * struct t_group - newsgroup info from active file
  */
 struct t_group {
-	char *name;			/* newsgroup / mailbox name */
+	char *name;			/* newsgroup/mailbox name */
 	char *aliasedto;		/* =new.group in active file, NULL if not */
 	char *description;	/* text from NEWSLIBDIR/newsgroups file */
 	char *spooldir;		/* groups spool directory */
@@ -1584,7 +1463,7 @@ struct t_group {
 	long count;			/* article number count */
 	long xmax;			/* max. article number */
 	long xmin;			/* min. article number */
-	unsigned int type:4;		/* grouptype - newsgroup / mailbox / savebox */
+	unsigned int type:4;		/* grouptype - newsgroup/mailbox/savebox */
 	t_bool inrange:1;		/* TRUE if group selected via # range command */
 	t_bool read_during_session:1;	/* TRUE if group entered during session */
 	t_bool art_was_posted:1;	/* TRUE if art was posted to group */
@@ -1648,14 +1527,14 @@ struct t_filter {
 	char *from;			/* From: line */
 	char *msgid;			/* Message-ID: line */
 	char lines_cmp;			/* Lines compare <> */
-	int  lines_num;			/* Lines: line */
+	int lines_num;			/* Lines: line */
 	char gnksa_cmp;			/* GNKSA compare <> */
-	int  gnksa_num;			/* GNKSA code */
-	int  score;			/* score to give if rule matches */
+	int gnksa_num;			/* GNKSA code */
+	int score;			/* score to give if rule matches */
 	char *xref;			/* groups in xref line */
 	time_t time;			/* expire time in seconds */
 	struct t_filter *next;		/* next rule valid in group */
-	unsigned int inscope:4;		/* if group matches scope ie. 'comp.os.*' */
+	unsigned int inscope:4;		/* if group matches scope e.g. 'comp.os.*' */
 	unsigned int icase:2;		/* Case sensitive filtering */
 	unsigned int fullref:4;		/* use full references or last entry only */
 };
@@ -1756,7 +1635,7 @@ struct t_option {
 	int *variable;		/* ptr to variable to change */
 	constext **opt_list;	/* ptr to list entries if OPT_LIST */
 	int opt_count;		/* no. of list entries if OPT_LIST */
-	struct opttxt *txt;	/* ptr to information / help on option */
+	struct opttxt *txt;	/* ptr to information/help on option */
 };
 
 /*
@@ -1778,9 +1657,9 @@ typedef struct {
 	int curr;					/* Current cursor pos (cur_groupnum, index_point, thread_index_point) */
 	int max;					/* Max # on current menu (group_top, top_base, top_thread) */
 	int first;					/* First # on current menu */
-	int last;					/* Last # on current menu (first,last_*_on_screen) */
 	void (*redraw) (void);		/* Redraw function */
 	void (*draw_arrow) (void);	/* Arrow draw */
+	void (*draw_item) (int item);	/* draw the specified item */
 } t_menu;
 
 
@@ -1806,7 +1685,7 @@ typedef struct _TIMEINFO {
 /*
  * mailcap fields
  * the x-token field is missing, we would need something like
- * struct t_xtoken { char *xtoken; t_xtoken *next; } for that...
+ * struct t_xtoken { char *xtoken; t_xtoken *next; } for that ...
  */
 typedef struct {
 	char *type;		/* content-type, mandatory */
@@ -1853,11 +1732,7 @@ typedef int (*t_compfunc)(t_comptype, t_comptype);
 #define _CDECL
 
 /* Seperator between dir part of path & the filename */
-#ifdef VMS
-#	define DIRSEP	']'
-#else
-#	define DIRSEP	'/'
-#endif /* VMS */
+#define DIRSEP	'/'
 
 /*
  * mouse buttons for use in xterm
@@ -1868,20 +1743,6 @@ typedef int (*t_compfunc)(t_comptype, t_comptype);
 
 #define TIN_EDITOR_FMT_OFF		"%E %F"
 
-#ifdef VMS
-#	define REDIRECT_OUTPUT	""
-#	define REDIRECT_PGP_OUTPUT	""
-#	define ENV_VAR_MAILER		"TIN_MAILER"
-/*#	define ENV_VAR_SHELL		"SHELL"*/
-#	define ENV_VAR_POSTER		"TIN_POST"
-#	define TIN_EDITOR_FMT_ON	"%E %F"
-#	define TMPDIR "SYS$SCRATCH:"
-#	ifdef HAVE_KEY_PREFIX
-#		define KEY_PREFIX	0x9b
-#	endif /* HAVE_KEY_PREFIX */
-extern void joindir (char *result, const char *dir, const char *file);
-#endif /* VMS */
-
 #ifdef M_UNIX
 #	define REDIRECT_OUTPUT		"> /dev/null 2>&1"
 #	define REDIRECT_PGP_OUTPUT		"> /dev/null"
@@ -1920,9 +1781,9 @@ extern void joindir (char *result, const char *dir, const char *file);
 #endif /* !TMPDIR */
 
 #if !defined(S_ISDIR)
-#	if defined(M_UNIX) || defined(VMS)
+#	if defined(M_UNIX)
 #		define S_ISDIR(m)	(((m) & S_IFMT) == S_IFDIR)
-#	endif /* M_UNIX || VMS */
+#	endif /* M_UNIX */
 #endif /* !S_ISDIR */
 
 #if !defined(S_ISREG)
@@ -1966,11 +1827,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #endif /* !S_ISVTX */
 
 #ifdef DONT_HAVE_PIPING
-#	ifdef VMS
-#		define TIN_PRINTFILE "Sys$Scratch:TIN_PRINT%d.TMP"
-#	else
-#		define TIN_PRINTFILE "tinprint%d.tmp"
-#	endif /* VMS */
+#	define TIN_PRINTFILE "tinprint%d.tmp"
 #endif /* DONT_HAVE_PIPING */
 
 /*
@@ -1990,7 +1847,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #endif /* !F_OK */
 
 /* Various function redefinitions */
-#ifdef USE_DBMALLOC
+#if defined(USE_DBMALLOC) || defined(USE_DMALLOC)
 #	define my_malloc(size)	malloc(size)
 #	define my_calloc(nmemb, size)	calloc((nmemb), (size))
 #	define my_realloc(ptr, size)	realloc((ptr), (size))
@@ -1998,7 +1855,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #	define my_malloc(size)	my_malloc1(__FILE__, __LINE__, (size))
 #	define my_calloc(nmemb, size)	my_calloc1(__FILE__, __LINE__, (nmemb), (size))
 #	define my_realloc(ptr, size)	my_realloc1(__FILE__, __LINE__, (ptr), (size))
-#endif /* USE_DBMALLOC */
+#endif /* USE_DBMALLOC || USE_DMALLOC */
 
 #define ARRAY_SIZE(array)	((int) (sizeof(array) / sizeof(array[0])))
 
@@ -2007,8 +1864,8 @@ extern void joindir (char *result, const char *dir, const char *file);
 
 #define BlankIfNull(p)	((p) ? (p) : "")
 
-#define my_group_find(x)	add_my_group(x, FALSE)
-#define my_group_add(x)		add_my_group(x, TRUE)
+#define my_group_find(x)	add_my_group(x, FALSE, FALSE)
+#define my_group_add(x, y)		add_my_group(x, TRUE, y)
 #define for_each_group(x)	for (x = 0; x < num_active; x++)
 #define for_each_art(x)		for (x = 0; x < top_art; x++)
 #define for_each_art_in_thread(x, y)	for (x = (int) base[y]; x >= 0; x = arts[x].thread)
@@ -2094,6 +1951,9 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #ifdef USE_DMALLOC
 #	include <dmalloc.h>
 #	define DMALLOC_FUNC_CHECK
+#	ifdef HAVE_STRDUP
+#		define my_strdup(s) strdup((s))
+#	endif /* HAVE_STRDUP */
 #endif /* USE_DMALLOC */
 
 #ifdef DOALLOC
@@ -2116,18 +1976,11 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 	extern void	no_leaks(void);
 #endif /* DOALLOC */
 
-#ifdef __DECC		/* VMS */
-#	ifndef ferror
-#		define ferror(x)		(0)
-#		define EndWin		EndWind
-#	endif /* !ferror */
-#endif /* __DECC */
-
 #ifndef my_tmpfile_only
 /*
- * shortcut if we arn't interrested in the tmpfiles filename/location
+ * shortcut if we aren't interested in the tmpfiles filename/location
  * argument can't be a pointer and if argument is changed on return
- * we must unlik the tmp-file ourself
+ * we must unlink the tmp-file ourself
  */
 #	define my_tmpfile_only(a)	my_tmpfile(a, sizeof(a) - 1, FALSE, (char *) 0)
 #endif /* !my_tmpfile_only */
@@ -2193,9 +2046,14 @@ extern struct tm *localtime(time_t *);
 
 /* libcanlock */
 #ifdef USE_CANLOCK
-#	include "../libcanlock/canlock.h"
+#	include "../libcanlock/include/canlock.h"
 #endif /* USE_CANLOCK */
 
+/* gsasl */
+#ifdef USE_SASL
+#	include <gsasl.h>
+#endif /* USE_SASL */
+
 /* snprintf(), vsnprintf() */
 #ifndef HAVE_SNPRINTF
 #	define snprintf	plp_snprintf
@@ -2230,14 +2088,22 @@ extern struct tm *localtime(time_t *);
 #endif /* !EOF */
 
 /* various filenames used by tin */
-#ifndef VMS	/* TODO: is the VMS case needed? */
-#	define TIN_ARTICLE_NAME	".article"
-#	define TIN_CANCEL_NAME	".cancel"
-#	define TIN_LETTER_NAME	".letter"
-#else
-#	define TIN_ARTICLE_NAME	"article."
-#	define TIN_CANCEL_NAME	"cancel."
-#	define TIN_LETTER_NAME	"letter."
-#endif /* !VMS */
+#define TIN_ARTICLE_NAME	".article"
+#define TIN_CANCEL_NAME	".cancel"
+#define TIN_LETTER_NAME	".letter"
+
+/* read_news_active_file() / open_newsgroups_fp() */
+#ifndef DISABLE_PIPELINING
+#	define PIPELINE_LIMIT 30
+#endif /* DISABLE_PIPELINING */
+
+#ifndef DEBUG_H
+#	include "debug.h"
+#endif /* !DEBUG_H */
+
+struct t_overview_fmt {
+	char *name;
+	enum f_type type;
+};
 
 #endif /* !TIN_H */
