@@ -3,10 +3,10 @@
  *  Module    : art.c
  *  Author    : I.Lea & R.Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-08-16
+ *  Updated   : 2008-04-27
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2008 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -70,6 +70,8 @@ static int from_comp_asc(t_comptype p1, t_comptype p2);
 static int from_comp_desc(t_comptype p1, t_comptype p2);
 static int global_get_multiparts(int aindex, MultiPartInfo **malloc_and_setme_info);
 static int global_look_for_multipart_info(int aindex, MultiPartInfo *setme, char start, char stop, int *offset);
+static int last_date_comp_base_asc(t_comptype p1, t_comptype p2);
+static int last_date_comp_base_desc(t_comptype p1, t_comptype p2);
 static int lines_comp_asc(t_comptype p1, t_comptype p2);
 static int lines_comp_desc(t_comptype p1, t_comptype p2);
 static int read_art_headers(struct t_group *group, int total, long top);
@@ -84,9 +86,11 @@ static long find_first_unread(struct t_group *group);
 static long setup_hard_base(struct t_group *group);
 static t_bool parse_headers(FILE *fp, struct t_article *h);
 static t_compfunc eval_sort_arts_func(unsigned int sort_art_type);
+static time_t get_last_posting_date(long n);
 static void sort_base(unsigned int sort_threads_type);
-static void thread_by_subject(void);
 static void thread_by_multipart(void);
+static void thread_by_percentage(struct t_group *group);
+static void thread_by_subject(void);
 
 
 /*
@@ -179,7 +183,7 @@ base_comp(
  *   Read the article numbers existing in the group into base[]
  *   If the LISTGROUP failed, issue a GROUP command. Use the results to
  *   create a less accurate version of base[]
- *	 This data will already be sorted
+ *   This data will already be sorted
  *
  * on local spool:
  *   Read the spool dir to populate base[] as above. Sort it.
@@ -194,7 +198,6 @@ static long
 setup_hard_base(
 	struct t_group *group)
 {
-	char buf[NNTP_STRLEN];
 	long art;
 	long total = 0;
 
@@ -205,6 +208,7 @@ setup_hard_base(
 	 */
 	if (read_news_via_nntp && group->type == GROUP_TYPE_NEWS) {
 #ifdef NNTP_ABLE
+		char buf[NNTP_STRLEN];
 		FILE *fp;
 
 #	ifdef BROKEN_LISTGROUP
@@ -226,7 +230,8 @@ setup_hard_base(
 			char *ptr;
 
 #	ifdef DEBUG
-			debug_nntp("setup_hard_base", buf);
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "setup_hard_base(%s)", buf);
 #	endif /* DEBUG */
 
 			while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
@@ -259,8 +264,15 @@ setup_hard_base(
 			if (sscanf(line, "%ld %ld %ld", &count, &start, &last) != 3)
 				return -1;
 
+#	ifdef DEBUG
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "setup_hard_base(%s)", buf);
+#	endif /* DEBUG */
+			/*
+			 * TODO: AFAICS "total" and "count" arn't used in the code below
+			 *       anymore, why do we bother to set them?
+			 */
 			total = count;
-
 			if (last - count > start)
 				count = last - start;
 
@@ -277,15 +289,16 @@ setup_hard_base(
 	} else {
 		DIR *d;
 		DIR_BUF *e;
+		char group_path[PATH_LEN];
 
-		make_base_group_path(group->spooldir, group->name, buf);
+		make_base_group_path(group->spooldir, group->name, group_path, sizeof(group_path));
 
-		if (access(buf, R_OK) != 0) {
+		if (access(group_path, R_OK) != 0) {
 			error_message(_(txt_not_exist));
 			return -1;
 		}
 
-		if ((d = opendir(buf)) != NULL) {
+		if ((d = opendir(group_path)) != NULL) {
 			while ((e = readdir(d)) != NULL) {
 				art = atol(e->d_name);
 				if (art >= 1) {
@@ -324,7 +337,7 @@ index_group(
 	struct t_group *group)
 {
 	int i;
-	int changed = 0;				/* Count of articles whose overview has changed */
+	int changed;				/* Count of articles whose overview has changed */
 	int respnum;
 	int total;
 	long last_read_article;
@@ -344,7 +357,7 @@ index_group(
 	free_art_array();
 	free_msgids();
 
-	BegStopWatch("setup_hard_base");
+	BegStopWatch("setup_hard_base()");
 
 	/*
 	 * Get list of valid article numbers
@@ -355,10 +368,12 @@ index_group(
 	EndStopWatch();
 	PrintStopWatch();
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Before read_overview");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("Before read_overview");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 
 	min = grpmenu.max ? base[0] : group->xmin;
 	max = grpmenu.max ? base[grpmenu.max - 1] : min - 1;
@@ -390,7 +405,7 @@ index_group(
 	 * When reading local spool, this will pull in the system wide overview
 	 * cache (if found) otherwise the private overview cache will be read
 	 */
-	caching_xover = (tinrc.cache_overview_files && xover_cmd && group->type == GROUP_TYPE_NEWS);
+	caching_xover = (tinrc.cache_overview_files && nntp_caps.over_cmd && group->type == GROUP_TYPE_NEWS);
 	if ((changed = read_overview(group, min, max, &last_read_article, caching_xover)) == -1)
 		return FALSE;	/* user aborted indexing */
 
@@ -428,19 +443,23 @@ index_group(
 			return FALSE;		/* user aborted indexing */
 	}
 
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("Before parse_unread_arts()");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("Before parse_unread_arts()");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 	/*
 	 * Do this before calling art_mark(,, ART_READ) if you want
 	 * the unread count to be correct.
 	 */
 	parse_unread_arts(group);
-#ifdef DEBUG_NEWSRC
-	debug_print_comment("After parse_unread_arts()");
-	debug_print_bitmap(group, NULL);
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
+		debug_print_comment("After parse_unread_arts()");
+		debug_print_bitmap(group, NULL);
+	}
+#endif /* DEBUG */
 
 	/*
 	 * Stat all articles to see if any have expired
@@ -448,9 +467,10 @@ index_group(
 	for_each_art(i) {
 		if (arts[i].thread == ART_EXPIRED) {
 			changed++;
-#ifdef DEBUG_NEWSRC
-			debug_print_comment("art.c: index_group() purging...");
-#endif /* DEBUG_NEWSRC */
+#ifdef DEBUG
+			if (debug & DEBUG_NEWSRC)
+				debug_print_comment("art.c: index_group() purging...");
+#endif /* DEBUG */
 			art_mark(group, &arts[i], ART_READ);
 		}
 	}
@@ -473,7 +493,7 @@ index_group(
 	 */
 	filtered = filter_articles(group);
 
-	BegStopWatch("make_thread");
+	BegStopWatch("make_threads()");
 
 	/*
 	 * Thread the group
@@ -589,7 +609,7 @@ read_art_headers(
 		char buf[PATH_LEN];
 
 		get_cwd(dir);
-		make_base_group_path(group->spooldir, group->name, buf);
+		make_base_group_path(group->spooldir, group->name, buf, sizeof(buf));
 		my_chdir(buf);
 	}
 
@@ -635,7 +655,8 @@ read_art_headers(
 			char buf[PATH_LEN];
 
 			snprintf(buf, sizeof(buf), "FAILED parse_headers(%ld)", art);
-			debug_nntp("read_art_headers", buf);
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "read_art_headers() %s", buf);
 #endif /* DEBUG */
 			continue;
 		}
@@ -717,6 +738,77 @@ thread_by_subject(
 #endif /* 0 */
 }
 
+/*
+ * This Threading algorithm threads articles into 'buckets' where each bucket
+ * contains all the articles which match the root article's subject line to
+ * the configured percentage. Eg, if the root article had the subject "asdf"
+ * and the match percentage was configured to be 75% then any article would
+ * match if its subject was no different in more than a single character.
+ */
+static void
+thread_by_percentage(
+	struct t_group *group)
+{
+	int i, j, k;
+	int root_num = 0; /* The index number of the root we are currently working on. */
+	int unmatched; /* This is the number of characters that don't match between the two strings */
+	unsigned int percentage = 100 - group->attribute->thread_perc;
+	size_t slen;
+
+	/* First we need to sort art[] to simplify and speed up the matching. */
+	SortBy(subj_comp_asc);
+
+	/*
+	 * Now we put all the articles which match enough into the thread. If
+	 * an article doesn't match enough we create a new thread and then add
+	 * to that and so on.
+	 */
+	base[0] = 0;
+	arts[0].prev = ART_NORMAL;
+	for_each_art(i) {
+		if (i == 0)
+			continue;
+
+		/* Check each character to see if it matched enough */
+		k = 0;
+		unmatched = 0;
+		for (j = 0; arts[base[root_num]].subject[j] != '\0' && arts[i].subject[k] != '\0'; j++, k++) {
+			if (arts[base[root_num]].subject[j] != arts[i].subject[k])
+				unmatched++;
+		}
+
+		/*
+		 * By getting here we have a number of unmatched characters
+		 * between the two strings. We also have the length of the
+		 * strings available to us easily.
+		 * All we need to do is see if the match is good enough, but
+		 * we count differences in the length of the strings against
+		 * them matching.
+		 */
+		if (!(slen = strlen(arts[base[root_num]].subject)))
+			slen++;
+		unmatched += abs(slen - strlen(arts[i].subject));
+		if ((unmatched * 100) / slen > percentage) {
+			/*
+			 * If there is less greater than percentage% different start a
+			 * new thread.
+			 */
+			base[++root_num] = i;
+			arts[i].prev = ART_NORMAL;
+			continue;
+		} else {
+			/*
+			 * The subject lines match enough to consider them part of a single
+			 * thread, so add the current article to the thread.
+			 */
+			if (arts[base[root_num]].thread < 0)
+				arts[base[root_num]].thread = i;
+			arts[i].prev = i - 1;
+			arts[i - 1].thread = i;
+			continue;
+		}
+	}
+}
 
 /*
  * This was brought over from tags.c, however this version doesn't not
@@ -896,7 +988,6 @@ thread_by_multipart(
 	MultiPartInfo *minfo = NULL;
 
 	for_each_art(i) {
-
 		if (IGNORE_ART_THREAD(i) || arts[i].prev >= 0 || !global_get_multiparts(i, &minfo))
 			continue;
 
@@ -925,6 +1016,7 @@ thread_by_multipart(
  *	THREAD_REFS		Threads are created using the References headers
  *	THREAD_BOTH		Threads created using References and then Subject
  *	THREAD_MULTI	Threads created using Subject to search for Multiparts
+ *	THREAD_PERC		Threads based upon a char for char match of greater than x%
  *
  * .thread and .prev are used to hold the threading information, see tin.h for
  * more information
@@ -948,7 +1040,7 @@ make_threads(
 		info_message((group->attribute->thread_arts == THREAD_NONE ? _(txt_unthreading_arts) : _(txt_threading_arts)));
 
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		error_message("rethread=[%d]  thread_arts=[%d]  attr_thread_arts=[%d]",
 				rethread, tinrc.thread_articles, group->attribute->thread_arts);
 #endif /* DEBUG */
@@ -982,14 +1074,15 @@ make_threads(
 			/* Should never happen if tree is built properly */
 			if (arts[i].refptr == 0) {
 #ifdef DEBUG
-				my_fprintf(stderr, "\nError  : art->refptr is NULL\n");
-				my_fprintf(stderr, "Artnum : %ld\n", arts[i].artnum);
-				my_fprintf(stderr, "Subject: %s\n", arts[i].subject);
-				my_fprintf(stderr, "From   : %s\n", arts[i].from);
-				assert(arts[i].refptr != 0);
-#else
-				continue;
+				if (debug & DEBUG_REFS) {
+					my_fprintf(stderr, "\nError  : art->refptr is NULL\n");
+					my_fprintf(stderr, "Artnum : %ld\n", arts[i].artnum);
+					my_fprintf(stderr, "Subject: %s\n", arts[i].subject);
+					my_fprintf(stderr, "From   : %s\n", arts[i].from);
+					assert(arts[i].refptr != 0);
+				} else
 #endif /* DEBUG */
+					continue;
 			}
 			arts[i].refptr->article = i;
 		}
@@ -1019,6 +1112,10 @@ make_threads(
 			thread_by_multipart();
 			break;
 
+		case THREAD_PERC:
+			thread_by_percentage(group);
+			break;
+
 		default: /* not reached */
 			break;
 	}
@@ -1095,6 +1192,14 @@ sort_base(
 		case SORT_THREADS_BY_SCORE_ASCEND:
 			qsort(base, (size_t) grpmenu.max, sizeof(long), score_comp_base);
 			break;
+
+		case SORT_THREADS_BY_LAST_POSTING_DATE_DESCEND:
+			qsort(base, (size_t) grpmenu.max, sizeof(long), last_date_comp_base_desc);
+			break;
+
+		case SORT_THREADS_BY_LAST_POSTING_DATE_ASCEND:
+			qsort(base, (size_t) grpmenu.max, sizeof(long), last_date_comp_base_asc);
+			break;
 	}
 }
 
@@ -1117,7 +1222,6 @@ parse_headers(
 {
 	char art_from_addr[HEADER_LEN];
 	char art_full_name[HEADER_LEN];
-	char art_trunc_subj[HEADER_LEN];
 	char *hdr, *ptr;
 	unsigned int lineno = 0;
 	unsigned int max_lineno = 25;
@@ -1212,25 +1316,17 @@ parse_headers(
 
 				/* Received:  If found it's probably a mail article */
 				if (!got_received) {
-					if ((hdr = parse_header(ptr + 1, "eceived", FALSE, FALSE))) {
+					if (parse_header(ptr + 1, "eceived", FALSE, FALSE)) {
 						max_lineno <<= 1;		/* double the max number of line to read for mails */
 						got_received = TRUE;
 					}
 				}
 				break;
 
-			/*
-			 * FIXME: Subject: truncation is a HACK and it's not multibyte safe
-			 *        the core problem are probably fixed length buffers
-			 *        (i.e. in rfc1522_encode() called from write_overview()
-			 *         with the data read in here).
-			 */
 			case 'S':	/* Subject:  mandatory */
 				if (!h->subject) {
-					if ((hdr = parse_header(ptr + 1, "ubject", FALSE, FALSE))) {
-						strncpy(art_trunc_subj, eat_re(eat_tab(convert_to_printable(rfc1522_decode(hdr))), FALSE), sizeof(art_trunc_subj) - 1);
-						h->subject = hash_str(art_trunc_subj);
-					}
+					if ((hdr = parse_header(ptr + 1, "ubject", FALSE, FALSE)))
+						h->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(hdr))), FALSE));
 				}
 				break;
 
@@ -1273,11 +1369,6 @@ parse_headers(
 }
 
 
-#ifdef DEBUG
-#	define handle_overview_fmt_error()	else oerror += 1<<count
-#else
-#	define handle_overview_fmt_error()
-#endif /* DEBUG */
 /*
  * Read in an overview index file. Fields are separated by TAB.
  * return the number of expired articles encountered or -1 if the user aborted
@@ -1307,6 +1398,7 @@ read_overview(
 {
 	FILE *fp;
 	char *ptr;
+	char *q;
 	char *buf;
 	char *group_msg;
 	char art_full_name[HEADER_LEN];
@@ -1315,9 +1407,7 @@ read_overview(
 	int expired = 0;
 	long artnum;
 	struct t_article *art;
-#ifdef DEBUG
-	unsigned int oerror = 0;
-#endif /* DEBUG */
+	size_t over_fields;
 
 	/*
 	 * open the overview file (whether it be local or via nntp)
@@ -1329,6 +1419,31 @@ read_overview(
 		group->xmax = max;
 
 	group_msg = fmt_string(_(txt_group), cCOLS - strlen(_(txt_group)) + 2 - 3, group->name);
+
+	/* get the number of fields per over-record as announced by LIST OVERVIEW.FMT */
+	for (over_fields = 1; ofmt[over_fields].name; over_fields++)
+		;
+	if (!--over_fields) { /* e.g. nntp_caps.type == CAPABILITIES && !nntp_caps.list_overview_fmt -> assume defaults */
+		ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (8 + 1));
+		ofmt[1].type = OVER_T_STRING;
+		ofmt[1].name = strdup("Subject:");
+		ofmt[2].type = OVER_T_STRING;
+		ofmt[2].name = strdup("From:");
+		ofmt[3].type = OVER_T_STRING;
+		ofmt[3].name = strdup("Date:");
+		ofmt[4].type = OVER_T_STRING;
+		ofmt[4].name = strdup("Message-ID:");
+		ofmt[5].type = OVER_T_STRING;
+		ofmt[5].name = strdup("References:");
+		ofmt[6].type = OVER_T_INT;
+		ofmt[6].name = strdup("Bytes:");
+		ofmt[7].type = OVER_T_INT;
+		ofmt[7].name = strdup("Lines:");
+		ofmt[8].type = OVER_T_ERROR;
+		ofmt[8].name = NULL;
+		over_fields = 7;
+	}
+
 	while ((buf = tin_fgets(fp, FALSE)) != NULL) {
 		if (need_resize) {
 			handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
@@ -1383,75 +1498,222 @@ read_overview(
 		 *       to check for additions like we do with xref_supported
 		 */
 		for (count = 1; (ptr = tin_strtok(NULL, "\t")) != NULL; count++) {
-			switch (count) {
-				case 1:		/* Subject */
-					art->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr))), FALSE));
-					break;
-
-				case 2:		/* From */
-					art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
-					art->from = hash_str(buffer_to_ascii(art_from_addr));
-
-					if (*art_full_name)
-						art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name))));
-					break;
-
-				case 3:		/* Date */
-					art->date = parsedate(ptr, (TIMEINFO *) 0);
+			/* skip unexpected tailing fields */
+			if (count > over_fields) {
 #ifdef DEBUG
-					if (art->date == (time_t) -1)
-						oerror += 1<<count;
-#endif /* DEBUG */
-					break;
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "OVER: %d Unexpected overview-field %d of %d: %s", artnum, count, over_fields, ptr);
+#endif	/* DEBUG */
 
-				case 4:		/* Message-ID */
-					if (*ptr)
-						art->msgid = my_strdup(ptr);
-					handle_overview_fmt_error();
-					break;
-
-				case 5:		/* References */
-					if (*ptr)
-						art->refs = my_strdup(ptr);
-					break;
-
-				case 6:		/* Bytes */
+				/* "common error" Xref:full in overview-data but not in OVERVIEW.FTM */
+				if (count == over_fields + 1) {
+					if (!strncasecmp(ptr, "Xref: ", 6)) {
 #ifdef DEBUG
-					if (!isdigit((unsigned char) *ptr))
-						oerror += 1<<count;
-#endif /* DEBUG */
-					break;
-
-				case 7:		/* Lines */
-					if (isdigit((unsigned char) *ptr))
-						art->line_count = atoi(ptr);
-					handle_overview_fmt_error();
-					break;
-
-				case 8:		/* Xref: */
-					if (!xref_supported)
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "OVER: found unexpected Xref: on semi std. position");
+#endif  /* DEBUG */
+						over_fields++;
+						ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (over_fields + 2)); /* + 2 = artnum and end-marker */
+						ofmt[over_fields].type = OVER_T_FSTRING;
+						ofmt[over_fields].name = my_strdup("Xref:");
+						ofmt[over_fields + 1].type = OVER_T_ERROR;
+						ofmt[over_fields + 1].name = NULL;
+						xref_supported = TRUE;
+					} else
 						continue;
-					/* TODO: crosscheck artnum against Xref:-line (if Xref:full) */
-					if ((ptr = parse_header(ptr, "Xref", FALSE, FALSE)) != NULL)
-						art->xref = my_strdup(ptr);
-					handle_overview_fmt_error();
-					break;
+				} else
+					continue;
+			}
+
+			if (expensive_over_parse) { /* strange order */
+				/* madatory fields */
+				if (ofmt[count].type == OVER_T_STRING) {
+					if (!strcasecmp(ofmt[count].name, "Subject:")) {
+						if (*ptr)
+							art->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr))), FALSE));
+						else {
+							art->subject = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) overview-field %s empty", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "From:")) {
+						if (*ptr) {
+							art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
+							art->from = hash_str(buffer_to_ascii(art_from_addr));
+							if (*art_full_name)
+								art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name))));
+						} else {
+							art->from = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) overview-field %s empty", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "Date:")) {
+						art->date = parsedate(ptr, (TIMEINFO *) 0);
+#ifdef DEBUG
+						if ((debug & DEBUG_NNTP) && art->date == (time_t) -1)
+							debug_print_file("NNTP", "OVER(%d) overview-field %s bogus: %s", artnum, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "Message-ID:")) {
+						if (*ptr)
+							art->msgid = my_strdup(ptr);
+						else {
+							art->msgid = NULL;
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) overview-field %s empty", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "References:")) {
+						if (*ptr)
+							art->refs = my_strdup(ptr);
+						else
+							art->refs = NULL;
+						continue;
+					}
+				}
+				/* metadata fiels */
+				if (ofmt[count].type == OVER_T_INT) {
+					if (!strcasecmp(ofmt[count].name, "Bytes:")) {
+						if (*ptr) {
+#ifdef DEBUG
+							if (!isdigit((unsigned char) *ptr))
+								debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						}
+						continue;
+					}
+
+					if (!strcasecmp(ofmt[count].name, "Lines:")) {
+						if (*ptr) {
+							if (isdigit((unsigned char) *ptr))
+								art->line_count = atoi(ptr);
+							else {
+								art->line_count = 0;
+#ifdef DEBUG
+								debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+							}
+						} else
+							art->line_count = 0;
+						continue;
+					}
+				}
+			} else { /* first 7 fields are in RFC 3977 order */
+				switch(count) {
+					case 1: /* Subject: */
+						if (*ptr)
+							art->subject = hash_str(eat_re(eat_tab(convert_to_printable(rfc1522_decode(ptr))), FALSE));
+						else {
+							art->subject = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) overview-field %s empty", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 2:	/* From: */
+						if (*ptr) {
+							art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
+							art->from = hash_str(buffer_to_ascii(art_from_addr));
+							if (*art_full_name)
+								art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name))));
+						} else {
+							art->from = hash_str("");
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) overview-field %s empty", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 3:	/* Date: */
+						art->date = parsedate(ptr, (TIMEINFO *) 0);
+#ifdef DEBUG
+						if ((debug & DEBUG_NNTP) && art->date == (time_t) -1)
+							debug_print_file("NNTP", "OVER(%d) overview-field %s bogus: %s", artnum, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						break;
+
+					case 4:	/* Message-ID: */
+						if (*ptr)
+							art->msgid = my_strdup(ptr);
+						else {
+							art->msgid = NULL;
+#ifdef DEBUG
+							if (debug & DEBUG_NNTP)
+								debug_print_file("NNTP", "OVER(%d) overview-field %s empty", artnum, ofmt[count].name);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 5:	/* References: */
+						if (*ptr)
+							art->refs = my_strdup(ptr);
+						else
+							art->refs = NULL;
+						break;
+
+					case 6:	/* :bytes || Bytes: */
+						if (*ptr) {
+#ifdef DEBUG
+							if (!isdigit((unsigned char) *ptr))
+								debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+						}
+						break;
+
+					case 7:	/* :lines || Lines: */
+						if (*ptr) {
+							if (isdigit((unsigned char) *ptr))
+								art->line_count = atoi(ptr);
+							else {
+								art->line_count = 0;
+#ifdef DEBUG
+								debug_print_file("NNTP", "OVER(%d) overview field %d (%s) missmatch: %s", artnum, count, ofmt[count].name, ptr);
+#endif /* DEBUG */
+							}
+						} else
+							art->line_count = 0;
+						break;
+
+					default:
+						break;
+				}
+			}
+
+			/* optional fields */
+			if (ofmt[count].type == OVER_T_FSTRING) {
+				if (!strcasecmp(ofmt[count].name, "Xref:")) {
+					if ((q = parse_header(ptr, "Xref", FALSE, FALSE)) != NULL)
+						art->xref = my_strdup(q);
+#ifdef DEBUG
+					else {
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "OVER(%d) overview-field %s bogus: %s", artnum, ofmt[count].name, ptr);
+					}
+#endif /* DEBUG */
+				}
+				continue;
 			}
 		}
 
-#ifdef DEBUG
-		/* Complain if incorrect # of fields */
-		if (count < (xref_supported ? 8 : 7) || oerror) {
-			char errbuf[LEN];
-
-			error_message(_("%d Bad overview record (%d fields) '%s'"), oerror, count, BlankIfNull(ptr)); /* TODO move to lang.c */
-			snprintf(errbuf, sizeof(errbuf), "%d Bad overview record (%d fields)", oerror, count);
-			debug_nntp("read_overview", errbuf);
-		}
-		debug_print_header(art);
-		oerror = 0;
-#endif /* DEBUG */
-
 		/*
 		 * RFC says Message-ID is mandatory in newsgroups (but not in
 		 * mailgroups etc..) NB. a NULL Message-ID would abort if we ever do
@@ -1466,13 +1728,46 @@ read_overview(
 
 		top_art++;				/* Basically this statement commits the article */
 	}
-	free(group_msg);
 
+	free(group_msg);
 	TIN_FCLOSE(fp);
 
 	if (tin_errno)
 		return -1;
 
+#if defined(NNTP_ABLE) && defined(XHDR_XREF)
+	if (read_news_via_nntp && !read_saved_news && !xref_supported && nntp_caps.hdr_cmd) {
+		char cbuf[HEADER_LEN];
+
+		snprintf(cbuf, sizeof(cbuf), "%s XREF %ld-%ld", nntp_caps.hdr_cmd, min, max);
+		group_msg = fmt_string("%s XREF loop", nntp_caps.hdr_cmd); /* TODO: find a better message, move to lang.c */
+		if ((fp = nntp_command(cbuf, OK_HEAD, NULL, 0)) != NULL) {
+			while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
+				artnum = atol(ptr);
+				if (artnum <= 0 || artnum < group->xmin || artnum > group->xmax)
+					continue;
+				art = &arts[top_art];
+				set_article(art);
+				if (!art->xref && !strstr(ptr, "(none)")) {
+					if ((q = strchr(ptr, ' ')) == NULL) /* skip article number */
+						continue;
+					ptr = q;
+					while (*ptr && isspace((int) *ptr))
+						ptr++;
+					q = strchr(ptr, '\n');
+					if (q)
+						*q = '\0';
+					art->xref = my_strdup(ptr);
+				}
+				/* we might loose accuracy here, but that shouldn't hurt */
+				if (artnum % MODULO_COUNT_NUM == 0)
+					show_progress(group_msg, artnum - min, max - min);
+			}
+		}
+		free(group_msg);
+	}
+#endif /* NNTP_ABLE && XHDR_XREF */
+
 	return expired;
 }
 
@@ -1535,12 +1830,13 @@ write_overview(
 	fprintf(fp, "%s\n", group->name);
 
 	for_each_art(i) {
+		char *p;
+		char *q, *ref;
+
 		article = &arts[i];
 
 		if (article->thread != ART_EXPIRED && article->artnum >= group->xmin) {
-			char *p;
-			char *q = NULL, *ref = NULL;
-
+			q = ref = NULL;
 			/*
 			 * TODO: instead of tinrc.mm_local_charset we'd better use UTF-8
 			 *       here and in print_from() in the CHARSET_CONVERSION case.
@@ -1589,9 +1885,9 @@ write_overview(
 
 			fprintf(fp, "\n");
 			free(p);
-			if (q != ref) {
-				free(ref);
-				ref = q = NULL;
+			if (article->refs) {
+				FreeIfNeeded(ref);
+				q = NULL;
 			}
 		}
 	}
@@ -1646,8 +1942,8 @@ find_nov_file(
 
 		case GROUP_TYPE_NEWS:
 			/*
-			 * xover_cmd is not an issue here, any gripes and warnings
-			 * about XOVER are handled in nntp_open()
+			 * nntp.caps.over_cmd is not an issue here, any gripes and warnings
+			 * about [X]OVER are handled in nntp_open()
 			 */
 
 			/*
@@ -1664,8 +1960,8 @@ find_nov_file(
 			 */
 #ifndef NNTP_ONLY
 			if (!read_news_via_nntp) {
-				make_base_group_path(novrootdir, group->name, buf);
-				joinpath(nov_file, buf, novfilename);
+				make_base_group_path(novrootdir, group->name, buf, sizeof(buf));
+				joinpath(nov_file, sizeof(nov_file), buf, novfilename);
 				if (access(nov_file, R_OK) == 0) {
 					if (mode == R_OK)
 						return nov_file;		/* Use system wide overviews */
@@ -1719,7 +2015,7 @@ find_nov_file(
 			 * now that we know it is valid
 			 */
 			if (!once_only)
-				joinpath(local_newsgroups_file, index_newsdir, NEWSGROUPS_FILE);
+				joinpath(local_newsgroups_file, sizeof(local_newsgroups_file), index_newsdir, NEWSGROUPS_FILE);
 
 			dir = index_newsdir;
 			break;
@@ -1741,17 +2037,17 @@ find_nov_file(
 		char *ptr;
 
 		snprintf(buf, sizeof(buf), "%lu.%d", hash, i);
-		joinpath(nov_file, dir, buf);
+		joinpath(nov_file, sizeof(nov_file), dir, buf);
 
 		if ((fp = fopen(nov_file, "r")) == NULL)
-			return nov_file;
+			break;
 
 		/*
 		 * No group name header, so not a valid index file => overwrite it
 		 */
 		if (fgets(buf, (int) sizeof(buf), fp) == NULL) {
 			fclose(fp);
-			return nov_file;
+			break;
 		}
 		fclose(fp);
 
@@ -1759,7 +2055,7 @@ find_nov_file(
 			*ptr = '\0';
 
 		if (strcmp(buf, group->name) == 0)
-			return nov_file;
+			break;
 
 	}
 
@@ -2080,6 +2376,61 @@ score_comp_base(
 }
 
 
+/*
+ * Compare the date of the last posted article of two threads.
+ * Used for sorting base[].
+ */
+static int
+last_date_comp_base_desc(
+	t_comptype p1,
+	t_comptype p2)
+{
+	time_t s1_last = get_last_posting_date(*(const long *) p1);
+	time_t s2_last = get_last_posting_date(*(const long *) p2);
+
+	if (s2_last < s1_last)
+		return -1;
+
+	if (s2_last > s1_last)
+		return 1;
+
+	return 0;
+}
+
+
+static int
+last_date_comp_base_asc(
+	t_comptype p1,
+	t_comptype p2)
+{
+	time_t s1_last = get_last_posting_date(*(const long *) p1);
+	time_t s2_last = get_last_posting_date(*(const long *) p2);
+
+	if (s2_last > s1_last)
+		return -1;
+
+	if (s2_last < s1_last)
+		return 1;
+
+	return 0;
+}
+
+
+static time_t get_last_posting_date(
+	long n)
+{
+	long i;
+	time_t last = (time_t) 0;
+
+	for (i = n; i >= 0; i = arts[i].thread) {
+		if (arts[i].date > last)
+			last = arts[i].date;
+	}
+
+	return last;
+}
+
+
 void
 set_article(
 	struct t_article *art)
@@ -2173,8 +2524,8 @@ static char *
 print_from(
 	struct t_article *article)
 {
-	static char from[PATH_LEN];
 	char *p;
+	static char from[PATH_LEN];
 
 	*from = '\0';
 
@@ -2208,10 +2559,10 @@ open_xover_fp(
 	t_bool local)
 {
 #ifdef NNTP_ABLE
-	if (!local && xover_cmd && *mode == 'r' && group->type == GROUP_TYPE_NEWS) {
+	if (!local && nntp_caps.over_cmd && *mode == 'r' && group->type == GROUP_TYPE_NEWS) {
 		char line[NNTP_STRLEN];
 
-		snprintf(line, sizeof(line), "%s %ld-%ld", xover_cmd, min, max);
+		snprintf(line, sizeof(line), "%s %ld-%ld", nntp_caps.over_cmd, min, max);
 		return (nntp_command(line, OK_XOVER, NULL, 0));
 	}
 #endif /* NNTP_ABLE */
