@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   : 2004-07-22
+ *  Updated   : 2008-04-24
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2004 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2008 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,9 +40,7 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef RFC2046_H
-#	include "rfc2046.h"
-#endif /* !RFC2046_H */
+
 
 /*
  * We malloc() this many t_lineinfo's at a time
@@ -57,14 +55,13 @@
 #define MATCH_REGEX(x,y,z)	(pcre_exec(x.re, x.extra, y, z, 0, 0, NULL, 0) >= 0)
 
 
-static int put_rest(char **rest, char **line, int *max_line_len, const int offset);
-static int read_decoded_base64_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read, char **rest);
-static int read_decoded_qp_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read);
 static t_bool header_wanted(const char *line);
 static t_part *new_uue(t_part **part, char *name);
 static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part, int hide_uue, int tabs);
 static void put_cooked(size_t buf_len, t_bool wrap_lines, int flags, const char *fmt, ...);
-static void set_rest(char **rest, const char *ptr);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	static t_bool wexpand_ctrl_chars(wchar_t **wline, size_t *length, size_t lcook_width);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #ifdef DEBUG_ART
 	static void dump_cooked(void);
 #endif /* DEBUG_ART */
@@ -85,63 +82,111 @@ static t_openartinfo *art;
 t_bool
 expand_ctrl_chars(
 	char **line,
-	int *length,
+	size_t *length,
 	size_t lcook_width)
 {
+	t_bool ctrl_L = FALSE;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wline = char2wchar_t(*line);
+	size_t wlen;
+
+	/*
+	 * remove the assert() before release
+	 * it should help us find problems with wide-char strings
+	 * in the development branch
+	 */
+	assert (wline != NULL);
+	wlen = wcslen(wline);
+	ctrl_L = wexpand_ctrl_chars(&wline, &wlen, lcook_width);
+	free(*line);
+	*line = wchar_t2char(wline);
+	free(wline);
+	assert (line != NULL);
+	*length = strlen(*line);
+#else
 	int curr_len = LEN;
 	int i = 0, j;
 	char *buf = my_malloc(curr_len);
-	char *c;
-	t_bool ctrl_L = FALSE, resize = FALSE;
+	unsigned char *c;
 
-	c = *line;
+	c = (unsigned char *) *line;
 	while (*c) {
-		if (resize) {
+		if (i > curr_len - 3) {
 			curr_len <<= 1;
 			buf = my_realloc(buf, curr_len);
-			resize = FALSE;
 		}
 		if (*c == '\t') { /* expand tabs */
-/*			j = ((i + lcook_width) / lcook_width) * lcook_width; */
 			j = i + lcook_width - (i % lcook_width);
-			if (j > curr_len - 2) {
-				resize = TRUE;
-				continue;
-			}
 			for (; i < j; i++)
 				buf[i] = ' ';
+		} else if (((*c) & 0xFF) < ' ' && *c != '\n' && (!IS_LOCAL_CHARSET("Big5") || *c != 27)) {	/* literal ctrl chars */
+			buf[i++] = '^';
+			buf[i++] = ((*c) & 0xFF) + '@';
+			if (*c == '\f')		/* ^L detected */
+				ctrl_L = TRUE;
 		} else {
-			if (((*c) & 0xFF) < ' ' && *c != '\n' && (!IS_LOCAL_CHARSET("Big5") || *c != 27)) {	/* literal ctrl chars */
-				if (i > curr_len - 4) {
-					resize = TRUE;
-					continue;
-				}
-				buf[i++] = '^';
-				buf[i++] = ((*c) & 0xFF) + '@';
-				if (*c == '\f')	/* ^L detected */
-					ctrl_L = TRUE;
-			} else {
-				if (i > curr_len - 3) {
-					resize = TRUE;
-					continue;
-				}
+			if (!my_isprint(*c) && *c != '\n')
+				buf[i++] = '?';
+			else
 				buf[i++] = *c;
-			}
 		}
 		c++;
 	}
-	/* put_cooked() requires a newline at the end of the line */
-	if (buf[i - 1] != '\n')
-		buf[i++] = '\n';	/* Force last char of string to be \n */
 	buf[i] = '\0';
 	*length = i + 1;
 	*line = my_realloc(*line, *length);
 	strcpy(*line, buf);
 	free(buf);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	return ctrl_L;
 }
 
 
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+static t_bool
+wexpand_ctrl_chars(
+	wchar_t **wline,
+	size_t *length,
+	size_t lcook_width)
+{
+	size_t cur_len = LEN, i = 0, j;
+	wchar_t *wbuf = my_malloc(cur_len * sizeof(wchar_t));
+	wchar_t *wc;
+	t_bool ctrl_L = FALSE;
+
+	wc = *wline;
+	while (*wc) {
+		if (i > cur_len - 3) {
+			cur_len <<= 1;
+			wbuf = my_realloc(wbuf, cur_len * sizeof(wchar_t));
+		}
+		if (*wc == '\t') {		/* expand_tabs */
+			j = i + lcook_width - (i % lcook_width);
+			for (; i < j; i++)
+				wbuf[i] = ' ';
+		} else if (*wc  < ' ' && *wc != '\n' && (!IS_LOCAL_CHARSET("Big5") || *wc != 27)) {	/* literal ctrl chars */
+			wbuf[i++] = '^';
+			wbuf[i++] = *wc + '@';
+			if (*wc == '\f')	/* ^L detected */
+				ctrl_L = TRUE;
+		} else {
+			if (!iswprint((wint_t) *wc) && *wc != '\n')
+				wbuf[i++] = '?';
+			else
+				wbuf[i++] = *wc;
+		}
+		wc++;
+	}
+	wbuf[i] = '\0';
+	*length = i + 1;
+	*wline = my_realloc(*wline, *length * sizeof(wchar_t));
+	wcscpy(*wline, wbuf);
+	free(wbuf);
+	return ctrl_L;
+}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+
 /*
  * Output text to the cooked stream. Wrap lines as necessary.
  * Update the line count and the array of line offsets
@@ -160,7 +205,7 @@ put_cooked(
 	char *p, *bufp, *buf;
 	int wrap_column;
 	int space;
-	static int overflow = 0;
+/*	static int overflow = 0; */ /* TODO: we don't use it (anymore?) */
 	static int saved_flags = 0;
 	va_list ap;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -176,7 +221,11 @@ put_cooked(
 	if (tinrc.wrap_column < 0)
 		wrap_column = ((tinrc.wrap_column > -cCOLS) ? cCOLS + tinrc.wrap_column : cCOLS);
 	else
+#if 1
 		wrap_column = ((tinrc.wrap_column > 0) ? tinrc.wrap_column : cCOLS);
+#else	/* never cut off long lines */
+		wrap_column = (((tinrc.wrap_column > 0) && (tinrc.wrap_column < cCOLS)) ? tinrc.wrap_column : cCOLS);
+#endif /* 1 */
 
 	p = bufp = buf;
 
@@ -209,7 +258,7 @@ put_cooked(
 		if (*p == '\n')
 			p++;
 		bufp = p;
-		overflow = 0;
+/*		overflow = 0; */
 
 		if (art->cooked_lines == 0) {
 			art->cookl = my_malloc(sizeof(t_lineinfo) * CHUNK);
@@ -246,7 +295,7 @@ put_cooked(
 	if (*bufp != '\0') {
 		fputs(bufp, art->cooked);
 		saved_flags = flags;
-		overflow += strlen(bufp);
+/*		overflow += strlen(bufp); */
 	}
 
 	va_end(ap);
@@ -254,322 +303,6 @@ put_cooked(
 }
 
 
-/*
- * FIXME: should go into rfc1521.c
- *
- * Set everything in ptr as the rest of a physical line to be processed
- * later.
- */
-static void
-set_rest(
-	char **rest,
-	const char *ptr)
-{
-	char *old_rest = *rest;
-
-	if (ptr == NULL || strlen(ptr) == 0) {
-		FreeAndNull(*rest);
-		return;
-	}
-	*rest = my_strdup(ptr);
-	FreeIfNeeded(old_rest);
-}
-
-
-/*
- * FIXME: should go into rfc1521.c
- *
- * Copy things that were left over from the last decoding into the new line.
- * If there's a newline in the rest, copy everything up to and including that
- * newline into the expected buffer, adjust rest and return. If there's no
- * newline in the rest, copy all of it to the expected buffer and return.
- *
- * Side effects: resizes line if necessary, adjusts max_line_len
- * accordingly.
- *
- * This function returns the number of characters written to the line buffer.
- */
-static int
-put_rest(
-	char **rest,
-	char **line,
-	int *max_line_len,
-	const int offset)
-{
-	char *my_rest = *rest;
-	char *ptr;
-	char c;
-	int put_chars = offset;
-
-	if ((ptr = my_rest) == NULL)
-		return put_chars;
-	if (strlen(my_rest) == 0) {
-		FreeAndNull(*rest);
-		return put_chars;
-	}
-
-	while ((c = *ptr++) && (c != '\n')) {
-		if ((c == '\r') && (*ptr == '\n'))
-			continue;	/* step over CRLF */
-		/*
-		 * Resize line if necessary. Keep in mind that we add LF and \0 later.
-		 */
-		if (put_chars >= *max_line_len - 2) {
-			if (*max_line_len == 0)
-				*max_line_len = LEN;
-			else
-				*max_line_len <<= 1;
-			*line = my_realloc(*line, *max_line_len);
-		}
-		(*line)[put_chars++] = c;
-	}
-	if (c == '\n') {
-		/*
-		 * FIXME: Adding a newline may be not correct. At least it may
-		 * be not what the author of that article intended.
-		 * Unfortunately, a newline is expected at the end of a line by
-		 * some other code in cook.c and even those functions invoking
-		 * this one rely on it.
-		 */
-		(*line)[put_chars++] = '\n';
-		set_rest(rest, ptr);
-	} else /* c == 0 */
-		/* rest is now empty */
-		FreeAndNull(*rest);
-
-	(*line)[put_chars] = '\0';	/* don't count the termining NULL! */
-	return put_chars;
-}
-
-
-/*
- * FIXME: should go into rfc1521.c
- *
- * Read a logical base64 encoded line into the specified line buffer.
- * Logical lines can be split over several physical base64 encoded lines and
- * a single physical base64 encoded line can contain serveral logical lines.
- * This function keeps track of all these cases and always copies only one
- * decoded line to the line buffer.
- *
- * Side effects: resizes line if necessary, adjusts max_line_len
- * accordingly.
- *
- * This function returns the number of physical lines read or a negative
- * value on error.
- */
-static int
-read_decoded_base64_line(
-	FILE *file,
-	char **line,
-	int *max_line_len,
-	const int max_lines_to_read,
-	char **rest)
-{
-	char *buf2;	/* holds the entire decoded line */
-	char *buf;	/* holds the entire encoded line*/
-	int count = 0;
-	int lines_read = 0;
-	int put_chars = 0;
-
-	/*
-	 * First of all, catch everything that is left over from the last decoding.
-	 * If there's a newline in that rest, copy everything up to and including
-	 * that newline in the expected buffer, adjust rest and return. If there's
-	 * no newline in the rest, copy all of it (modulo length of the buffer) to
-	 * the expected buffer and continue as if there was no rest.
-	 */
-	put_chars = put_rest(rest, line, max_line_len, 0);
-	if (put_chars && ((*line)[put_chars - 1] == '\n'))
-		return 0;	/* we didn't read any new lines but filled the line */
-
-	/*
-	 * At this point, either there was no rest or there was no newline in the
-	 * rest. In any case, we need to read further encoded lines and decode
-	 * them until we find a newline or there are no more (encoded or physical)
-	 * lines in this part of the posting. To be sure, now allocate memory for
-	 * the output if it wasn't already done.
-	 */
-	if (*max_line_len == 0) {
-		*max_line_len = LEN;
-		*line = my_malloc(*max_line_len);
-	}
-
-	/*
-	 * max_lines_to_read==0 occurs at end of an encoded part and if there was
-	 * no trailing newline in the encoded text. So we put one there and exit.
-	 * FIXME: Adding a newline may be not correct. At least it may be not
-	 * what the author of that article intended. Unfortunately, a newline is
-	 * expected at the end of a line by some other code in cook.c.
-	 */
-	if (max_lines_to_read <= 0) {
-		if (put_chars) {
-			(*line)[put_chars++] = '\n';
-			(*line)[put_chars] = '\0';
-		}
-		return max_lines_to_read;
-	}
-	/*
-	 * Ok, now read a new line from the original article.
-	 */
-	do {
-		if ((buf = tin_fgets(file, FALSE)) == NULL) {
-			/*
-			 * Premature end of file (or file error), leave loop. To prevent
-			 * re-invoking of this function, set the numbers of read lines to
-			 * the expected maximum that should be read at most.
-			 *
-			 * FIXME: Adding a newline may be not correct. At least it may be
-			 * not what the author of that article intended. Unfortunately, a
-			 * newline is expected at the end of a line by some other code in
-			 * cook.c.
-			 */
-			if (put_chars > *max_line_len - 2) {
-				*max_line_len <<= 1;
-				*line = my_realloc(*line, *max_line_len);
-			}
-			(*line)[put_chars++] = '\n';
-			(*line)[put_chars] = '\0';
-			return max_lines_to_read;
-		}
-		lines_read++;
-		buf2 = my_malloc(strlen(buf) + 1); /* decoded string is always shorter than encoded string, so this is safe */
-		count = mmdecode(buf, 'b', '\0', buf2);
-		buf2[count] = '\0';
-		FreeIfNeeded(*rest);
-		*rest = buf2;
-		put_chars = put_rest(rest, line, max_line_len, put_chars);
-		if (put_chars && ((*line)[put_chars - 1] == '\n')) /* end of logical line reached */
-			return lines_read;
-	} while (lines_read < max_lines_to_read);
-	/*
-	 * FIXME: Adding a newline may be not correct. At least it may be
-	 * not what the author of that article intended. Unfortunately, a
-	 * newline is expected at the end of a line by some other code in
-	 * cook.c.
-	 */
-	if (put_chars > *max_line_len - 2) {
-		*max_line_len <<= 1;
-		*line = my_realloc(*line, *max_line_len);
-	}
-	if ((0 == put_chars) || ('\n' != (*line)[put_chars - 1]))
-			(*line)[put_chars++] = '\n';
-	(*line)[put_chars] = '\0';
-	return lines_read;
-}
-
-
-/*
- * FIXME: should go into rfc1521.c
- *
- * Read a logical quoted-printable encoded line into the specified line
- * buffer. Quoted-printable lines can be split over several physical lines,
- * so this function collects all affected lines, concatenates and decodes
- * them.
- *
- * Side effects: resizes line if necessary, adjusts max_line_len
- * accordingly.
- *
- * This function returns the number of physical lines read or a negative
- * value on error.
- */
-static int
-read_decoded_qp_line(
-	FILE *file,
-	char **line,					/* where to copy the decoded line */
-	int *max_line_len,				/* (maximum) line length */
-	const int max_lines_to_read)	/* don't read more physical lines than told here */
-{
-	char *buf, *buf2;
-	char *ptr;
-	char c;
-	int buflen = LEN;
-	int count = 0;
-	int lines_read = 0;
-	size_t chars_to_add = 0;
-
-	buf = my_malloc(buflen); /* initial internal line buffer */
-	*buf = '\0';
-	do {
-		if ((buf2 = tin_fgets(file, FALSE)) == NULL) {
-			/*
-			 * Premature end of file (or file error, leave loop. To prevent
-			 * re-invokation of this function, set the numbers of read lines
-			 * to the expected maximum that should be read at most.
-			 */
-			lines_read = max_lines_to_read;
-			break;
-		}
-		lines_read++;
-		if ((chars_to_add = strlen(buf2)) == 0) /* Empty line, leave loop. */
-			break;
-
-		/*
-		 * Strip trailing white space at the end of the line.
-		 * See RFC 2045, section 6.7, #3
-		 */
-		c = buf2[chars_to_add - 1];
-		while ((chars_to_add > 0) && ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r'))) {
-			--chars_to_add;
-			c = (chars_to_add > 0 ? buf2[chars_to_add - 1] : '\0');
-		}
-
-		/*
-		 * '=' at the end of a line indicates a soft break meaning
-		 * that the following physical line "belongs" to this one.
-		 * (See RFC 2045, section 6.7, #5)
-		 *
-		 * Skip that equal sign now; since c holds this char, the
-		 * loop is not left but the next line is read and concatenated
-		 * with this one while the '=' is overwritten.
-		 */
-		if (c == '=') /* c is 0 when chars_to_add is 0 so this is safe */
-			buf2[--chars_to_add] = '\0';
-
-		/*
-		 * Join physical lines to a logical one; keep in mind that a LF is
-		 * added afterwards.
-		 */
-		if (chars_to_add > buflen - strlen(buf) - 2) {
-			buflen <<= 1;
-			buf = my_realloc(buf, buflen);
-		}
-		strncat(buf, buf2, buflen);
-	} while ((c == '=') && (lines_read < max_lines_to_read));
-	/*
-	 * re-add newline and NULL termination at end of line
-	 * FIXME: Adding a newline may be not correct. At least it may be not
-	 * what the author of that article intended. Unfortunately, a newline is
-	 * expected at the end of a line by some other code in cook.c.
-	 */
-	strcat(buf, "\n");
-
-	/*
-	 * Now decode complete (logical) line from buf to buf2 and copy it to the
-	 * buffer where the invoking function expects it. Don't decode directly
-	 * to the buffer of the other function to prevent buffer overruns and to
-	 * decide if the encoding was ok.
-	 */
-	buf2 = my_malloc(strlen(buf) + 1); /* Don't use realloc here, tin_fgets relies on its internal state! */
-	count = mmdecode(buf, 'q', '\0', buf2);
-
-	if (count >= 0) {
-		buf2[count] = '\0';
-		ptr = buf2;
-	} else	/* error in encoding: copy raw line */
-		ptr = buf;
-
-	if (*max_line_len < (int) strlen(ptr) + 1) {
-		*max_line_len = strlen(ptr) + 1;
-		*line = my_realloc(*line, *max_line_len);
-	}
-	strncpy(*line, ptr, *max_line_len);
-	(*line)[*max_line_len - 1] = '\0'; /* be sure to terminate string */
-	free(buf);
-	free(buf2);
-	return lines_read;
-}
-
-
 /*
  * Add a new uuencode attachment description to the current part
  */
@@ -667,12 +400,13 @@ process_text_body_part(
 {
 	char *rest = NULL;
 	char *line = NULL, *buf;
-	int max_line_len = 0;
+	size_t max_line_len = 0;
 	int flags, len, lines_left;
 	int offsets[6];
 	int size_offsets = ARRAY_SIZE(offsets);
 	t_bool in_sig = FALSE;			/* Set when in sig portion */
 	t_bool in_uue = FALSE;			/* Set when in uuencoded section */
+	t_bool verbatim = FALSE;		/* Set when in verbatim section */
 	t_bool is_uubody;				/* Set when current line looks like a uuencoded line */
 	t_part *curruue = NULL;
 
@@ -709,7 +443,7 @@ process_text_body_part(
 				 * especially if we must resize it.
 				 * So copy buf to line (and resize line if necessary).
 				 */
-				if (max_line_len < (int) strlen(buf) + 2) {
+				if (max_line_len < strlen(buf) + 2) {
 					max_line_len = strlen(buf) + 2;
 					line = my_realloc(line, max_line_len);
 				};
@@ -738,100 +472,111 @@ process_text_body_part(
 
 		len = (int) strlen(line);
 
-		/*
-		 * Detect and skip signatures if necessary
-		 */
-		if (!in_sig) {
-			if (strcmp(line, SIGDASHES) == 0) {
-				in_sig = TRUE;
+		/* look for verbatim marks */
+		if (!in_sig && !in_uue && !verbatim && MATCH_REGEX(verbatim_begin_regex, line, len))
+			verbatim = TRUE;
+		if (verbatim && MATCH_REGEX(verbatim_end_regex, line, len))
+			verbatim = FALSE;
+
+		if (!verbatim) {
+			/*
+			 * Detect and skip signatures if necessary
+			 */
+			if (!in_sig) {
+				if (strcmp(line, SIGDASHES) == 0) {
+					in_sig = TRUE;
+					if (in_uue) {
+						in_uue = FALSE;
+						if (hide_uue)
+							PUT_UUE(curruue, _(txt_incomplete));
+					}
+				}
+			}
+
+			if (in_sig && !tinrc.show_signatures)
+				continue;					/* No further processing needed */
+
+			/*
+			 * Detect and process uuencoded sections
+			 * Look for the start or the end of a uuencoded section
+			 *
+			 * TODO: look for a tailing size line after end (non standard
+			 *       extension)?
+			 */
+			if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
+				in_uue = TRUE;
+				curruue = new_uue(&part, line + offsets[1]);
+				if (hide_uue)
+					continue;				/* Don't cook the 'begin' line */
+			} else if (strncmp(line, "end\n", 4) == 0) {
 				if (in_uue) {
 					in_uue = FALSE;
-					if (hide_uue)
-						PUT_UUE(curruue, _(txt_incomplete));
+					if (hide_uue) {
+						PUT_UUE(curruue, "");
+						continue;			/* Don't cook the 'end' line */
+					}
 				}
 			}
-		}
 
-		if (in_sig && !tinrc.show_signatures)
-			continue;					/* No further processing needed */
-
-		/*
-		 * Detect and process uuencoded sections
-		 * Look for the start or the end of a uuencoded section
-		 */
-		if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
-			in_uue = TRUE;
-			curruue = new_uue(&part, line + offsets[1]);
-			if (hide_uue)
-				continue;				/* Don't cook the 'begin' line */
-		} else if (strncmp(line, "end\n", 4) == 0) {
-			if (in_uue) {
-				in_uue = FALSE;
-				if (hide_uue) {
-					PUT_UUE(curruue, "");
-					continue;			/* Don't cook the 'end' line */
-				}
-			}
-		}
-
-		/*
-		 * See if this line looks like a uuencoded 'body' line
-		 */
-		is_uubody = FALSE;
-
-		if (MATCH_REGEX (uubody_regex, line, len)) {
-			int sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
-
-			/* sum = 0 in a uubody only on the last line, a single ` */
-			if (sum == 0 && len == 1 + 1)			/* +1 for the \n */
-				is_uubody = TRUE;
-			else if (len == sum + 1 + 1)
-				is_uubody = TRUE;
-#ifdef DEBUG_ART
-			if (debug == 2)
-				fprintf(stderr, "%s sum=%d len=%d (%s)\n", bool_unparse(is_uubody), sum, len, line);
-#endif /* DEBUG_ART */
-		}
-
-		if (in_uue) {
-			if (is_uubody)
-				curruue->line_count++;
-			else {
-				if (line[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
-					/* fprintf(stderr, "not a uue line while reading a uue body?\n"); */
-					in_uue = FALSE;
-					if (hide_uue)
-						/* don't continue here, so we see the line that 'broke' in_uue */
-						PUT_UUE(curruue, _(txt_incomplete));
-				}
-			}
-		} else {
 			/*
-			 * UUE_ALL = 'Try harder' - we never saw a begin line, but useful
-			 * when uue sections are split across > 1 article
+			 * See if this line looks like a uuencoded 'body' line
 			 */
-			if (is_uubody && hide_uue == UUE_ALL) {
-				char name[] = N_("(unknown)");
+			is_uubody = FALSE;
 
-				curruue = new_uue(&part, name);
-				curruue->line_count++;
-				in_uue = TRUE;
-				continue;
+			if (MATCH_REGEX(uubody_regex, line, len)) {
+				int sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
+
+				/* sum = 0 in a uubody only on the last line, a single ` */
+				if (sum == 0 && len == 1 + 1)			/* +1 for the \n */
+					is_uubody = TRUE;
+				else if (len == sum + 1 + 1)
+					is_uubody = TRUE;
+#ifdef DEBUG_ART
+				if (debug & DEBUG_MISC)
+					fprintf(stderr, "%s sum=%d len=%d (%s)\n", bool_unparse(is_uubody), sum, len, line);
+#endif /* DEBUG_ART */
 			}
+
+			if (in_uue) {
+				if (is_uubody)
+					curruue->line_count++;
+				else {
+					if (line[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
+						/* fprintf(stderr, "not a uue line while reading a uue body?\n"); */
+						in_uue = FALSE;
+						if (hide_uue)
+							/* don't continue here, so we see the line that 'broke' in_uue */
+							PUT_UUE(curruue, _(txt_incomplete));
+					}
+				}
+			} else {
+				/*
+				 * UUE_ALL = 'Try harder' - we never saw a begin line, but useful
+				 * when uue sections are split across > 1 article
+				 */
+				if (is_uubody && hide_uue == UUE_ALL) {
+					char name[] = N_("(unknown)");
+
+					curruue = new_uue(&part, name);
+					curruue->line_count++;
+					in_uue = TRUE;
+					continue;
+				}
+			}
+
+			/*
+			 * Skip output if we're hiding uue or the sig
+			 */
+			if (in_uue && hide_uue)
+				continue;	/* No further processing needed */
 		}
 
-		/*
-		 * Skip output if we're hiding uue or the sig
-		 */
-		if (in_uue && hide_uue)
-			continue;						/* No further processing needed */
-
-		flags = in_sig ? C_SIG : C_BODY;
+		flags = verbatim ? 0 : in_sig ? C_SIG : C_BODY;
 
 		/*
-		 * Don't do any further handling of uue lines - the data is binary after all
+		 * Don't do any further handling of uue || verbatim lines
 		 */
-		if (in_uue) {
+		if (in_uue || verbatim) {
 			put_cooked(max_line_len, wrap_lines, flags, "%s", line);
 			continue;
 		}
@@ -858,13 +603,6 @@ process_text_body_part(
 		if (MATCH_REGEX(news_regex, line, len))
 			flags |= C_NEWS;
 
-		/*
-		 * Basically, c_b2p() does: if (!(my_isprint(*c) || *c==8 || *c==9 || *c==12))
-		 * It is only used here
-		 * How about if !isprint() && !isctrl() - expand_ctrl_chars is done at display time.
-		 * TODO: integrate into expand_ctrl_chars
-		 */
-		convert_body2printable(line);
 		if (expand_ctrl_chars(&line, &max_line_len, tabs))
 			flags |= C_CTRLL;				/* Line contains form-feed */
 		put_cooked(max_line_len, wrap_lines && (!IS_LOCAL_CHARSET("Big5")), flags, "%s", line);
@@ -997,9 +735,13 @@ cook_article(
 		}
 
 		if (header_wanted(line)) {	/* Put cooked data */
-			int i = LEN;
-			char *l = my_strdup(convert_body2printable(rfc1522_decode(line)));	/* FIXME: don't decode addr-part of From:/Cc:/ etc.pp. */
+			size_t i = LEN;
+			char *l = my_strdup(rfc1522_decode(line));	/* FIXME: don't decode addr-part of From:/Cc:/ etc.pp. */
 
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			if (IS_LOCAL_CHARSET("UTF-8"))
+				utf8_valid(l);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			header_put = TRUE;
 			expand_ctrl_chars(&l, &i, tabs);
 			put_cooked(LEN, wrap_lines, C_HEADER, "%s", l);
