@@ -3,10 +3,10 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-07-14
+ *  Updated   : 2008-03-14
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2008 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -97,14 +97,30 @@ main(
 	}
 #endif /* HAVE_SETLOCALE && !NO_LOCALE */
 
+	/*
+	 * determine local charset
+	 */
+#ifndef NO_LOCALE
+	{
+		const char *p;
+
+		if ((p = tin_nl_langinfo(CODESET)) != NULL) {
+			if (!strcasecmp(p, "ANSI_X3.4-1968"))
+				STRCPY(tinrc.mm_local_charset, "US-ASCII");
+			else
+				STRCPY(tinrc.mm_local_charset, p);
+		}
+	}
+#endif /* !NO_LOCALE */
+	/* always set a default value */
+	if (!*tinrc.mm_local_charset)
+		STRCPY(tinrc.mm_local_charset, "US-ASCII");
+
 	set_signal_handlers();
 
 	debug = 0;	/* debug OFF */
 
 	base_name(argv[0], tin_progname);
-#ifdef VMS
-	argv[0] = tin_progname;
-#endif /* VMS */
 
 #ifdef NNTP_ONLY
 	read_news_via_nntp = TRUE;
@@ -129,6 +145,7 @@ main(
 	hash_init();
 	init_selfinfo();
 	init_group_hash();
+	setup_default_keys(); /* preinit keybindings */
 
 	/*
 	 * Read user local & global config files
@@ -186,9 +203,13 @@ main(
 
 	/*
 	 * exit early - unfortunately we can't do that in read_cmd_line_options()
-	 * as xover_cmd is set in nntp_open()
+	 * as nntp_caps.over_cmd is set in nntp_open()
+	 *
+	 * TODO: does the logic make sense? what
+	 * if (update_index && !nntp_caps.over_cmd && !tinrc.cache_overview_files)
+	 * no error message? why?
 	 */
-	if (update_index && xover_cmd && !tinrc.cache_overview_files) {
+	if (update_index && nntp_caps.over_cmd && !tinrc.cache_overview_files) {
 		error_message(_(txt_batch_update_unavail), tin_progname);
 		giveup();
 	}
@@ -196,33 +217,35 @@ main(
 	/*
 	 * Check if overview indexes contain Xref: lines
 	 */
-	if (xover_cmd)
+#ifdef NNTP_ABLE
+	if ((read_news_via_nntp && nntp_caps.over_cmd) || !read_news_via_nntp)
+#endif /* NNTP_ABLE */
 		xref_supported = overview_xref_support();
 
-#ifdef DEBUG_NEWSRC
-	{
+#ifdef DEBUG
+	if (debug & DEBUG_NEWSRC) {
 		char file[PATH_LEN];
 
-		joinpath(file, TMPDIR, "BITMAP");
+		joinpath(file, sizeof(file), TMPDIR, "BITMAP");
 		unlink(file);
-		/* newsrc_test_harness(); */
 	}
-#endif /* DEBUG_NEWSRC */
+#endif /* DEBUG */
+
+	/*
+	 * avoid empty regexp, we also need to do this in batch_mode
+	 * as read_overview() calls eat_re() which uses a regexp to
+	 * modify the subject *sigh*
+	 */
+	postinit_regexp();
 
 	if (!(batch_mode || post_postponed_and_exit)) {
 		/*
 		 * Read user specific keybindings and input history
 		 */
 		wait_message(0, _(txt_reading_keymap_file));
-		if (!read_keymap_file())
-			prompt_continue();
+		read_keymap_file();
 		read_input_history_file();
 
-		/*
-		 * avoid empty regexp
-		 */
-		postinit_regexp();
-
 		/*
 		 * Load the mail & news active files into active[]
 		 *
@@ -284,8 +307,11 @@ main(
 	/*
 	 * Read text descriptions for mail and/or news groups
 	 */
-	if (show_description && !batch_mode)
+	if (show_description && !batch_mode) {
+		no_write = tmp_no_write; /* restore original value */
 		read_descriptions(TRUE);
+		no_write = TRUE; /* disable newsrc updates */
+	}
 
 	/*
 	 * TODO: what has write_config_file() to do with create_mail_save_dirs ()
@@ -442,7 +468,7 @@ read_cmd_line_options(
 				show_description = FALSE;
 				break;
 
-			case 'D':		/* debug mode 1=NNTP, 2=ALL, 3=newsrc, 4=malloc */
+			case 'D':		/* debug mode */
 #ifdef DEBUG
 				debug = atoi(optarg);
 				debug_delete_files();
@@ -637,24 +663,24 @@ read_cmd_line_options(
 	max_cmdargs = argc;
 	if (!newsrc_set) {
 		if (read_news_via_nntp)
-			get_newsrcname(newsrc, getserverbyfile(NNTP_SERVER_FILE));
+			get_newsrcname(newsrc, sizeof(newsrc), getserverbyfile(NNTP_SERVER_FILE));
 		else {
 #if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
 			struct utsname uts;
 			(void) uname(&uts);
-			get_newsrcname(newsrc, uts.nodename);
+			get_newsrcname(newsrc, sizeof(newsrc), uts.nodename);
 #else
 			char nodenamebuf[256]; /* SUSv2 limit; better use HOST_NAME_MAX */
 #ifdef HAVE_GETHOSTNAME
 			(void) gethostname(nodenamebuf, sizeof(nodenamebuf));
 #endif /* HAVE_GETHOSTNAME */
-			get_newsrcname(newsrc, nodenamebuf);
+			get_newsrcname(newsrc, sizeof(newsrc), nodenamebuf);
 #endif /* HAVE_SYS_UTSNAME_H && HAVE_UNAME */
 		}
 	}
 
 	/*
-	 * Sort out conflicts of options....
+	 * Sort out option conflicts
 	 */
 	if (!batch_mode) {
 		if (verbose) {
@@ -671,6 +697,33 @@ read_cmd_line_options(
 			read_saved_news = FALSE;
 		}
 	}
+	if (post_postponed_and_exit && force_no_post) {
+		wait_message(2, _(txt_useless_combination), "-o", "-x", "-x");
+		force_no_post = FALSE;
+	}
+	if (post_article_and_exit && force_no_post) {
+		wait_message(2, _(txt_useless_combination), "-w", "-x", "-x");
+		force_no_post = FALSE;
+	}
+	if (catchup && start_any_unread) {
+		wait_message(2, _(txt_useless_combination), "-c", "-z", "-c");
+		catchup = FALSE;
+	}
+	if (catchup && no_write) {
+		wait_message(2, _(txt_useless_combination), "-c", "-X", "-c");
+		catchup = FALSE;
+	}
+	if (catchup && check_any_unread) {
+		wait_message(2, _(txt_useless_combination), "-c", "-Z", "-c");
+		catchup = FALSE;
+	}
+	if (mail_news || save_news || update_index || check_any_unread || catchup)
+		batch_mode = TRUE;
+	else
+		batch_mode = FALSE;
+	if (batch_mode && (post_article_and_exit || post_postponed_and_exit))
+		batch_mode = FALSE;
+
 #ifdef NNTP_ABLE
 	/*
 	 * If we're reading from an NNTP server and we've been asked not to look
@@ -820,7 +873,7 @@ read_cmd_line_groups(
 
 			for_each_group(i) {
 				if (match_group_list(active[i].name, cmdargs[num])) {
-					if (my_group_add(active[i].name) != -1)
+					if (my_group_add(active[i].name, TRUE) != -1)
 						matched++;
 				}
 			}
@@ -843,7 +896,7 @@ create_mail_save_dirs(
 	struct stat sb;
 
 	if (!strfpath(tinrc.maildir, path, sizeof(path), NULL))
-		joinpath(path, homedir, DEFAULT_MAILDIR);
+		joinpath(path, sizeof(path), homedir, DEFAULT_MAILDIR);
 
 	if (stat(path, &sb) == -1) {
 		my_mkdir(path, (mode_t) (S_IRWXU));
@@ -851,7 +904,7 @@ create_mail_save_dirs(
 	}
 
 	if (!strfpath(tinrc.savedir, path, sizeof(path), NULL))
-		joinpath(path, homedir, DEFAULT_SAVEDIR);
+		joinpath(path, sizeof(path), homedir, DEFAULT_SAVEDIR);
 
 	if (stat(path, &sb) == -1) {
 		my_mkdir(path, (mode_t) (S_IRWXU));
