@@ -3,10 +3,10 @@
  *  Module    : misc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-08-20
+ *  Updated   : 2008-02-25
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2008 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,9 +50,6 @@
 #ifndef TIN_POLICY_H
 #	include "policy.h"
 #endif /* !TIN_POLICY_H */
-#ifndef RFC2046_H
-#	include "rfc2046.h"
-#endif /* !RFC2046_H */
 
 #if defined(HAVE_IDNA_H) && !defined(_IDNA_H)
 #	include <idna.h>
@@ -83,7 +80,7 @@ static int gnksa_dequote_plainphrase(char *realname, char *decoded, int addrtype
 static int strfeditor(char *editor, int linenum, const char *filename, char *s, size_t maxsize, char *format);
 static void write_input_history_file(void);
 #ifdef CHARSET_CONVERSION
-	static t_bool buffer_to_local(char **line, int *max_line_len, const char *network_charset, const char *local_charset);
+	static t_bool buffer_to_local(char **line, size_t *max_line_len, const char *network_charset, const char *local_charset);
 #endif /* CHARSET_CONVERSION */
 #if 0 /* currently unused */
 	static t_bool stat_article(long art, const char *group_path);
@@ -199,7 +196,7 @@ copy_fp(
  * try to backup filename as backupname. on success backupname has the same
  * permissions as filename.
  *
- * returncodes:
+ * return codes:
  * TRUE  = backup complete or source file was missing
  * FALSE = backup failed
  */
@@ -321,7 +318,7 @@ copy_body(
 					}
 					buf2[i] = '\0';
 					if (status_char)	/* already quoted */
-						retcode = fprintf(fp_op, "%s>%s", buf2, strchr(buf, '>'));
+						retcode = fprintf(fp_op, "%s>%s", buf2, BlankIfNull(strchr(buf, '>')));
 					else	/* ... to be quoted ... */
 						retcode = fprintf(fp_op, "%s%s", prefixbuf, buf);
 				} else	/* line was not already quoted (no >) */
@@ -554,7 +551,7 @@ tin_done(
 			group = &active[my_group[i]];
 			if (group->read_during_session) {
 				if (ask) {
-					if (prompt_yn(cLINES, _(txt_catchup_all_read_groups), FALSE) == 1) {
+					if (prompt_yn(_(txt_catchup_all_read_groups), FALSE) == 1) {
 						ask = FALSE;
 						tinrc.thread_articles = THREAD_NONE;	/* speeds up index loading */
 					} else
@@ -571,8 +568,10 @@ tin_done(
 	 * chance to try again
 	 */
 	if (!no_write) {
-		forever {
-			if (((wrote_newsrc_lines = write_newsrc()) >= 0L) && (wrote_newsrc_lines >= read_newsrc_lines)) {
+		i = 3; /* max retries */
+		while (i--) {
+			wrote_newsrc_lines = write_newsrc();
+			if ((wrote_newsrc_lines >= 0L) && (wrote_newsrc_lines >= read_newsrc_lines)) {
 				if (!batch_mode || verbose)
 					my_fputs(_(txt_newsrc_saved), stdout);
 				break;
@@ -590,15 +589,13 @@ tin_done(
 			}
 
 			if (!batch_mode) {
-				if (!prompt_yn(cLINES, _(txt_newsrc_again), TRUE))
+				if (prompt_yn(_(txt_newsrc_again), TRUE) <= 0)
 					break;
 			}
 		}
 
 		write_input_history_file();
-#if 0
 		write_attributes_file(local_attributes_file);
-#endif /* 0 */
 
 #ifdef HAVE_MH_MAIL_HANDLING
 		write_mail_active_file();
@@ -615,6 +612,7 @@ tin_done(
 
 	free_all_arrays();
 
+	/* TODO: why do we make this exception here? */
 #ifdef SIGUSR1
 	if (ret != -SIGUSR1) {
 #endif /* SIGUSR1 */
@@ -650,12 +648,6 @@ tin_done(
 	}
 #endif /* USE_DBMALLOC */
 
-#ifdef VMS
-	if (!ret)
-		ret = 1;
-	vms_close_stdin(); /* free resources used by ReadCh */
-#endif /* VMS */
-
 	exit(ret);
 }
 
@@ -747,30 +739,6 @@ rename_file(
 #endif /* M_UNIX */
 
 
-#ifdef VMS
-void
-rename_file(
-	char *old_filename,
-	char *new_filename)
-{
-	char new_filename_vms[1024];
-
-	if (!strchr(strchr(new_filename, ']') ? strchr(new_filename, ']') : new_filename, '.')) {
-		/* without final dot the new filename is not as tin expects */
-		if (strlen(new_filename) >= sizeof(new_filename_vms)) {
-			perror_message("length of %s is too large", new_filename);
-			return;
-		}
-		strcpy(new_filename_vms, new_filename);
-		strcat(new_filename_vms, ".");
-		new_filename = &new_filename_vms[0];
-	}
-
-	if (rename(old_filename, new_filename))
-		perror_message(_(txt_rename_error), old_filename, new_filename);
-}
-#endif /* VMS */
-
 
 /*
  * Note that we exit screen/curses mode when invoking
@@ -782,7 +750,9 @@ invoke_cmd(
 {
 	int ret;
 	t_bool save_cmd_line = cmd_line;
+#ifndef IGNORE_SYSTEM_STATUS
 	t_bool success;
+#endif /* IGNORE_SYSTEM_STATUS */
 
 	if (!save_cmd_line) {
 		EndWin();
@@ -792,13 +762,9 @@ invoke_cmd(
 
 	TRACE(("called system(%s)", _nc_visbuf(nam)));
 	ret = system(nam);
-#ifdef VMS
-	system_status = 0;
-#else
-#	ifndef USE_SYSTEM_STATUS
+#ifndef USE_SYSTEM_STATUS
 	system_status = (ret >= 0 && WIFEXITED(ret)) ? WEXITSTATUS(ret) : 0;
-#	endif /* !USE_SYSTEM_STATUS */
-#endif /* VMS */
+#endif /* !USE_SYSTEM_STATUS */
 	TRACE(("return %d (%d)", ret, system_status));
 
 	set_signal_catcher(TRUE);
@@ -812,11 +778,7 @@ invoke_cmd(
 	return TRUE;
 #else
 
-#	ifdef VMS
-		success = (ret != 0);
-#	else
-		success = (ret == 0);
-#	endif /* VMS */
+	success = (ret == 0);
 
 	if (!success || system_status != 0)
 		error_message(_(txt_command_failed), nam);
@@ -869,9 +831,6 @@ base_name(
 	char *file)				/* baz */
 {
 	size_t i;
-#ifdef VMS
-	char *cp;
-#endif /* VMS */
 
 	strcpy(file, fullpath);
 
@@ -881,10 +840,6 @@ base_name(
 			break;
 		}
 	}
-#ifdef VMS
-	if ((cp = strrchr(file, '.')) != 0)
-		*cp = '\0';
-#endif /* VMS */
 }
 
 
@@ -911,7 +866,10 @@ dir_name(
 
 /*
  * Return TRUE if new mail has arrived
+ *
+ * TODO: why not cache the mailbox_name?
  */
+#define MAILDIR_NEW	"new"
 t_bool
 mail_check(
 	void)
@@ -921,8 +879,33 @@ mail_check(
 
 	mailbox_name = get_val("MAIL", mailbox);
 
-	if (mailbox_name != 0 && stat(mailbox_name, &buf) >= 0 && buf.st_atime < buf.st_mtime && buf.st_size > 0)
-		return TRUE;
+	if (mailbox_name != 0 && stat(mailbox_name, &buf) >= 0) {
+		if ((int) (buf.st_mode & S_IFMT) == (int) S_IFDIR) { /* maildir setup */
+			DIR *dirp;
+			char *maildir_box;
+			size_t maildir_box_len = strlen(mailbox_name) + strlen(MAILDIR_NEW) + 2;
+			struct dirent *dp;
+
+			maildir_box = my_malloc(maildir_box_len);
+			joinpath(maildir_box, maildir_box_len, mailbox_name, MAILDIR_NEW);
+
+			if (!(dirp = opendir(maildir_box))) {
+				free(maildir_box);
+				return FALSE;
+			}
+			free(maildir_box);
+			while ((dp = readdir(dirp)) != NULL) {
+				if ((strcmp(dp->d_name, ".")) && (strcmp(dp->d_name, ".."))) {
+					closedir(dirp);
+					return TRUE;
+				}
+			}
+			closedir(dirp);
+		} else {
+			if (buf.st_atime < buf.st_mtime && buf.st_size > 0)
+				return TRUE;
+		}
+	}
 	return FALSE;
 }
 
@@ -952,13 +935,13 @@ eat_re(
 		data = pcre_exec(strip_re_regex.re, strip_re_regex.extra, s, strlen(s), 0, 0, offsets, size_offsets);
 		if (offsets[0] == 0)
 			s += offsets[1];
-	} while (data > 0);
+	} while (data >= 0);
 
 	if (eat_was) do {
 		data = pcre_exec(strip_was_regex.re, strip_was_regex.extra, s, strlen(s), 0, 0, offsets, size_offsets);
 		if (offsets[0] > 0)
 			s[offsets[0]] = '\0';
-	} while (data > 0);
+	} while (data >= 0);
 
 	return s;
 }
@@ -1120,7 +1103,7 @@ create_index_lock_file(
 /*
  * strfquote() - produce formatted quote string
  *   %A  Articles Email address
- *   %D  Articles Date
+ *   %D  Articles Date (uses tinrc.date_format)
  *   %F  Articles Address+Name
  *   %G  Groupname of Article
  *   %M  Articles MessageId
@@ -1215,8 +1198,10 @@ strfquote(
 					}
 					break;
 
-				case 'D':	/* Articles Date */
-					STRCPY(tbuf, BlankIfNull(pgart.hdr.date));
+				case 'D':	/* Articles Date (reformated as specified in tinrc.date_format) */
+					if (!my_strftime(tbuf, LEN - 1, tinrc.date_format, localtime(&arts[this_resp].date))) {
+						STRCPY(tbuf, BlankIfNull(pgart.hdr.date));
+					}
 					break;
 
 				case 'F':	/* Articles Address+Name */
@@ -1465,12 +1450,7 @@ _strfpath(
 		/*
 		 * If just a normal part of the pathname copy it
 		 */
-#ifdef VMS
-		if (!strchr("~=+%", *format))
-#else
-		if (!strchr("~$=+%", *format))
-#endif /* VMS */
-		{
+		if (!strchr("~$=+%", *format)) {
 			*str++ = *format;
 			continue;
 		}
@@ -1479,7 +1459,7 @@ _strfpath(
 			case '~':			/* Users or another users homedir */
 				switch (*++format) {
 					case '/':	/* users homedir */
-						joinpath(tbuf, homedir, "");
+						joinpath(tbuf, sizeof(tbuf), homedir, "");
 						break;
 
 					default:	/* some other users homedir */
@@ -1501,7 +1481,6 @@ _strfpath(
 					return 0;
 				break;
 
-#ifndef VMS
 			case '$':	/* Read the envvar and use its value */
 				i = 0;
 				format++;
@@ -1539,7 +1518,6 @@ _strfpath(
 					return 0;
 				}
 				break;
-#endif /* !VMS */
 
 			case '=':
 				/*
@@ -1555,9 +1533,9 @@ _strfpath(
 
 					if (strfpath(group->attribute->maildir, buf, sizeof(buf), group)) {
 						if (*(format + 1) == '\0')				/* Just an = */
-							joinpath(tbuf, buf, group->name);
+							joinpath(tbuf, sizeof(tbuf), buf, group->name);
 						else
-							joinpath(tbuf, buf, "");
+							joinpath(tbuf, sizeof(tbuf), buf, "");
 						if ((str = strfpath_cp(str, tbuf, endp)) == NULL)
 							return 0;
 					} else {
@@ -1586,8 +1564,8 @@ _strfpath(
 #else
 						my_strncpy(tmp, group->name, 14);
 #endif /* HAVE_LONG_FILE_NAMES */
-						JOINPATH(tbuf, buf, tmp);	/* Add the group name */
-						joinpath(tmp, tbuf, "");
+						joinpath(tbuf, sizeof(tbuf), buf, tmp);	/* Add the group name */
+						joinpath(tmp, sizeof(tmp), tbuf, "");
 						if ((str = strfpath_cp(str, tmp, endp)) == NULL)
 							return 0;
 					} else {
@@ -1658,7 +1636,6 @@ strfpath(
 }
 
 
-
 /*
  * TODO: Properly explain this
  */
@@ -1983,9 +1960,6 @@ make_group_path(
 	const char *name,
 	char *path)
 {
-#ifdef VMS
-	sprintf(path, "[%s]", name);
-#else
 	while (*name) {
 		*path = ((*name == '.') ? '/' : *name);
 		name++;
@@ -1993,7 +1967,6 @@ make_group_path(
 	}
 	*path++ = '/';
 	*path = '\0';
-#endif /* VMS */
 }
 
 
@@ -2007,12 +1980,13 @@ void
 make_base_group_path(
 	const char *base_dir,
 	const char *group_name,
-	char *group_path)
+	char *group_path,
+	size_t group_path_len)
 {
 	char buf[LEN];
 
 	make_group_path(group_name, buf);
-	joinpath(group_path, base_dir, buf);
+	joinpath(group_path, group_path_len, base_dir, buf);
 }
 
 
@@ -2026,15 +2000,12 @@ cleanup_tmp_files(
 #if 0
 	char acNovFile[PATH_LEN];
 
-	if (xover_cmd && !tinrc.cache_overview_files) {
+	if (nntp_caps.over_cmd && !tinrc.cache_overview_files) {
 		snprintf(acNovFile, sizeof(acNovFile), "%s%d.idx", TMPDIR, (int) process_id);
 		unlink(acNovFile);
 	}
 #endif /* 0 */
 
-	if (!tinrc.cache_overview_files)
-		unlink(local_newsgroups_file);
-
 	if (batch_mode)
 		unlink(lock_file);
 }
@@ -2320,7 +2291,7 @@ strip_name(
 static t_bool
 buffer_to_local(
 	char **line,
-	int *max_line_len,
+	size_t *max_line_len,
 	const char *network_charset,
 	const char *local_charset)
 {
@@ -2459,7 +2430,7 @@ buffer_to_local(
 				} while (inbytesleft > 0);
 
 				**&outbuf = '\0';
-				if (*max_line_len < (int) strlen(obuf) + 1) {
+				if (*max_line_len < strlen(obuf) + 1) {
 					*max_line_len = strlen(obuf) + 1;
 					*line = my_realloc(*line, *max_line_len);
 				}
@@ -2569,7 +2540,7 @@ buffer_to_ascii(
 void
 process_charsets(
 	char **line,
-	int *max_line_len,
+	size_t *max_line_len,
 	const char *network_charset,
 	const char *local_charset,
 	t_bool conv_tex2iso)
@@ -3394,7 +3365,15 @@ gnksa_split_from(
 		}
 	}
 
+	/*
+	 * if we allow <> as From: we must disallow <> as Mesage-ID,
+	 * see code in post.c:check_article_to_be_posted()
+	 */
+#if 0
+	if (!strchr(address, '@') && *address) /* check for From: without an @ but allow <> */
+#else
 	if (!strchr(address, '@')) /* check for From: without an @ */
+#endif /* 0 */
 		return GNKSA_ATSIGN_MISSING;
 
 	/* split successful */
@@ -3421,7 +3400,7 @@ gnksa_do_check_from(
 	decoded[0] = '\0';
 
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		wait_message(0, "From:=[%s]", from);
 #endif /* DEBUG */
 
@@ -3431,7 +3410,7 @@ gnksa_do_check_from(
 		return code;
 
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		wait_message(0, "address=[%s]", address);
 #endif /* DEBUG */
 
@@ -3445,7 +3424,7 @@ gnksa_do_check_from(
 		*addr_begin++ = '\0';
 
 #ifdef DEBUG
-		if (debug == 2)
+		if (debug & DEBUG_MISC)
 			wait_message(0, "FQDN=[%s]", addr_begin);
 #endif /* DEBUG */
 
@@ -3465,7 +3444,7 @@ gnksa_do_check_from(
 	}
 
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		wait_message(0, "realname=[%s]", realname);
 #endif /* DEBUG */
 
@@ -3477,9 +3456,9 @@ gnksa_do_check_from(
 		strcpy(realname, decoded);
 
 #ifdef DEBUG
-	if (debug == 2) {
+	if (debug & DEBUG_MISC) { /* TODO: dump to a file instead of wait_message() */
 		if (GNKSA_OK != code)
-			wait_message(3, "From:=[%s], GNKSA=[%d]", from, code);
+			wait_message(2, "From:=[%s], GNKSA=[%d]", from, code);
 		else
 			wait_message(0, "GNKSA=[%d]", code);
 	}
@@ -3575,88 +3554,95 @@ utf8_valid(
 			numc++;
 		} while ((d <<= 1) & 0x80);	/* get sequence length */
 
-		d = *c;
-		e = *(c + 1);
-
-		switch (numc) {
-			case 2:
-				/* out of range or sequences which would also fit into 1 byte */
-				if (d < 0xc2 || d > 0xdf)
-					illegal = TRUE;
-				break;
-
-			case 3:
-				f = *(c + 2);
-				/* out of range or sequences which would also fit into 2 bytes */
-				if (d < 0xe0 || d > 0xef || (d == 0xe0 && e < 0xa0))
-					illegal = TRUE;
-				/* U+D800 ... U+DFFF */
-				if (d == 0xed && e > 0x9f)
-					illegal = TRUE;
-				/* U+FDD0 ... U+FDEF */
-				if (d == 0xef && e == 0xb7 && (f >= 0x90 && f <= 0xaf))
-					illegal = TRUE;
-				/* U+FFFE, U+FFFF */
-				if (d == 0xef && e == 0xbf && (f == 0xbe || f == 0xbf))
-					illegal = TRUE;
-				break;
-
-			case 4:
-				f = *(c + 2);
-				g = *(c + 3);
-				/* out of range or sequences which would also fit into 3 bytes */
-				if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
-					illegal = TRUE;
-				/* largest current used sequence */
-				if ((d == 0xf4 && e > 0x8f) || d > 0xf4)
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
-				if (d == 0xf0 && (e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+4FFFE, U+4FFFF, U+5FFFE, U+5FFFF, U+6FFFE, U+6FFFF, U+7FFFE, U+7FFFF */
-				/* U+8FFFE, U+8FFFF, U+9FFFE, U+9FFFF, U+AFFFE, U+AFFFF, U+BFFFE, U+BFFFF */
-				/* U+CFFFE, U+CFFFF, U+DFFFE, U+DFFFF, U+EFFFE, U+EFFFF, U+FFFFE, U+FFFFF */
-				if ((d == 0xf1 || d == 0xf2 || d == 0xf3) && (e == 0x8f || e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				/* Unicode 3.1 noncharacters */
-				/* U+10FFFE, U+10FFFF */
-				if (d == 0xf4 && e == 0x8f && f == 0xbf && (g == 0xbe || g == 0xbf))
-					illegal = TRUE;
-				break;
-
-#	if 0	/* currently not used, see also check above */
-			case 5:
-				/* out of range or sequences which would also fit into 4 bytes */
-				if (d < 0xf8 || d > 0xfb || (d == 0xf8 && e < 0x88))
-					illegal = TRUE;
-				break;
-
-			case 6:
-				/* out of range or sequences which would also fit into 5 bytes */
-				if (d < 0xfc || d > 0xfd || (d == 0xfc && e < 0x84))
-					illegal = TRUE;
-				break;
-#	endif /* 0 */
-
-			default:
-				/*
-				 * with the check for plain US-ASCII above all other sequence
-				 * length are illegal.
-				 */
+		d = 1;
+		while (d < numc) {
+			if (*(c + d) == '\0' || *(c + d) == '\n')
 				illegal = TRUE;
-				break;
+			d--;
 		}
 
 		if (!illegal) {
-			for (d = 1; d < numc; d++) {
-				e = *(c + d);
-				if (e < 0x80 || e > 0xbf || *(c + d) == '\0' || *(c + d) == '\n')
+			d = *c;
+			e = *(c + 1);
+
+			switch (numc) {
+				case 2:
+					/* out of range or sequences which would also fit into 1 byte */
+					if (d < 0xc2 || d > 0xdf)
+						illegal = TRUE;
+					break;
+
+				case 3:
+					f = *(c + 2);
+					/* out of range or sequences which would also fit into 2 bytes */
+					if (d < 0xe0 || d > 0xef || (d == 0xe0 && e < 0xa0))
+						illegal = TRUE;
+					/* U+D800 ... U+DFFF */
+					if (d == 0xed && e > 0x9f)
+						illegal = TRUE;
+					/* U+FDD0 ... U+FDEF */
+					if (d == 0xef && e == 0xb7 && (f >= 0x90 && f <= 0xaf))
+						illegal = TRUE;
+					/* U+FFFE, U+FFFF */
+					if (d == 0xef && e == 0xbf && (f == 0xbe || f == 0xbf))
+						illegal = TRUE;
+					break;
+
+				case 4:
+					f = *(c + 2);
+					g = *(c + 3);
+					/* out of range or sequences which would also fit into 3 bytes */
+					if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
+						illegal = TRUE;
+					/* largest current used sequence */
+					if ((d == 0xf4 && e > 0x8f) || d > 0xf4)
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
+					if (d == 0xf0 && (e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+4FFFE, U+4FFFF, U+5FFFE, U+5FFFF, U+6FFFE, U+6FFFF, U+7FFFE, U+7FFFF */
+					/* U+8FFFE, U+8FFFF, U+9FFFE, U+9FFFF, U+AFFFE, U+AFFFF, U+BFFFE, U+BFFFF */
+					/* U+CFFFE, U+CFFFF, U+DFFFE, U+DFFFF, U+EFFFE, U+EFFFF, U+FFFFE, U+FFFFF */
+					if ((d == 0xf1 || d == 0xf2 || d == 0xf3) && (e == 0x8f || e == 0x9f || e == 0xaf || e == 0xbf) && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					/* Unicode 3.1 noncharacters */
+					/* U+10FFFE, U+10FFFF */
+					if (d == 0xf4 && e == 0x8f && f == 0xbf && (g == 0xbe || g == 0xbf))
+						illegal = TRUE;
+					break;
+
+#	if 0	/* currently not used, see also check above */
+				case 5:
+					/* out of range or sequences which would also fit into 4 bytes */
+					if (d < 0xf8 || d > 0xfb || (d == 0xf8 && e < 0x88))
+						illegal = TRUE;
+					break;
+
+				case 6:
+					/* out of range or sequences which would also fit into 5 bytes */
+					if (d < 0xfc || d > 0xfd || (d == 0xfc && e < 0x84))
+						illegal = TRUE;
+					break;
+#	endif /* 0 */
+
+				default:
+					/*
+					 * with the check for plain US-ASCII above all other sequence
+					 * length are illegal.
+					 */
 					illegal = TRUE;
+					break;
 			}
 		}
 
+		for (d = 1; d < numc && !illegal; d++) {
+			e = *(c + d);
+			if (e < 0x80 || e > 0xbf || e == '\0' || e == '\n')
+				illegal = TRUE;
+		}
+
 		if (!illegal)
 			c += numc; /* skip over valid sequence */
 		else {
@@ -3753,6 +3739,8 @@ tin_version_info(
 		wlines++;
 #	endif /* TIN_LIBS */
 #endif /* TIN_LD */
+	fprintf(fp, "\tPCRE     = \"%s\"\n", pcre_version());
+	wlines++;
 
 	fprintf(fp, "Characteristics:\n\t"
 /* TODO: complete list and do some useful grouping */
@@ -3963,24 +3951,26 @@ stat_article(
 	long art,
 	const char *group_path)
 {
-	char buf[NNTP_STRLEN];
 	struct t_group currgrp;
 
 	currgrp = CURR_GROUP;
 
 #	ifdef NNTP_ABLE
 	if (read_news_via_nntp && currgrp.type == GROUP_TYPE_NEWS) {
+		char buf[NNTP_STRLEN];
+
 		snprintf(buf, sizeof(buf), "STAT %ld", art);
 		return (nntp_command(buf, OK_NOTEXT, NULL, 0) != NULL);
 	} else
 #	endif /* NNTP_ABLE */
 	{
+		char filename[PATH_LEN];
 		struct stat sb;
 
-		joinpath(buf, currgrp.spooldir, group_path);
-		snprintf(&buf[strlen(buf)], sizeof(buf), "/%ld", art);
+		joinpath(filename, sizeof(filename), currgrp.spooldir, group_path);
+		snprintf(&filename[strlen(filename)], sizeof(filename), "/%ld", art);
 
-		return (stat(buf, &sb) != -1);
+		return (stat(filename, &sb) != -1);
 	}
 }
 #endif /* 0 */
