@@ -3,7 +3,7 @@
  *  Module    : nntplib.c
  *  Author    : S. Barber & I. Lea
  *  Created   : 1991-01-12
- *  Updated   : 2004-03-13
+ *  Updated   : 2008-04-24
  *  Notes     : NNTP client routines taken from clientlib.c 1.5.11 (1991-02-10)
  *  Copyright : (c) Copyright 1991-99 by Stan Barber & Iain Lea
  *              Permission is hereby granted to copy, reproduce, redistribute
@@ -25,12 +25,7 @@
 #	include "tnntp.h"
 #endif /* !TNNTP_H */
 
-#ifdef VMS /* M.St. 15.01.98 */
-#	undef VMS
-#endif /* VMS */
-
 char *nntp_server = NULL;
-constext *xover_cmd = NULL;
 #ifdef NO_POSTING
 	t_bool can_post = FALSE;
 #else
@@ -39,6 +34,8 @@ constext *xover_cmd = NULL;
 
 /* Flag to show whether tin did reconnect in last get_server() */
 t_bool reconnected_in_last_get_server = FALSE;
+/* Flag used in LIST ACVTIVE loop */
+t_bool did_reconnect = FALSE;
 
 static TCP *nntp_rd_fp = NULL;
 static TCP *nntp_wr_fp = NULL;
@@ -47,11 +44,7 @@ static TCP *nntp_wr_fp = NULL;
 	/* Copy of last NNTP command sent, so we can retry it if needed */
 	static char last_put[NNTP_STRLEN];
 	static constext *xover_cmds = "XOVER";
-#	if 0 /* currently not used */
-	static constext *xhdr_cmd = NULL;
 	static constext *xhdr_cmds = "XHDR";
-#	endif /* 0 */
-	static t_bool have_list_extensions = FALSE;
 	/* Set so we don't reconnect just to QUIT */
 	static t_bool quitting = FALSE;
 #endif /* NNTP_ABLE */
@@ -60,10 +53,12 @@ static TCP *nntp_wr_fp = NULL;
  * local prototypes
  */
 #ifdef NNTP_ABLE
+	static int mode_reader(t_bool *sec);
 	static int reconnect(int retry);
-	static int server_init(char *machine, const char *cservice, int port, char *text, size_t mlen);
-	static void check_extensions(void);
+	static int server_init(char *machine, const char *cservice, unsigned short port, char *text, size_t mlen);
+	static int check_extensions(t_bool *sec);
 	static void close_server(void);
+	static void list_motd(void);
 #	ifdef INET6
 		static int get_tcp6_socket(char *machine, unsigned short port);
 #	else
@@ -124,17 +119,20 @@ getserverbyfile(
 	static char *old_env = NULL;
 #	endif /* !HAVE_SETENV && HAVE_PUTENV */
 #endif /* NNTP_ABLE */
+
 	if (!read_news_via_nntp) {
 		STRCPY(buf, "local");	/* what if a server is named "local"? */
 		return buf;
 	}
+
 	if (read_saved_news) {
 		STRCPY(buf, "reading saved news");
 		return buf;
 	}
+
 #ifdef NNTP_ABLE
 	if (cmdline_nntpserver[0] != '\0') {
-		get_nntpserver(buf, cmdline_nntpserver);
+		get_nntpserver(buf, sizeof(buf), cmdline_nntpserver);
 #	ifdef HAVE_SETENV
 		setenv("NNTPSERVER", buf, 1);
 #	else
@@ -150,7 +148,7 @@ getserverbyfile(
 	}
 
 	if ((cp = getenv("NNTPSERVER")) != NULL) {
-		get_nntpserver(buf, cp);
+		get_nntpserver(buf, sizeof(buf), cp);
 		return buf;
 	}
 
@@ -173,18 +171,11 @@ getserverbyfile(
 		(void) fclose(fp);
 
 		if (cp != NULL) {
-			get_nntpserver(buf, cp);
+			get_nntpserver(buf, sizeof(buf), cp);
 			return buf;
 		}
 	}
 
-#	ifdef USE_INN_NNTPLIB
-	if ((cp = GetConfigValue(_CONF_SERVER)) != NULL) {
-		(void) STRCPY(buf, cp);
-		return buf;
-	}
-#	endif /* USE_INN_NNTPLIB */
-
 #	ifdef NNTP_DEFAULT_SERVER
 	if (*(NNTP_DEFAULT_SERVER))
 		return strcpy(buf, NNTP_DEFAULT_SERVER);
@@ -215,7 +206,7 @@ static int
 server_init(
 	char *machine,
 	const char *cservice,	/* usually a literal */
-	int port,
+	unsigned short port,
 	char *text,
 	size_t mlen)
 {
@@ -223,9 +214,7 @@ server_init(
 	char temp[256];
 	char *service = strncpy(temp, cservice, sizeof(temp) - 1); /* ...calls non-const funcs */
 #	endif /* !INET6 */
-#	ifndef VMS
 	int sockt_rd, sockt_wr;
-#	endif /* !VMS */
 
 #	ifdef DECNET
 	char *cp;
@@ -239,16 +228,15 @@ server_init(
 		sockt_rd = get_tcp_socket(machine, service, port);
 #	else
 #		ifdef INET6
-	sockt_rd = get_tcp6_socket(machine, (unsigned short) port);
+	sockt_rd = get_tcp6_socket(machine, port);
 #		else
-	sockt_rd = get_tcp_socket(machine, service, (unsigned short) port);
+	sockt_rd = get_tcp_socket(machine, service, port);
 #		endif /* INET6 */
 #	endif /* DECNET */
 
 	if (sockt_rd < 0)
 		return sockt_rd;
 
-#	ifndef VMS
 	/*
 	 * Now we'll make file pointers (i.e., buffered I/O) out of
 	 * the socket file descriptor. Note that we can't just
@@ -266,23 +254,20 @@ server_init(
 		return -errno;
 	}
 
-#		ifdef TLI /* Transport Level Interface */
+#	ifdef TLI /* Transport Level Interface */
 	if (t_sync(sockt_rd) < 0) {	/* Sync up new fd with TLI */
 		t_error("server_init: t_sync()");
 		nntp_rd_fp = NULL;
 		return -EPROTO;
 	}
-#		else
+#	else
 	if ((nntp_wr_fp = (TCP *) s_fdopen(sockt_wr, "w")) == NULL) {
 		perror("server_init: fdopen() #2");
 		nntp_rd_fp = NULL;
 		return -errno;
 	}
-#		endif /* TLI */
+#	endif /* TLI */
 
-#	else
-	sockt_wr = sockt_rd;
-#	endif /* !VMS */
 
 	last_put[0] = '\0';		/* no retries in get_respcode */
 	/*
@@ -335,7 +320,7 @@ get_tcp_socket(
 	else
 		strcpy(device, "/dev/tcp");
 
-	if ((s = t_open(device, O_RDWR, (struct t_info *) 0)) < 0){
+	if ((s = t_open(device, O_RDWR, (struct t_info *) 0)) < 0) {
 		t_error(txt_error_topen);
 		return -EPROTO;
 	}
@@ -362,14 +347,14 @@ get_tcp_socket(
 			t_close(s);
 			return -EHOSTUNREACH;
 		}
-		memcpy((char *) &sock_in.sin_addr, hp->h_addr, hp->h_length);
+		memcpy((char *) &sock_in.sin_addr, hp->h_addr_list[0], hp->h_length);
 	}
 
 	/*
 	 * Allocate a t_call structure and initialize it.
 	 * Let t_alloc() initialize the addr structure of the t_call structure.
 	 */
-	if ((callptr = (struct t_call *) t_alloc(s, T_CALL, T_ADDR)) == NULL){
+	if ((callptr = (struct t_call *) t_alloc(s, T_CALL, T_ADDR)) == NULL) {
 		t_error("t_alloc");
 		t_close(s);
 		return -EPROTO;
@@ -439,12 +424,12 @@ get_tcp_socket(
 #			endif /* HAVE_GETSERVBYNAME */
 
 	/* If not a raw ip address, try nameserver */
-	if (!isdigit((unsigned char) *machine) ||
+	if (!isdigit((unsigned char) *machine)
 #			ifdef HAVE_INET_ATON
-	    !inet_aton(machine, &defaddr)
+	    || !inet_aton(machine, &defaddr)
 #			else
 #				ifdef HAVE_INET_ADDR
-	    (long) (defaddr.s_addr = (long) inet_addr(machine)) == -1
+	    || (long) (defaddr.s_addr = (long) inet_addr(machine)) == -1
 #				endif /* HAVE_INET_ADDR */
 #			endif /* HAVE_INET_ATON */
 	    )
@@ -457,7 +442,7 @@ get_tcp_socket(
 #			ifdef h_addr
 		def.h_addr_list = alist;
 #			endif /* h_addr */
-		def.h_addr = (char *) &defaddr;
+		def.h_addr_list[0] = (char *) &defaddr;
 		def.h_length = sizeof(struct in_addr);
 		def.h_addrtype = AF_INET;
 		def.h_aliases = 0;
@@ -574,7 +559,7 @@ get_tcp_socket(
 
 	/* And then connect */
 
-	memcpy((char *) &sock_in.sin_addr, hp->h_addr, hp->h_length);
+	memcpy((char *) &sock_in.sin_addr, hp->h_addr_list[0], hp->h_length);
 
 	if (connect(s, (struct sockaddr *) &sock_in, sizeof(sock_in)) < 0) {
 		save_errno = errno;
@@ -612,7 +597,7 @@ get_tcp6_socket(
 {
 	char mymachine[MAXHOSTNAMELEN + 1];
 	char myport[12];
-	int c = -1, s = -1, err = -1;
+	int s = -1, err = -1;
 	struct addrinfo hints, *res, *res0;
 
 	snprintf(mymachine, sizeof(mymachine), "%s", machine);
@@ -629,8 +614,8 @@ get_tcp6_socket(
 #		endif /* PF_UNSPEC */
 #	endif /* AF_UNSPEC */
 	memset(&hints, 0, sizeof(hints));
-/*	hints.ai_flags    = AI_CANONNAME; */
-	hints.ai_family   = ADDRFAM;
+/*	hints.ai_flags = AI_CANONNAME; */
+	hints.ai_family = ADDRFAM;
 	hints.ai_socktype = SOCK_STREAM;
 	res = (struct addrinfo *) 0;
 	res0 = (struct addrinfo *) 0;
@@ -643,7 +628,7 @@ get_tcp6_socket(
 	for (res = res0; res; res = res->ai_next) {
 		if ((s = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) < 0)
 			continue;
-		if ((c = connect(s, res->ai_addr, res->ai_addrlen)) < 0)
+		if (connect(s, res->ai_addr, res->ai_addrlen) != 0)
 			close(s);
 		else {
 			err = 0;
@@ -702,6 +687,7 @@ get_dnet_socket(
 			sdn.sdn_add.a_addr[0] = node % 256;
 			sdn.sdn_add.a_addr[1] = node / 256;
 			break;
+
 		default:
 			if ((np = getnodebyname(machine)) == NULL) {
 				my_fprintf(stderr, _(txt_gethostbyname), "", machine);
@@ -747,16 +733,18 @@ get_dnet_socket(
 /*
  * u_put_server -- send data to the server. Do not flush output.
  */
-#ifndef VMS
-#	ifdef NNTP_ABLE
+#ifdef NNTP_ABLE
 void
 u_put_server(
 	const char *string)
 {
 	s_puts(string, nntp_wr_fp);
-#		ifdef DEBUG
-	debug_nntp(">>>", string);
-#		endif /* DEBUG */
+#	ifdef DEBUG
+	if (debug & DEBUG_NNTP) {
+		if (strcmp(string, "\r\n"))
+			debug_print_file("NNTP", ">>> %s", string);
+	}
+#	endif /* DEBUG */
 }
 
 
@@ -767,13 +755,12 @@ u_put_server(
  * Returns: Nothing.
  *
  *	Side effects: Talks to the server.
- *			Closes connection if things are not right.
+ *	              Closes connection if things are not right.
  *
  * Note: This routine flushes the buffer each time it is called. For large
- *	      transmissions (i.e., posting news) don't use it. Instead, do the
- *	      fprintf's yourself, and then a final fflush.
+ *       transmissions (i.e., posting news) don't use it. Instead, do the
+ *       fprintf's yourself, and then a final fflush.
  *       Only cache commands, don't cache data transmissions.
- *
  */
 void
 put_server(
@@ -783,14 +770,22 @@ put_server(
 		DEBUG_IO((stderr, "put_server(%s)\n", string));
 		s_puts(string, nntp_wr_fp);
 		s_puts("\r\n", nntp_wr_fp);
-#		ifdef DEBUG
-		debug_nntp(">>>", string);
-#		endif /* DEBUG */
+#	ifdef DEBUG
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", ">>> %s", string);
+#	endif /* DEBUG */
 		/*
 		 * remember the last command we wrote to be able to resend it after a
 		 * reconnect. reconnection is handled by get_server()
+		 *
+		 * don't cache "LIST ACTIVE something" as we would need to
+		 * resend all of them but we remeber just the last one. we cache
+		 * "LIST" instead, this will slow down things, but that's ok on
+		 * reconnect.
 		 */
-		if (last_put != string)
+		if (!strncmp(string, "LIST ACTIVE ", 12))
+			STRCPY(last_put, "LIST");
+		else
 			STRCPY(last_put, string);
 	}
 	(void) s_flush(nntp_wr_fp);
@@ -826,7 +821,7 @@ reconnect(
 	 * Exit tin if the user says no to reconnect. The exit code stops tin from trying
 	 * to disconnect again - the connection is already dead
 	 */
-	if (!tinrc.auto_reconnect && prompt_yn(cLINES, _(txt_reconnect_to_news_server), TRUE) != 1)
+	if (!tinrc.auto_reconnect && prompt_yn(_(txt_reconnect_to_news_server), TRUE) != 1)
 		tin_done(NNTP_ERROR_EXIT);		/* user said no to reconnect */
 
 	clear_message();
@@ -842,13 +837,15 @@ reconnect(
 			snprintf(last_put, sizeof(last_put), "GROUP %s", curr_group->name);
 			put_server(last_put);
 			s_gets(last_put, NNTP_STRLEN, nntp_rd_fp);
-#		ifdef DEBUG
-			debug_nntp("<<<", last_put);
-#		endif /* DEBUG */
+#	ifdef DEBUG
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "<<< %s", last_put);
+#	endif /* DEBUG */
 			DEBUG_IO((stderr, _("Read (%s)\n"), last_put));
 		}
 		DEBUG_IO((stderr, _("Resend last command (%s)\n"), buf));
 		put_server(buf);
+		did_reconnect = TRUE;
 		return 0;
 	}
 
@@ -884,16 +881,19 @@ get_server(
 
 	/*
 	 * NULL socket reads indicates socket has closed. Try a few times more
+	 *
+	 * TODO: add a timeout (some servers do not close the connection but
+	 *       simply do not send any response data -> we need a timeout to
+	 *       leave the s_gets() in that case)
 	 */
 	while (nntp_rd_fp == NULL || s_gets(string, size, nntp_rd_fp) == NULL) {
-
 		if (quitting)						/* Don't bother to reconnect */
 			tin_done(NNTP_ERROR_EXIT);		/* And don't try to disconnect again! */
 
-#		ifdef DEBUG
+#	ifdef DEBUG
 		if (errno != 0 && errno != EINTR)	/* Will only confuse end users */
 			perror_message("get_server()");
-#		endif /* DEBUG */
+#	endif /* DEBUG */
 
 		/*
 		 * Reconnect only if command was not "QUIT" anyway (in which case a
@@ -902,6 +902,15 @@ get_server(
 		 * when user is quitting tin if tinrc.auto_reconnect is false.
 		 */
 		if (strncmp(last_put, "QUIT", 4)) {
+			/*
+			 * Typhoon v2.1.1.363 colses the connection right after an unknown
+			 * command, (i.e. CAPABILITIES) so we avoid the reissue it on a
+			 * reconnect if it was the last command.
+			 */
+			if (!strncmp(last_put, "CAPABILITIES", 12)) {
+				strcpy(last_put, "MODE READER");
+				nntp_caps.type = BROKEN;
+			}
 			retry = reconnect(retry);		/* Will abort when out of tries */
 			reconnected_in_last_get_server = TRUE;
 		} else {
@@ -946,74 +955,327 @@ close_server(
 	s_end();
 	nntp_wr_fp = nntp_rd_fp = NULL;
 }
-#	endif /* NNTP_ABLE */
-#endif /* !VMS */
+#endif /* NNTP_ABLE */
 
 
 #ifdef NNTP_ABLE
 /*
- * Try and use LIST EXTENSIONS here. Get this list before issuing
- * other NNTP commands because the correct methods may be
- * mentioned in the list of extensions.
- * Possible extensions include:
- * - HDR & LIST HEADERS
- * - OVER [MSGID] & LIST OVERVIEW.FMT
- * - LISTGROUP
- * - STARTTLS
+ * Try and use CAPABILITIES/LIST EXTENSIONS here. Get this list before
+ * issuing other NNTP commands because the correct methods may be mentioned
+ * in the list of extensions.
  *
- * Sets up: have_list_extensions, xover_cmd, (xhdr_cmd)
+ * Sets up: t_capabilities nntp_caps
  */
-static void
+static int
 check_extensions(
-	void)
+	t_bool *sec)
 {
-	FILE *fp;
 	char *ptr;
+	int ret = 0;
+#	if 1 /* "CAPABILITIES" replaces "LIST EXTENSIONS" */
+	char buf[NNTP_STRLEN];
+	char *d;
 	int i;
 
-	if ((fp = nntp_command("LIST EXTENSIONS", OK_EXTENSIONS, NULL, 0)) == NULL)
-		return;
-
-	have_list_extensions = TRUE;
-
-	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
-		/*
-		 * Check for (X)OVER
-		 * XOVER should not be listed in EXTENSIONS (but sometimes is)
-		 * checking for it if OVER is not found does no harm.
-		 */
-		if (!xover_cmd) {
-			for (i = 1; i >= 0; i--) {
-				if (strcmp(ptr, &xover_cmds[i]) == 0) {
-					xover_cmd = &xover_cmds[i];
-					break;
+	buf[0] = '\0';
+	i = new_nntp_command("CAPABILITIES", INF_CAPABILITIES, buf, sizeof(buf));
+	switch (i) {
+		case INF_CAPABILITIES:
+			nntp_caps.type = CAPABILITIES;
+			while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
+#		ifdef DEBUG
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< %s", ptr);
+#		endif /* DEBUG */
+				/* look for version number(s) */
+				if (!nntp_caps.version && nntp_caps.type == CAPABILITIES) {
+					if (!strncasecmp(ptr, "VERSION", 7)) {
+						d = ptr + 7;
+						d = strpbrk(d, " \t");
+						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
+							d++;
+							nntp_caps.version = MAX(nntp_caps.version, (unsigned int) atoi(d));
+							d = strpbrk(d, " \t");
+						}
+					}
 				}
-			}
-		}
-#		if 0 /* currently not used */
-		/*
-		 * Check for (X)HDR
-		 * XHDR should not be listed in EXTENSIONS (but sometimes is)
-		 * checking for it if HDR is not found does no harm.
-		 */
-		if (!xhdr_cmd) {
-			for (i = 1; i >= 0; i--) {
-				if (strcmp(ptr, &xhdr_cmds[i]) == 0) {
-					xhdr_cmd = &xhdr_cmds[i];
-					break;
-				}
-			}
-		}
+				/* we currently only support CAPABILITIES VERSION 2 */
+				if (nntp_caps.version == 2) {
+					/*
+					 * check for LIST variants
+					 */
+					if (!strncasecmp(ptr, "LIST", 4)) {
+						d = ptr + 4;
+						d = strpbrk(d, " \t");
+						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
+							d++;
+							if (!strncasecmp(d, "ACTIVE.TIMES", 12))
+								nntp_caps.list_active_times = TRUE;
+							else if (!strncasecmp(d, "ACTIVE", 6))
+								nntp_caps.list_active = TRUE;
+							else if (!strncasecmp(d, "DISTRIB.PATS", 12))
+								nntp_caps.list_distrib_pats = TRUE;
+							else if (!strncasecmp(d, "DISTRIBUTIONS", 13)) /* "private" extension, RFC 2980 */
+								nntp_caps.list_distributions = TRUE;
+							else if (!strncasecmp(d, "HEADERS", 7))
+								nntp_caps.list_headers = TRUE; /* HDR requires LIST HEADERS, but not vice versa */
+							else if (!strncasecmp(d, "NEWSGROUPS", 10))
+								nntp_caps.list_newsgroups = TRUE;
+							else if (!strncasecmp(d, "OVERVIEW.FMT", 12)) /* OVER requires OVERVIEW.FMT, but not vice versa */
+								nntp_caps.list_overview_fmt = TRUE;
+							else if (!strncasecmp(d, "MOTD", 4)) /* "private" extension */
+								nntp_caps.list_motd = TRUE;
+							else if (!strncasecmp(d, "SUBSCRIPTIONS", 13)) /* "private" extension, RFC 2980 */
+								nntp_caps.list_subscriptions = TRUE;
+							else if (!strncasecmp(d, "MODERATORS", 10)) /* "private" extension */
+								nntp_caps.list_moderators = TRUE;
+							d = strpbrk(d, " \t");
+						}
+					} else if (!strncasecmp(ptr, "IMPLEMENTATION", 14)) {
+						nntp_caps.implementation = my_strdup(ptr + 14);
+						str_trim(nntp_caps.implementation);
+					} else if (!strcasecmp(ptr, "MODE-READER")) {
+						if (!nntp_caps.reader)
+							nntp_caps.mode_reader = TRUE;
+					} else if (!strcasecmp(ptr, "READER")) {
+						nntp_caps.reader = TRUE;
+						nntp_caps.mode_reader = FALSE;
+					} else if (!strcasecmp(ptr, "POST"))
+						nntp_caps.post = TRUE;
+					else if (!strcasecmp(ptr, "NEWNEWS"))
+						nntp_caps.newnews = TRUE;
+					else if (!strcasecmp(ptr, "XPAT")) /* extension, RFC 2980 */
+						nntp_caps.xpat = TRUE;
+					else if (!strcasecmp(ptr, "STARTTLS"))
+						nntp_caps.starttls = TRUE;
+					/*
+					 * NOTE: if we saw OVER, LIST OVERVIEW.FMT _must_ be implemented
+					 */
+					else if (!strncasecmp(ptr, &xover_cmds[1], strlen(&xover_cmds[1]))) {
+						nntp_caps.list_overview_fmt = nntp_caps.over = TRUE;
+						nntp_caps.over_cmd = &xover_cmds[1];
+						d = ptr + strlen(&xover_cmds[1]);
+						d = strpbrk(d, " \t");
+						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
+							d++;
+							if (!strcasecmp(d, "MSGID"))
+								nntp_caps.over_msgid = TRUE;
+							d = strpbrk(d, " \t");
+						}
+					}
+					/*
+					 * NOTE: if we saw HDR, LIST HEADERS _must_ be implemented
+					 */
+					else if (!strncasecmp(ptr, &xhdr_cmds[1], strlen(&xhdr_cmds[1]))) {
+						nntp_caps.hdr_cmd = &xhdr_cmds[1];
+						nntp_caps.list_headers = nntp_caps.hdr = TRUE;
+					} else if (!strncasecmp(ptr, "AUTHINFO", 8)) {
+						d = ptr + 8;
+						d = strpbrk(d, " \t");
+						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
+							d++;
+							if (!strncasecmp(d, "USER", 4))
+								nntp_caps.authinfo_user = TRUE;
+							if (!strncasecmp(d, "SASL", 4))
+								nntp_caps.authinfo_sasl = TRUE;
+							d = strpbrk(d, " \t");
+						}
+					} else if (!strncasecmp(ptr, "SASL", 4)) {
+						d = ptr + 4;
+						d = strpbrk(d, " \t");
+						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
+							d++;
+							if (!strncasecmp(d, "CRAM-MD5", 8))
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_cram_md5 = TRUE;
+							if (!strncasecmp(d, "DIGEST-MD5", 10))
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_digest_md5 = TRUE;
+							if (!strncasecmp(d, "PLAIN", 5))
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_plain = TRUE;
+							if (!strncasecmp(d, "GSSAPI", 6))
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_gssapi = TRUE;
+							if (!strncasecmp(d, "EXTERNAL", 8))
+								nntp_caps.authinfo_sasl = nntp_caps.sasl_external = TRUE;
+						}
+					}
+#		if 0 /* we don't need these */
+					else if (!strcasecmp(ptr, "IHAVE"))
+						nntp_caps.ihave = TRUE;
+					else if (!strcasecmp(ptr, "STREAMING"))
+						nntp_caps.streaming = TRUE;
 #		endif /* 0 */
-		 /*
-		  * additional checks for
-		  * - LISTGROUP
-		  * - LIST OVERVIEW.FMT
-		  * - LIST HEADERS
-		  * go here whenever they are needed
-		  */
+				} else
+					nntp_caps.type = NONE;
+			}
+			break;
+
+		/*
+		 * XanaNewz 2 Server Version 2.0.0.3 doesn't know CAPABILITIES
+		 * but respondes with 400 _without_ closing the connection. If
+		 * you use tin on a XanaNewz 2 Server comment out the following
+		 * case.
+		 */
+		case ERR_GOODBYE:
+			ret = i;
+			error_message(buf);
+			break;
+
+		default:
+			break;
 	}
-	return;
+#		ifdef DEBUG
+	debug_print_nntp_extensions();
+#		endif /* DEBUG */
+	if ((ret != ERR_GOODBYE) && !*sec && !nntp_caps.reader) {
+		if (nntp_caps.type == CAPABILITIES && !nntp_caps.mode_reader) {
+			if (!nntp_caps.post) { /* as a last resort check if post was mentioned */
+				error_message(_("CAPABILITIES did not announce any of READER, MODE-READER, POST")); /* TODO: -> lang.c */
+				return -1; /* give up */
+			}
+		}
+		if ((ret = mode_reader(&*sec)) != 0)
+			return ret;
+		else if (nntp_caps.type == CAPABILITIES) /* 2nd pass */
+			ret = check_extensions(&*sec);
+	}
+
+#	else
+	/*
+	 * TODO: the following code can go away in tin 2.0 as it's
+	 *       obsolete by RFC 3977
+	 *
+	 * "LIST EXTENSIONS" is somewhat troublesome as there are a lot
+	 * of broken implementations out there and it is a multiline response
+	 */
+	if (nntp_caps.type == NONE) {
+		char buf[NNTP_STRLEN];
+		int i;
+
+		buf[0] = '\0';
+		i = new_nntp_command("LIST EXTENSIONS", OK_EXTENSIONS, buf, sizeof(buf));
+		switch (i) {
+			case 215:	/* Netscape-Collabra/3.52 (badly broken); NetWare-News-Server/5.1 */
+#		ifdef DEBUG
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "LIST EXTENSIONS skipping data");
+#		endif /* DEBUG */
+				while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL)
+					;
+				break;
+
+			case OK_EXTENSIONS:	/* as defined draft-ietf-nntpext-base-27.txt */
+			case 205:	/* M$ Exchange 5.5 */
+				nntp_caps.type = LIST_EXTENSIONS;
+				while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
+					if (nntp_caps.type == LIST_EXTENSIONS) {
+#		ifdef DEBUG
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "<<< %s", ptr);
+#		endif /* DEBUG */
+						/*
+						 * some servers (e.g. Hamster 1.3) have leading spaces in
+						 * the extension list
+						 */
+						str_trim(ptr);
+						/*
+						 * Check for (X)OVER
+						 * XOVER should not be listed in EXTENSIONS (but sometimes
+						 * is) checking for it if OVER is not found does no harm.
+						 */
+						if (!nntp_caps.over_cmd) {
+							for (i = 1; i >= 0; i--) {
+								if (strcasecmp(ptr, &xover_cmds[i]) == 0) {
+									nntp_caps.over_cmd = &xover_cmds[i];
+									break;
+								}
+							}
+						}
+					}
+				}
+				/*
+				 * as the server did support LIST EXTENSIONS it's likely that it
+				 * also can do LIST MOTD (we don't bother to parse the LIST
+				 * EXTENSIONS output for MOTD as it never was standardized;
+				 * draft-ietf-nntpext-base-24.txt only described OVER, HDR and
+				 * LISTGROUP).
+				 */
+				nntp_caps.list_motd = TRUE;
+				break;
+
+			case ERR_GOODBYE:
+				ret = i;
+				error_message(buf);
+				break;
+
+			default:
+				break;
+		}
+#		ifdef DEBUG
+		debug_print_nntp_extensions();
+#		endif /* DEBUG */
+		if ((ret != ERR_GOODBYE) && !*sec)
+			ret = mode_reader(&*sec);
+	}
+#	endif /* 1 */
+	return ret;
+}
+
+
+/*
+ * Switch INN into NNRP mode with 'mode reader'
+ */
+static int
+mode_reader(
+	t_bool *sec)
+{
+	int ret = 0;
+
+	if (!nntp_caps.reader) {
+		char line[NNTP_STRLEN];
+#ifdef DEBUG
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "mode_reader() MODE READER");
+#endif /* DEBUG */
+		DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
+		put_server("MODE READER");
+
+		/*
+		 * According to the latest NNTP draft (May 2005), MODE READER may only
+		 * return the following response codes:
+		 *
+		 *   200 (OK_CANPOST)     Hello, you can post
+		 *   201 (OK_NOPOST)      Hello, you can't post
+		 *   502 (ERR_ACCESS)     Service unavailable
+		 *
+		 * However, there may be old servers out there that do not implement this
+		 * command and therefore return ERR_COMMAND (500). Unfortunately there
+		 * are some new servers out there (i.e. INN 2.4.0 (20020220 prerelease)
+		 * which do return ERR_COMMAND if they are feed only servers.
+		 */
+
+		switch ((ret = get_respcode(line, sizeof(line)))) {
+			case OK_CANPOST:
+				can_post = TRUE && !force_no_post;
+				*sec = TRUE;
+				ret = 0;
+				break;
+
+			case OK_NOPOST:
+				can_post = FALSE;
+				*sec = TRUE;
+				ret = 0;
+				break;
+
+			case ERR_GOODBYE:
+			case ERR_ACCESS:
+				error_message(line);
+				return ret;
+
+			case ERR_COMMAND:
+			default:
+				break;
+		}
+	}
+	return ret;
 }
 #endif /* NNTP_ABLE */
 
@@ -1041,7 +1303,8 @@ nntp_open(
 		return 0;
 
 #	ifdef DEBUG
-	debug_nntp("nntp_open", "BEGIN");
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_open() BEGIN");
 #	endif /* DEBUG */
 
 	if (nntp_server == NULL) {
@@ -1058,7 +1321,8 @@ nntp_open(
 	}
 
 #	ifdef DEBUG
-	debug_nntp("nntp_open", nntp_server);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_open() %s", nntp_server);
 #	endif /* DEBUG */
 
 	ret = server_init(nntp_server, NNTP_TCP_NAME, nntp_tcp_port, line, sizeof(line));
@@ -1068,7 +1332,8 @@ nntp_open(
 		my_fputc('\n', stdout);
 
 #	ifdef DEBUG
-	debug_nntp("nntp_open", line);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_open() %s", line);
 #	endif /* DEBUG */
 
 	switch (ret) {
@@ -1079,8 +1344,6 @@ nntp_open(
 		 * According to the ietf-nntp mailinglist:
 		 *   200 you may (try to) do anything
 		 *   201 you may not POST
-		 *  (202 you may not IHAVE)
-		 *  (203 you may not do EITHER)
 		 *   All unrecognised 200 series codes should be assumed as success.
 		 *   All unrecognised 300 series codes should be assumed as notice to continue.
 		 *   All unrecognised 400 series codes should be assumed as temporary error.
@@ -1088,12 +1351,10 @@ nntp_open(
 		 */
 
 		case OK_CANPOST:
-/*		case OK_NOIHAVE: */
 			can_post = TRUE && !force_no_post;
 			break;
 
 		case OK_NOPOST:
-/*		case OK_NOPOSTIHAVE: */
 			can_post = FALSE;
 			break;
 
@@ -1118,64 +1379,23 @@ nntp_open(
 		STRCPY(bug_nntpserver1, linep);
 	}
 
-	/*
-	 * Switch INN into NNRP mode with 'mode reader'
-	 */
-
-#	ifdef DEBUG
-	debug_nntp("nntp_open", "mode reader");
-#	endif /* DEBUG */
-	DEBUG_IO((stderr, "nntp_command(MODE READER)\n"));
-	put_server("MODE READER");
-
-	/*
-	 * According to the latest NNTP draft (Jan 2002), MODE READER may only
-	 * return the following response codes:
-	 *
-	 *   200 (OK_CANPOST)     Hello, you can post
-	 *   201 (OK_NOPOST)      Hello, you can't post
-	 *  (202 (OK_NOIHAVE)     discussed on the itef mailinglist)
-	 *  (203 (OK_NOPOSTIHAVE) discussed on the itef mailinglist)
-	 *   400 (ERR_GOODBYE)    Service temporarily unavailable
-	 *   502 (ERR_ACCESS)     Service unavailable
-	 *
-	 * However, there may be old servers out there that do not implement this
-	 * command and therefore return ERR_COMMAND (500). Unfortunately there
-	 * are some new servers out there (i.e. INN 2.4.0 (20020220 prerelease)
-	 * which do return ERR_COMMAND if they are feed only servers.
-	 */
-
-	switch ((ret = get_respcode(line, sizeof(line)))) {
-		case OK_CANPOST:
-/*		case OK_NOIHAVE: */
-			can_post = TRUE && !force_no_post;
-			sec = TRUE;
-			break;
-
-		case OK_NOPOST:
-/*		case OK_NOPOSTIHAVE: */
-			can_post = FALSE;
-			sec = TRUE;
-			break;
-
-		case ERR_GOODBYE:
-		case ERR_ACCESS:
-			error_message(line);
-			return ret;
-
-		case ERR_COMMAND:
-		default:
-			break;
-
-	}
-
 	/*
 	 * Find out which NNTP extensions are available
-	 * TODO: The authentication method required may be mentioned in the list of
-	 *       extensions. (For details about authentication methods, see
-	 *       draft-newman-nntpext-auth-01.txt).
+	 * - Typhoon v2.1.1.363 closes the connection after an unknown command
+	 *   (i.e. CAPABILITIES) but as we are not allowed to cache CAPABILITIES
+	 *   we reissue the command on reconnect. To prevent a loop we catch this
+	 *   case.
+     *
+	 * TODO: The authentication method required may be mentioned in the list
+	 *       of extensions. (For details about authentication methods, see
+	 *       RFC 4643).
 	 */
-	check_extensions();
+	if (nntp_caps.type != BROKEN) {
+		if ((ret = check_extensions(&sec)))
+			return ret; /* required "MODE READER/READER" failed, exit */
+		if (nntp_caps.type == CAPABILITIES && !nntp_caps.post)
+			can_post = FALSE;
+	}
 
 	/*
 	 * If the user wants us to authenticate on connection startup, do it now.
@@ -1186,33 +1406,12 @@ nntp_open(
 	 */
 	if (force_auth_on_conn_open) {
 #	ifdef DEBUG
-		debug_nntp("nntp_open", "authenticate");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "nntp_open() authenticate()");
 #	endif /* DEBUG */
 		authenticate(nntp_server, userid, TRUE);
-		put_server("MODE READER");
-		switch ((ret = get_respcode(line, sizeof(line)))) {
-			case OK_CANPOST:
-/*			case OK_NOIHAVE: */
-				can_post = TRUE && !force_no_post;
-				sec = TRUE;
-				break;
-
-			case OK_NOPOST:
-/*			case OK_NOPOSTIHAVE: */
-				can_post = FALSE;
-				sec = TRUE;
-				break;
-
-			case ERR_GOODBYE:
-			case ERR_ACCESS:
-				error_message(line);
-				return ret;
-
-			case ERR_COMMAND:	/* Uh-oh ... now we don't know if posting */
-			default:				/* is allowed or not ... so use last 200 */
-				break;			/* or 201 response to decide. */
-
-		}
+		if ((ret = mode_reader(&sec)))
+			return ret; /* "MODE READER" failed, exit */
 	}
 
 	if (!is_reconnect) {
@@ -1261,35 +1460,90 @@ nntp_open(
 	 * (successor of XOVER as of latest NNTP Draft (Jan 2002)
 	 * We have to check that we _don't_ get an ERR_COMMAND
 	 */
-	if (!have_list_extensions) {
-		for (i = 0; i < 2; i++) {
-			if (!nntp_command(&xover_cmds[i], ERR_COMMAND, NULL, 0)) {
-				xover_cmd = &xover_cmds[i];
-				break;
+	if (nntp_caps.type == NONE) {
+		int j = 0;
+
+		for (i = 0; i < 2 && j >= 0; i++) {
+			j = new_nntp_command(&xover_cmds[i], ERR_NCING, line, sizeof(line));
+			switch (j) {
+				case ERR_COMMAND:
+					break;
+
+				case 224:	/* unexpected multiline ok, e.g.: Synchronet 3.13 NNTP Service 1.92 */
+					nntp_caps.over_cmd = &xover_cmds[i];
+#	ifdef DEBUG
+					if (debug & DEBUG_NNTP)
+						debug_print_file("NNTP" "nntp_open() %s skipping data", &xover_cmds[i]);
+#	endif /* DEBUG */
+					while (tin_fgets(FAKE_NNTP_FP, FALSE))
+						;
+					j = -1;
+					break;
+
+				default:
+					nntp_caps.over_cmd = &xover_cmds[i];
+					j = -1;
+					break;
 			}
 		}
+#	ifdef XHDR_XREF
+		for (i = 0, j = 0; i < 2 && j >= 0; i++) {
+			j = new_nntp_command(&xhdr_cmds[i], ERR_CMDSYN, line, sizeof(line));
+			switch (j) {
+				case ERR_COMMAND:
+					break;
+
+				default:	/* usualy ERR_CMDSYN (args missing), Typhoon/Twister sends ERR_NCING */
+					nntp_caps.hdr_cmd = &xhdr_cmds[i];
+					j = -1;
+					break;
+			}
+		}
+#	endif /* XHDR_XREF */
 	} else {
-		if (!xover_cmd) {
+		if (!nntp_caps.over_cmd) {
 			/*
-			 * LIST EXTENSIONS didn't mention OVER or XOVER, try
+			 * CAPABILITIES/LIST EXTENSIONS didn't mention OVER or XOVER, try
 			 * XOVER
 			 */
-			if (!nntp_command(xover_cmds, ERR_COMMAND, NULL, 0))
-				xover_cmd = xover_cmds;
+			switch (new_nntp_command(xover_cmds, ERR_NCING, line, sizeof(line))) {
+				case ERR_COMMAND:
+					break;
+
+				case 224:	/* unexpected multiline ok, e.g.: Synchronet 3.13 NNTP Service 1.92 */
+					nntp_caps.over_cmd = xover_cmds;
+#	ifdef DEBUG
+					if (debug & DEBUG_NNTP)
+						debug_print_file("NNTP", "nntp_open() %s skipping data", xover_cmds);
+#	endif /* DEBUG */
+					while (tin_fgets(FAKE_NNTP_FP, FALSE))
+						;
+					break;
+
+				default:
+					nntp_caps.over_cmd = xover_cmds;
+					break;
+			}
 		}
-#	if 0 /* unused */
-		if (!xhdr_cmd) {
+#	ifdef XHDR_XREF
+		if (!nntp_caps.hdr_cmd) {
 			/*
-			 * LIST EXTENSIONS didn't mention HDR or XHDR, try
+			 * CAPABILITIES/LIST EXTENSIONS didn't mention HDR or XHDR, try
 			 * XHDR
 			 */
-			if (!nntp_command(xhdr_cmds, ERR_COMMAND, NULL, 0))
-				xhdr_cmd = xhdr_cmds;
+			switch (new_nntp_command(xhdr_cmds, ERR_CMDSYN, line, sizeof(line))) {
+				case ERR_COMMAND:
+					break;
+
+				default:	/* usualy ERR_CMDSYN (args missing), Typhoon/Twister sends ERR_NCING */
+					nntp_caps.hdr_cmd = xhdr_cmds;
+					break;
+			}
 		}
-#	endif /* 0 */
+#	endif /* XHDR_XREF */
 	}
 
-	if (!xover_cmd) {
+	if (!nntp_caps.over_cmd) {
 		if (!is_reconnect && !batch_mode) {
 			wait_message(2, _(txt_no_xover_support));
 
@@ -1298,24 +1552,25 @@ nntp_open(
 			else
 				wait_message(2, _(txt_caching_off));
 		}
+	}
 #	if 0
-	} else {
+	else {
 		/*
 		 * TODO: issue warning if old index files found?
 		 *	      in index_newsdir?
 		 */
-#	endif /* 0 */
 	}
+#	endif /* 0 */
 
-#	if 0
-	/*
-	 * TODO: if we're using -n, check for LIST NEWSGROUPS <wildmat> */
-	 * see also comments in open_newsgroups_fp() */
-	 */
-	if (newsrc_active && !list_active) { /* -n */
-		/* code goes here */
+	if (!is_reconnect && !batch_mode && show_description && check_for_new_newsgroups) {
+		/*
+		 * TODO:
+		 * - document that "-d" and/or "-q" turns off "LIST MOTD" (or add
+		 *   a tinrc var to turn LIST MOTD on/off)
+		 */
+		if (nntp_caps.list_motd)
+			list_motd();
 	}
-#	endif /* 0 */
 
 	is_reconnect = TRUE;
 
@@ -1336,14 +1591,15 @@ nntp_close(
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp) {
 #	ifdef DEBUG
-		debug_nntp("nntp_close", "END");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "nntp_close() END");
 #	endif /* DEBUG */
 		close_server();
 	}
 #endif /* NNTP_ABLE */
 }
 
-
+#ifdef NNTP_ABLE
 /*
  * Get a response code from the server.
  * Returns:
@@ -1359,46 +1615,53 @@ get_only_respcode(
 	char *message,
 	size_t mlen)
 {
-	int respcode = 0;
-#ifdef NNTP_ABLE
-	char *ptr, *end;
+	int respcode;
+	char *end, *ptr;
 
 	ptr = tin_fgets(FAKE_NNTP_FP, FALSE);
 
 	if (tin_errno || ptr == NULL) {
 #	ifdef DEBUG
-		debug_nntp("<<<", "Error: tin_error<>0 or ptr==NULL in get_only_respcode()");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "<<< Error: tin_error<>0 or ptr==NULL in get_only_respcode()");
 #	endif /* DEBUG */
 		return -1;
 	}
 
 #	ifdef DEBUG
-	debug_nntp("<<<", ptr);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "<<< %s",  ptr);
 #	endif /* DEBUG */
 	respcode = (int) strtol(ptr, &end, 10);
 	DEBUG_IO((stderr, "get_only_respcode(%d)\n", respcode));
 
-	/* TODO: reconnect on ERR_FAULT? */
-	if ((respcode == ERR_FAULT || respcode == ERR_GOODBYE || respcode == OK_GOODBYE) && last_put[0] != '\0' && strcmp(last_put, "QUIT")) {
+	/*
+	 * we also reconnect on ERR_FAULT if last_put was ARTICLE or LIST or POST
+	 * as inn (2.2.3) sends ERR_FAULT on timeout
+	 */
+	if (last_put[0] != '\0' && ((respcode == ERR_FAULT && !strncmp(last_put, "ARTICLE", 7)) || (respcode == ERR_FAULT && !strncmp(last_put, "POST", 4)) || (respcode == ERR_FAULT && !strcmp(last_put, "LIST")) || respcode == ERR_GOODBYE || respcode == OK_GOODBYE) && strcmp(last_put, "QUIT")) {
 		/*
 		 * Maybe server timed out.
 		 * If so, retrying will force a reconnect.
 		 */
 #	ifdef DEBUG
-		debug_nntp("get_only_respcode", "timeout");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "get_only_respcode() timeout");
 #	endif /* DEBUG */
 		put_server(last_put);
 		ptr = tin_fgets(FAKE_NNTP_FP, FALSE);
 
 		if (tin_errno) {
 #	ifdef DEBUG
-			debug_nntp("<<<", "Error: tin_errno <> 0");
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "<<< Error: tin_errno <> 0");
 #	endif /* DEBUG */
 			return -1;
 		}
 
 #	ifdef DEBUG
-		debug_nntp("<<<", ptr);
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "<<< %s", ptr);
 #	endif /* DEBUG */
 		respcode = (int) strtol(ptr, &end, 10);
 		DEBUG_IO((stderr, "get_only_respcode(%d)\n", respcode));
@@ -1406,7 +1669,6 @@ get_only_respcode(
 	if (message != NULL && mlen > 1)		/* Pass out the rest of the text */
 		my_strncpy(message, end, mlen - 1);
 
-#endif /* NNTP_ABLE */
 	return respcode;
 }
 
@@ -1426,8 +1688,7 @@ get_respcode(
 	char *message,
 	size_t mlen)
 {
-	int respcode = 0;
-#ifdef NNTP_ABLE
+	int respcode;
 	char savebuf[NNTP_STRLEN];
 	char *ptr, *end;
 
@@ -1437,11 +1698,23 @@ get_respcode(
 		 * Server requires authentication.
 		 */
 #	ifdef DEBUG
-		debug_nntp("get_respcode", "authentication");
+		if (debug & DEBUG_NNTP)
+			debug_print_file("NNTP", "get_respcode() authentication");
 #	endif /* DEBUG */
 		strncpy(savebuf, last_put, sizeof(savebuf) - 1);		/* Take copy, as authenticate() will clobber this */
 
 		if (authenticate(nntp_server, userid, FALSE)) {
+			if (curr_group != NULL) {
+				DEBUG_IO((stderr, _("Rejoin current group\n")));
+				snprintf(last_put, sizeof(last_put), "GROUP %s", curr_group->name);
+				put_server(last_put);
+				s_gets(last_put, NNTP_STRLEN, nntp_rd_fp);
+#	ifdef DEBUG
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< %s", last_put);
+#	endif /* DEBUG */
+				DEBUG_IO((stderr, _("Read (%s)\n"), last_put));
+			}
 			strcpy(last_put, savebuf);
 
 			put_server(last_put);
@@ -1449,13 +1722,15 @@ get_respcode(
 
 			if (tin_errno) {
 #	ifdef DEBUG
-				debug_nntp("<<<", "Error: tin_errno <> 0");
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< Error: tin_errno <> 0");
 #	endif /* DEBUG */
 				return -1;
 			}
 
 #	ifdef DEBUG
-			debug_nntp("<<<", ptr);
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "<<< %s", ptr);
 #	endif /* DEBUG */
 			respcode = (int) strtol(ptr, &end, 10);
 			if (message != NULL && mlen > 1)				/* Pass out the rest of the text */
@@ -1467,12 +1742,10 @@ get_respcode(
 			tin_done(EXIT_FAILURE);
 		}
 	}
-#endif /* NNTP_ABLE */
 	return respcode;
 }
 
 
-#ifdef NNTP_ABLE
 /*
  * Do an NNTP command. Send command to server, and read the reply.
  * If the reply code matches success, then return an open file stream
@@ -1489,22 +1762,110 @@ nntp_command(
 {
 DEBUG_IO((stderr, "nntp_command(%s)\n", command));
 #	ifdef DEBUG
-	debug_nntp("nntp command", command);
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_command(%s)", command);
 #	endif /* DEBUG */
 	put_server(command);
 
 	if (!bool_equal(dangerous_signal_exit, TRUE)) {
 		if (get_respcode(message, mlen) != success) {
 #	ifdef DEBUG
-			debug_nntp(command, "NOT_OK");
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "nntp_command(%s) NOT_OK", command);
 #	endif /* DEBUG */
 			/* error_message("%s", message); */
 			return (FILE *) 0;
 		}
 	}
 #	ifdef DEBUG
-	debug_nntp(command, "OK");
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "nntp_command(%s) OK", command);
 #	endif /* DEBUG */
 	return FAKE_NNTP_FP;
 }
+
+
+/*
+ * same as above, but with a slightly more usefull return code.
+ * TODO: use it instead of nntp_command in the rest of the code
+ *       (wherever it is more usefull).
+ */
+int
+new_nntp_command(
+	const char *command,
+	int success,
+	char *message,
+	size_t mlen)
+{
+	int respcode = 0;
+
+DEBUG_IO((stderr, "new_nntp_command(%s)\n", command));
+#	ifdef DEBUG
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "new_nntp_command(%s)", command);
+#	endif /* DEBUG */
+	put_server(command);
+
+	if (!bool_equal(dangerous_signal_exit, TRUE)) {
+		if ((respcode = get_respcode(message, mlen)) != success) {
+#	ifdef DEBUG
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "new_nntp_command(%s) NOT_OK - Expected: %d, got: %d", command, success, respcode);
+#	endif /* DEBUG */
+			return respcode;
+		}
+	}
+#	ifdef DEBUG
+	if (debug & DEBUG_NNTP)
+		debug_print_file("NNTP", "new_nntp_command(%s) OK", command);
+#	endif /* DEBUG */
+	return respcode;
+}
+
+
+static void
+list_motd(
+	void)
+{
+	char *ptr;
+	char buf[NNTP_STRLEN];
+	int i;
+	unsigned int l = 0;
+
+	buf[0] = '\0';
+	i = new_nntp_command("LIST MOTD", OK_MOTD, buf, sizeof(buf));
+
+	switch (i) {
+		case OK_MOTD:
+#	ifdef HAVE_COLOR
+			fcol(tinrc.col_message);
+#	endif /* HAVE_COLOR */
+			while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
+#	ifdef DEBUG
+				if (debug & DEBUG_NNTP)
+					debug_print_file("NNTP", "<<< %s", ptr);
+#	endif /* DEBUG */
+				/*
+				 * TODO: - store a hash value of the entire motd in the server-rc
+				 *         and only if it differs from the old value display the
+				 *         motd?
+				 *       - use some sort of pager?
+				 *       - -> lang.c
+				 */
+				my_printf("%s%s\n", _("MOTD: "), ptr);
+				l++;
+			}
+#	ifdef HAVE_COLOR
+			fcol(tinrc.col_normal);
+#	endif /* HAVE_COLOR */
+			if (l) {
+				my_flush();
+				sleep((l >> 1) | 0x01);
+			}
+			break;
+
+		default:	/* common response codes are 500, 501, 503 */
+			break;
+	}
+}
 #endif /* NNTP_ABLE */
