@@ -3,10 +3,10 @@
  *  Module    : options_menu.c
  *  Author    : Michael Bienia <michael@vorlon.ping.de>
  *  Created   : 2004-09-05
- *  Updated   : 2004-09-05
+ *  Updated   : 2008-02-25
  *  Notes     : Split from config.c
  *
- * Copyright (c) 2004 Michael Bienia <michael@vorlon.ping.de>
+ * Copyright (c) 2004-2008 Michael Bienia <michael@vorlon.ping.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,15 +38,13 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
-#ifndef TINTBL_H
+#ifndef TINCFG_H
 #	include "tincfg.h"
-#endif /* !TINTBL_H */
+#endif /* !TINCFG_H */
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
+
 
 #define option_lines_per_page (cLINES - INDEX_TOP - 3)
 
@@ -57,11 +55,15 @@ static enum option_enum first_option_on_screen, last_option_on_screen;
  */
 static enum option_enum move_cursor(enum option_enum cur_option, t_bool down);
 static enum option_enum next_option(enum option_enum option, t_bool incl_titles);
+static enum option_enum opt_scroll_down(enum option_enum option);
+static enum option_enum opt_scroll_up(enum option_enum option);
 static enum option_enum prev_option(enum option_enum option, t_bool incl_titles);
 static enum option_enum set_option_num(int num);
 static int get_option_num(enum option_enum option);
 static t_bool option_is_title(enum option_enum option);
 static t_bool option_on_page(enum option_enum option);
+static t_function option_left(void);
+static t_function option_right(void);
 static void highlight_option(enum option_enum option);
 static void print_any_option(enum option_enum the_option);
 static void redraw_screen(enum option_enum option);
@@ -81,10 +83,10 @@ static void unhighlight_option(enum option_enum option);
  */
 int
 option_row(
-	int option)
+	enum option_enum option)
 {
 	int i = 0;
-	int j = first_option_on_screen;
+	enum option_enum j = first_option_on_screen;
 
 	while (j < option) {
 		if (option_is_visible(j))
@@ -135,9 +137,10 @@ set_option_num(
  */
 t_bool
 option_is_visible(
-	int option)
+	enum option_enum option)
 {
 	switch (option) {
+#ifdef HAVE_COLOR
 		case OPT_COL_BACK:
 		case OPT_COL_FROM:
 		case OPT_COL_HEAD:
@@ -157,6 +160,9 @@ option_is_visible(
 		case OPT_COL_TEXT:
 		case OPT_COL_TITLE:
 		case OPT_COL_URLS:
+		case OPT_QUOTE_REGEX:
+		case OPT_QUOTE_REGEX2:
+		case OPT_QUOTE_REGEX3:
 			return tinrc.use_color;
 
 		case OPT_COL_MARKSTAR:
@@ -164,12 +170,17 @@ option_is_visible(
 		case OPT_COL_MARKSLASH:
 		case OPT_COL_MARKSTROKE:
 			return tinrc.word_highlight && tinrc.use_color;
+#endif /* HAVE_COLOR */
 
 		case OPT_WORD_H_DISPLAY_MARKS:
 		case OPT_MONO_MARKSTAR:
 		case OPT_MONO_MARKDASH:
 		case OPT_MONO_MARKSLASH:
 		case OPT_MONO_MARKSTROKE:
+		case OPT_SLASHES_REGEX:
+		case OPT_STARS_REGEX:
+		case OPT_STROKES_REGEX:
+		case OPT_UNDERSCORES_REGEX:
 			return tinrc.word_highlight;
 
 		default:
@@ -205,7 +216,7 @@ fmt_option_prompt(
 	char *dst,
 	size_t len,
 	t_bool editing,
-	int option)
+	enum option_enum option)
 {
 	char *buf;
 	size_t option_width = MAX(35, cCOLS / 2 - 9);
@@ -218,48 +229,36 @@ fmt_option_prompt(
 
 	if (!option_is_title(option)) {
 		int num = get_option_num(option);
+
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if (wbuf != NULL) {
-			wbuf2 = my_malloc(sizeof(wchar_t) * (option_width + 1));
-			wstrunc(wbuf, wbuf2, option_width + 1, option_width);
-
+			wbuf2 = wstrunc(wbuf, option_width);
 			if ((buf = wchar_t2char(wbuf2)) == NULL) {
 				/* conversion failed, truncate original string */
-				buf = my_malloc(option_width + 1);
-				strunc(_(option_table[option].txt->opt), buf, option_width + 1, option_width);
+				buf = strunc(_(option_table[option].txt->opt), option_width);
 				snprintf(dst, len, "%s %3d. %-*.*s: ", editing ? "->" : "  ", num, (int) option_width, (int) option_width, buf);
-			} else {
+			} else
 				snprintf(dst, len, "%s %3d. %-*.*s: ", editing ? "->" : "  ", num,
 					(int) (strlen(buf) + option_width - wcswidth(wbuf2, option_width + 1)),
 					(int) (strlen(buf) + option_width - wcswidth(wbuf2, option_width + 1)), buf);
-			}
 			free(wbuf2);
 		} else
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		{
 			/* truncate original string */
-			buf = my_malloc(option_width + 1);
-			strunc(_(option_table[option].txt->opt), buf, option_width + 1, option_width);
+			buf = strunc(_(option_table[option].txt->opt), option_width);
 			snprintf(dst, len, "%s %3d. %-*.*s: ", editing ? "->" : "  ", num, (int) option_width, (int) option_width, buf);
 		}
 	} else {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if (wbuf != NULL) {
-			wbuf2 = my_malloc(sizeof(wchar_t) * (cCOLS - 3 + 1));
-			wstrunc(wbuf, wbuf2, cCOLS - 3 + 1, cCOLS - 3);
-			if ((buf = wchar_t2char(wbuf2)) == NULL) {
-				/* conversion failed, truncate original string */
-				buf = my_malloc(cCOLS - 3 + 1);
-				strunc(_(option_table[option].txt->opt), buf, cCOLS - 3 + 1, cCOLS - 3);
-			}
+			wbuf2 = wstrunc(wbuf, cCOLS - 3);
+			if ((buf = wchar_t2char(wbuf2)) == NULL)	/* conversion failed, truncate original string */
+				buf = strunc(_(option_table[option].txt->opt), cCOLS - 3);
 			free(wbuf2);
 		} else
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-		{
-			/* truncate original string */
-			buf = my_malloc(cCOLS - 3 + 1);
-			strunc(_(option_table[option].txt->opt), buf, cCOLS - 3 + 1, cCOLS - 3);
-		}
+			buf = strunc(_(option_table[option].txt->opt), cCOLS - 3);	/* truncate original string */
 		snprintf(dst, len, "  %s", buf);
 	}
 
@@ -295,12 +294,12 @@ print_any_option(
 		case OPT_LIST:
 			list = option_table[option].opt_list;
 			ptr2 = my_strdup(list[*(option_table[option].variable) + ((strcasecmp(_(list[0]), _(txt_default)) == 0) ? 1 : 0)]);
-			snprintf(ptr, len, "%s", _(ptr2));
+			strncpy(ptr, _(ptr2), len);
 			free(ptr2);
 			break;
 
 		case OPT_STRING:
-			snprintf(ptr, len, "%s", OPT_STRING_list[option_table[option].var_index]);
+			strncpy(ptr, OPT_STRING_list[option_table[option].var_index], len);
 			break;
 
 		case OPT_NUM:
@@ -330,6 +329,7 @@ print_any_option(
 #endif /* USE_CURSES */
 }
 
+
 static void
 repaint_option(
 	enum option_enum option)
@@ -410,6 +410,76 @@ move_cursor(
 }
 
 
+/*
+ * scroll the screen one line down
+ * the selected option is only moved if it is scrolled off the screen
+ */
+static enum option_enum
+opt_scroll_down(
+	enum option_enum option)
+{
+	if (last_option_on_screen < LAST_OPT) {
+		first_option_on_screen = next_option(first_option_on_screen, TRUE);
+		set_last_option_on_screen(first_option_on_screen);
+#ifdef USE_CURSES
+		do_scroll(1);
+		print_any_option(last_option_on_screen);
+		stow_cursor();
+#else
+		show_config_page();
+#endif /* USE_CURSES */
+		if (option < first_option_on_screen) {
+			option = first_option_on_screen;
+			if (option_is_title(option))
+				option = next_option(option, FALSE);
+#ifdef USE_CURSES
+			highlight_option(option);
+#endif /* USE_CURSES */
+		}
+#ifndef USE_CURSES
+		/* in the !USE_CURSES case we must always highlight the option */
+		highlight_option(option);
+#endif /* !USE_CURSES */
+	}
+	return option;
+}
+
+
+/*
+ * scroll the screen one line up
+ * the selected option is only moved if it is scrolled off the screen
+ */
+static enum option_enum
+opt_scroll_up(
+	enum option_enum option)
+{
+	if (first_option_on_screen > 0) {
+		first_option_on_screen = prev_option(first_option_on_screen, TRUE);
+		set_last_option_on_screen(first_option_on_screen);
+#ifdef USE_CURSES
+		do_scroll(-1);
+		print_any_option(first_option_on_screen);
+		stow_cursor();
+#else
+		show_config_page();
+#endif /* USE_CURSES */
+		if (option > last_option_on_screen) {
+			option = last_option_on_screen;
+			if (option_is_title(option))
+				option = prev_option(option, FALSE);
+#ifdef USE_CURSES
+			highlight_option(option);
+#endif /* USE_CURSES */
+		}
+#ifndef USE_CURSES
+		/* in the !USE_CURSES case we must always highlight the option */
+		highlight_option(option);
+#endif /* !USE_CURSES */
+	}
+	return option;
+}
+
+
 /*
  * returns the next visible option
  * if 'incl_titles' is TRUE titles are also returned else they are skipped
@@ -506,7 +576,7 @@ unhighlight_option(
 {
 	/* Astonishing hack */
 	t_menu *savemenu = currmenu;
-	t_menu cfgmenu = { 0, 1, 0, 0, NULL, NULL };
+	t_menu cfgmenu = { 0, 1, 0, NULL, NULL, NULL };
 
 	currmenu = &cfgmenu;
 	currmenu->curr = option_row(option) - INDEX_TOP;
@@ -523,14 +593,13 @@ unhighlight_option(
  */
 void
 refresh_config_page(
-	int act_option)
+	enum option_enum act_option)
 {
-	static int last_option = 0;
-	t_bool force_redraw = FALSE;
+	static enum option_enum last_option = 0;
+	/* t_bool force_redraw = FALSE; */
 
-	if (act_option < 0) {
-		/* called by signal handler */
-		force_redraw = TRUE;
+	if (act_option == SIGNAL_HANDLER) {	/* called by signal handler */
+		/* force_redraw = TRUE; */
 		act_option = last_option;
 		set_last_option_on_screen(first_option_on_screen); /* terminal size may have changed */
 		if (!option_on_page(last_option)) {
@@ -580,13 +649,12 @@ show_config_page(
 	center_line(0, TRUE, _(txt_options_menu));
 
 	for (i = first_option_on_screen; i <= last_option_on_screen; i++) {
-		while(!option_is_visible(i))
+		while (!option_is_visible(i))
 			i++;
 		if (i > LAST_OPT)
 			break;
 		print_any_option(i);
 	}
-	CleartoEOS();
 
 	show_menu_help(txt_select_config_file_option);
 	my_flush();
@@ -609,23 +677,36 @@ check_score_defaults(
 }
 
 
+static t_function
+option_left(
+	void)
+{
+	return GLOBAL_QUIT;
+}
+
+
+static t_function
+option_right(
+	void)
+{
+	return CONFIG_SELECT;
+}
+
+
 /*
  * options menu so that the user can dynamically change parameters
  *
- * TODO: - why do we use ret_code when we never modify it?  what about calling
- *         code which checks the return value?
- *       - when we change something we need to update the related attributes
+ * TODO: - when we change something we need to update the related attributes
  *         as well (see line 2009).
  */
-int
+void
 change_config_file(
 	struct t_group *group)
 {
 	enum option_enum option, old_option;
-	int ch = 1;
-	int ret_code = NO_FILTERING;
-	int mime_encoding = MIME_ENCODING_7BIT;
+	int mime_encoding;
 	t_bool change_option = FALSE;
+	t_function func;
 
 	signal_context = cConfig;
 
@@ -635,74 +716,27 @@ change_config_file(
 
 	redraw_screen(option);
 	forever {
-		ch = ReadCh();
-
-		/*
-		 * convert arrow key codes to "normal" codes
-		 */
-		switch (ch) {
-			case ESC:	/* common arrow keys */
-#	ifdef HAVE_KEY_PREFIX
-			case KEY_PREFIX:
-#	endif /* HAVE_KEY_PREFIX */
-				switch (get_arrow_key(ch)) {
-					case KEYMAP_UP:
-						ch = map_to_local(iKeyUp, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_DOWN:
-						ch = map_to_local(iKeyDown, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_HOME:
-						ch = map_to_local(iKeyFirstPage, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_END:
-						ch = map_to_local(iKeyLastPage, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_PAGE_UP:
-						ch = map_to_local(iKeyPageUp, &menukeymap.config_change);
-						break;
-
-					case KEYMAP_PAGE_DOWN:
-						ch = map_to_local(iKeyPageDown, &menukeymap.config_change);
-						break;
-
-					default:
-						break;
-				} /* switch (get_arrow_key()) */
-				break;
-
-			default:
-				break;
-		}	/* switch (ch) */
-
-		switch (map_to_default(ch, &menukeymap.config_change)) {
-			case iKeyQuit:
+		switch ((func = handle_keypad(option_left, option_right, NULL, option_menu_keys))) {
+			case GLOBAL_QUIT:
 				write_config_file(local_config_file);
 				/* FALLTHROUGH */
-			case iKeyConfigNoSave:
+			case CONFIG_NO_SAVE:
 				clear_note_area();
-				return ret_code;
+				return;
 
-			case iKeyUp:
-			case iKeyUp2:
+			case GLOBAL_LINE_UP:
 				unhighlight_option(option);
 				option = move_cursor(option, FALSE);
 				highlight_option(option);
 				break;
 
-			case iKeyDown:
-			case iKeyDown2:
+			case GLOBAL_LINE_DOWN:
 				unhighlight_option(option);
 				option = move_cursor(option, TRUE);
 				highlight_option(option);
 				break;
 
-			case iKeyFirstPage:
-			case iKeyConfigFirstPage2:
+			case GLOBAL_FIRST_PAGE:
 				unhighlight_option(option);
 				option = 1;
 				first_option_on_screen = 0;
@@ -711,8 +745,7 @@ change_config_file(
 				/* highlight_option(option); is already done by redraw_screen() */
 				break;
 
-			case iKeyLastPage:
-			case iKeyConfigLastPage2:
+			case GLOBAL_LAST_PAGE:
 				unhighlight_option(option);
 				option = LAST_OPT;
 				last_option_on_screen = LAST_OPT;
@@ -721,9 +754,7 @@ change_config_file(
 				/* highlight_option(option); is already done by redraw_screen() */
 				break;
 
-			case iKeyPageUp:
-			case iKeyPageUp2:
-			case iKeyPageUp3:
+			case GLOBAL_PAGE_UP:
 				unhighlight_option(option);
 				if (option != first_option_on_screen &&	!(option_is_title(first_option_on_screen) && option == next_option(first_option_on_screen, FALSE))) {
 					option = first_option_on_screen;
@@ -743,41 +774,71 @@ change_config_file(
 					last_option_on_screen = prev_option(first_option_on_screen, TRUE);
 
 				set_first_option_on_screen(last_option_on_screen);
-				option = first_option_on_screen;
+				if (last_option_on_screen == LAST_OPT)
+					option = last_option_on_screen;
+				else
+					option = first_option_on_screen;
 				if (option_is_title(option))
 					option = next_option(option, FALSE);
 				redraw_screen(option);
 				/* highlight_option(option); is already done by redraw_screen() */
 				break;
 
-			case iKeyPageDown:
-			case iKeyPageDown2:
-			case iKeyPageDown3:
+			case GLOBAL_PAGE_DOWN:
 				unhighlight_option(option);
-				if (tinrc.scroll_lines == -2 ) {
-					int i = option_lines_per_page / 2;
+				if (option == LAST_OPT) {
+					/* wrap around */
+					first_option_on_screen = 0;
+					option = 0;
+				} else {
+					enum option_enum old_first = first_option_on_screen;
 
-					for (; i > 0; i--) {
-						first_option_on_screen = next_option(first_option_on_screen, TRUE);
-						if (first_option_on_screen == 0)	/* end on wrap_around */
-							break;
-					}
-				} else
-					first_option_on_screen = next_option(last_option_on_screen, TRUE);
+					if (tinrc.scroll_lines == -2) {
+						int i = option_lines_per_page / 2;
+
+						for (; i > 0; i--) {
+							first_option_on_screen = next_option(first_option_on_screen, TRUE);
+							if (first_option_on_screen == 0)	/* end on wrap_around */
+								break;
+						}
+					} else
+						first_option_on_screen = next_option(last_option_on_screen, TRUE);
+
+					if (first_option_on_screen == 0) {
+						first_option_on_screen = old_first;
+						option = LAST_OPT;
+						highlight_option(option);
+						break;
+					} else
+						option = first_option_on_screen;
+				}
 
 				set_last_option_on_screen(first_option_on_screen);
-				option = first_option_on_screen;
 				if (option_is_title(option))
 					option = next_option(option, FALSE);
-
 				redraw_screen(option);
 				/* highlight_option(option); is already done by redraw_screen() */
 				break;
 
-			case '1': case '2': case '3': case '4': case '5':
-			case '6': case '7': case '8': case '9':
+			case GLOBAL_SCROLL_UP:
+				option = opt_scroll_up(option);
+				break;
+
+			case GLOBAL_SCROLL_DOWN:
+				option = opt_scroll_down(option);
+				break;
+
+			case DIGIT_1:
+			case DIGIT_2:
+			case DIGIT_3:
+			case DIGIT_4:
+			case DIGIT_5:
+			case DIGIT_6:
+			case DIGIT_7:
+			case DIGIT_8:
+			case DIGIT_9:
 				unhighlight_option(option);
-				option = set_option_num(prompt_num(ch, _(txt_enter_option_num)));
+				option = set_option_num(prompt_num(func_to_key(func, option_menu_keys), _(txt_enter_option_num)));
 				if (!option_on_page(option)) {
 					first_option_on_screen = option;
 					set_last_option_on_screen(option);
@@ -786,14 +847,14 @@ change_config_file(
 					highlight_option(option);
 				break;
 
-			case iKeySearchSubjF:
-			case iKeySearchSubjB:
-			case iKeySearchRepeat:
-				if (ch == iKeySearchRepeat && i_key_search_last != iKeySearchSubjF && i_key_search_last != iKeySearchSubjB)
+			case GLOBAL_SEARCH_SUBJECT_FORWARD:
+			case GLOBAL_SEARCH_SUBJECT_BACKWARD:
+			case GLOBAL_SEARCH_REPEAT:
+				if (func == GLOBAL_SEARCH_REPEAT && last_search != GLOBAL_SEARCH_SUBJECT_FORWARD && last_search != GLOBAL_SEARCH_SUBJECT_BACKWARD)
 					break;
 
 				old_option = option;
-				option = search_config((ch == iKeySearchSubjF), (ch == iKeySearchRepeat), option, LAST_OPT);
+				option = search_config((func == GLOBAL_SEARCH_SUBJECT_FORWARD), (func == GLOBAL_SEARCH_REPEAT), option, LAST_OPT);
 				if (option != old_option) {
 					unhighlight_option(old_option);
 					if (!option_on_page(option)) {
@@ -805,15 +866,19 @@ change_config_file(
 				}
 				break;
 
-			case iKeyConfigSelect:
-			case iKeyConfigSelect2:
+			case CONFIG_SELECT:
 				change_option = TRUE;
 				break;
 
-			case iKeyRedrawScr:	/* redraw screen */
+			case GLOBAL_REDRAW_SCREEN:
+				set_last_option_on_screen(first_option_on_screen);
 				redraw_screen(option);
 				break;
 
+			case GLOBAL_VERSION:
+				info_message(cvers);
+				break;
+
 			default:
 				break;
 		} /* switch (ch) */
@@ -839,21 +904,20 @@ change_config_file(
 						case OPT_KEEP_DEAD_ARTICLES:
 						case OPT_MARK_IGNORE_TAGS:
 						case OPT_MARK_SAVED_READ:
-						case OPT_PGDN_GOTO_NEXT:
 						case OPT_POS_FIRST_UNREAD:
 						case OPT_POST_PROCESS_VIEW:
+#ifndef DISABLE_PRINTING
 						case OPT_PRINT_HEADER:
+#endif /*! DISABLE_PRINTING */
 						case OPT_PROCESS_ONLY_UNREAD:
 						case OPT_PROMPT_FOLLOWUPTO:
 						case OPT_SHOW_ONLY_UNREAD_GROUPS:
 						case OPT_SHOW_SIGNATURES:
 						case OPT_SIGDASHES:
 						case OPT_SIGNATURE_REPOST:
-						case OPT_SPACE_GOTO_NEXT_UNREAD:
 						case OPT_START_EDITOR_OFFSET:
 						case OPT_STRIP_BLANKS:
 						case OPT_STRIP_NEWSRC:
-						case OPT_TAB_GOTO_NEXT_UNREAD:
 						case OPT_TEX2ISO_CONV:
 						case OPT_THREAD_CATCHUP_ON_EXIT:
 #if defined(HAVE_ICONV_OPEN_TRANSLIT) && defined(CHARSET_CONVERSION)
@@ -1002,6 +1066,7 @@ change_config_file(
 						case OPT_COL_MARKSTROKE:
 						case OPT_COL_URLS:
 #endif /* HAVE_COLOR */
+						case OPT_GOTO_NEXT_UNREAD:
 						case OPT_HIDE_UUE:
 						case OPT_INTERACTIVE_MAILER:
 						case OPT_WORD_H_DISPLAY_MARKS:
@@ -1039,6 +1104,8 @@ change_config_file(
 										grpmenu.curr = n;
 								}
 							}
+							set_last_option_on_screen(first_option_on_screen);
+							redraw_screen(option);
 							clear_message();
 							break;
 
@@ -1099,9 +1166,6 @@ change_config_file(
 								glob_attributes.mm_network_charset = tinrc.mm_network_charset;
 								if (group)
 									group->attribute->mm_network_charset = tinrc.mm_network_charset;
-#	ifdef NO_LOCALE
-								strcpy(tinrc.mm_local_charset, txt_mime_charsets[tinrc.mm_network_charset]);
-#	endif /* NO_LOCALE */
 							}
 							/*
 							 * check if we have selected a 7bit charset, otherwise
@@ -1127,7 +1191,7 @@ change_config_file(
 										tinrc.post_mime_encoding = MIME_ENCODING_8BIT;
 										repaint_option(OPT_POST_MIME_ENCODING);
 									}
-								} else { /* and vice versa, if we have a 7bit chaset but a !7bit encoding, fix that */
+								} else { /* and vice versa, if we have a 7bit charset but a !7bit encoding, fix that */
 									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
 										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
 											tinrc.mail_mime_encoding = tinrc.post_mime_encoding = MIME_ENCODING_7BIT;
@@ -1181,9 +1245,6 @@ change_config_file(
 						case OPT_MAIL_ADDRESS:
 						case OPT_MAIL_QUOTE_FORMAT:
 						case OPT_METAMAIL_PROG:
-#ifndef CHARSET_CONVERSION
-						case OPT_MM_CHARSET:
-#endif /* !CHARSET_CONVERSION */
 						case OPT_NEWS_QUOTE_FORMAT:
 						case OPT_QUOTE_CHARS:
 						case OPT_SPAMTRAP_WARNING_ADDRESSES:
@@ -1192,6 +1253,25 @@ change_config_file(
 							prompt_option_string(option);
 							break;
 
+#ifndef CHARSET_CONVERSION
+						case OPT_MM_CHARSET:
+							prompt_option_string(option);
+							/*
+							 * No charset conversion available, assume local charset
+							 * to be network charset.
+							 */
+							STRCPY(tinrc.mm_local_charset, tinrc.mm_charset);
+							break;
+#else
+#	ifdef NO_LOCALE
+						case OPT_MM_LOCAL_CHARSET:
+							prompt_option_string(option);
+							/* no locales -> can't guess local charset */
+							break;
+
+#	endif /* NO_LOCALE */
+#endif /* !CHARSET_CONVERSION */
+
 						case OPT_NEWS_HEADERS_TO_DISPLAY:
 							prompt_option_string(option);
 							if (news_headers_to_display_array)
@@ -1320,6 +1400,24 @@ change_config_file(
 							compile_regex(tinrc.strip_was_regex, &strip_was_regex, 0);
 							break;
 
+						case OPT_VERBATIM_BEGIN_REGEX:
+							prompt_option_string(option);
+							FreeIfNeeded(verbatim_begin_regex.re);
+							FreeIfNeeded(verbatim_begin_regex.extra);
+							if (!strlen(tinrc.verbatim_begin_regex))
+								STRCPY(tinrc.verbatim_begin_regex, DEFAULT_VERBATIM_BEGIN_REGEX);
+							compile_regex(tinrc.verbatim_begin_regex, &verbatim_begin_regex, PCRE_ANCHORED);
+							break;
+
+						case OPT_VERBATIM_END_REGEX:
+							prompt_option_string(option);
+							FreeIfNeeded(verbatim_end_regex.re);
+							FreeIfNeeded(verbatim_end_regex.extra);
+							if (!strlen(tinrc.verbatim_end_regex))
+								STRCPY(tinrc.verbatim_end_regex, DEFAULT_VERBATIM_END_REGEX);
+							compile_regex(tinrc.verbatim_end_regex, &verbatim_end_regex, PCRE_ANCHORED);
+							break;
+
 						case OPT_DATE_FORMAT:
 							prompt_option_string(option);
 							if (!strlen(tinrc.date_format)) {
@@ -1379,6 +1477,12 @@ change_config_file(
 							redraw_screen(option);
 							break;
 
+						case OPT_THREAD_PERC:
+							prompt_option_num(option);
+							if (tinrc.thread_perc < 0 || tinrc.thread_perc > 100)
+								tinrc.thread_perc = THREAD_PERC_DEFAULT;
+							break;
+
 						case OPT_WRAP_COLUMN:
 							prompt_option_num(option);
 							/* recook if in an article is open */
@@ -1423,5 +1527,5 @@ change_config_file(
 		} /* if (change_option) */
 	} /* forever */
 	/* NOTREACHED */
-	return ret_code;
+	return;
 }
