@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-06-22
+ *  Updated   : 2008-04-25
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2008 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,12 +41,6 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
-#ifndef RFC2046_H
-/* #	include "rfc2046.h" */
-#endif /* !RFC2046_H */
 
 #ifdef HAVE_UUDEVIEW_H
 #	ifndef __UUDEVIEW_H__
@@ -54,9 +48,10 @@
 #	endif /* !__UUDEVIEW_H__ */
 #endif /* HAVE_UUDEVIEW_H */
 
+#ifndef HAVE_LIBUU
 #undef OFF
-
 enum state { INITIAL, MIDDLE, OFF, END };
+#endif /* !HAVE_LIBUU */
 
 /*
  * Local prototypes
@@ -65,6 +60,7 @@ static FILE *open_save_filename(const char *path, t_bool mbox);
 static int match_content_type(t_part *part, char *type);
 static t_bool check_save_mime_type(t_part *part, const char *mime_types);
 static t_bool decode_save_one(t_part *part, FILE *rawfp, t_bool postproc);
+static t_bool expand_save_filename(char *outpath, size_t outpath_len, const char *path);
 static void generate_filename(char *buf, int buflen, const char *suffix);
 static void post_process_uud(void);
 static void post_process_sh(void);
@@ -100,10 +96,10 @@ check_start_save_any_news(
 	FILE *artfp, *savefp;
 	FILE *fp_log = (FILE *) 0;
 	char *line;
-	char buf[LEN], logfile[LEN];
+	char buf[LEN];
 	char group_path[PATH_LEN];
 	char path[PATH_LEN];
-	char savefile[PATH_LEN];
+	char logfile[PATH_LEN], savefile[PATH_LEN];
 	char subject[HEADER_LEN];
 	int group_count = 0;
 	int i, j;
@@ -123,16 +119,16 @@ check_start_save_any_news(
 			break;
 
 		case MAIL_ANY_NEWS:
-			joinpath(savefile, TMPDIR, "tin");
+			joinpath(savefile, sizeof(savefile), TMPDIR, "tin");
 #ifdef APPEND_PID
 			snprintf(savefile + strlen(savefile), sizeof(savefile) - strlen(savefile), ".%d", (int) process_id);
 #endif /* APPEND_PID */
 			/* FALLTHROUGH */
 
 		case SAVE_ANY_NEWS:
-			joinpath(logfile, rcdir, "log");
+			joinpath(logfile, sizeof(logfile), rcdir, "log");
 
-			if (no_write || (fp_log = fopen(logfile, "w" FOPEN_OPTS)) == NULL) {
+			if (no_write || (fp_log = fopen(logfile, "w")) == NULL) {
 				perror_message(_(txt_cannot_open), logfile);
 				fp_log = stdout;
 				verbose = FALSE;
@@ -182,10 +178,10 @@ check_start_save_any_news(
 				char tmp[PATH_LEN];
 
 				if (!strfpath(group->attribute->savedir, tmp, sizeof(tmp), group))
-					joinpath(tmp, homedir, DEFAULT_SAVEDIR);
+					joinpath(tmp, sizeof(tmp), homedir, DEFAULT_SAVEDIR);
 
 				make_group_path(group->name, group_path);
-				joinpath(path, tmp, group_path);
+				joinpath(path, sizeof(path), tmp, group_path);
 				create_path(path);	/* TODO error handling */
 			}
 		}
@@ -204,7 +200,7 @@ check_start_save_any_news(
 						print_first = FALSE;
 					}
 					if (!verbose && !catchup) /* we don't need details */
-						return 2;
+						return NEWS_AVAIL_EXIT;
 					art_count++;
 					if (arts[j].score >= tinrc.score_select)
 						hot_count++;
@@ -223,10 +219,10 @@ check_start_save_any_news(
 
 					if (function == SAVE_ANY_NEWS) {
 						snprintf(buf, sizeof(buf), "%ld", arts[j].artnum);
-						joinpath(savefile, path, buf);
+						joinpath(savefile, sizeof(savefile), path, buf);
 					}
 
-					if ((savefp = fopen(savefile, "w" FOPEN_OPTS)) == NULL) {
+					if ((savefp = fopen(savefile, "w")) == NULL) {
 						fprintf(fp_log, _(txt_cannot_open), savefile);
 						if (verbose)
 							perror_message(_(txt_cannot_open), savefile);
@@ -295,11 +291,11 @@ check_start_save_any_news(
 			 * TODO: shall we return 2 or 0 in the -cZ case?
 			 */
 			if (unread_news && !catchup)
-				return 2;
+				return NEWS_AVAIL_EXIT;
 			else {
 				if (verbose)
 					wait_message(1, _(txt_there_is_no_news));
-				return 0;
+				return EXIT_SUCCESS;
 			}
 			/* NOTREACHED */
 
@@ -346,8 +342,8 @@ open_save_filename(
 	FILE *fp;
 	char keyappend[MAXKEYLEN], keyoverwrite[MAXKEYLEN], keyquit[MAXKEYLEN];
 	char mode[3];
-	int ch;
 	struct stat st;
+	t_function func;
 
 	strcpy(mode, "a+");
 
@@ -364,27 +360,31 @@ open_save_filename(
 		}
 /* TODO: will this get called every art? Should only be done once/batch */
 /* TODO: or add an option for defaulting on all future queries */
-		ch = prompt_slk_response(tinrc.default_save_mode,
-				&menukeymap.save_append_overwrite_quit,
+/* TODO: 'truncate' path if query exceeds screen-width */
+		func = prompt_slk_response((tinrc.default_save_mode == 'a' ? SAVE_APPEND_FILE : SAVE_OVERWRITE_FILE),
+				save_append_overwrite_keys,
 				_(txt_append_overwrite_quit), path,
-				printascii(keyappend, map_to_local(iKeySaveAppendFile, &menukeymap.save_append_overwrite_quit)),
-				printascii(keyoverwrite, map_to_local(iKeySaveOverwriteFile, &menukeymap.save_append_overwrite_quit)),
-				printascii(keyquit, map_to_local(iKeyQuit, &menukeymap.save_append_overwrite_quit)));
+				printascii(keyappend, func_to_key(SAVE_APPEND_FILE, save_append_overwrite_keys)),
+				printascii(keyoverwrite, func_to_key(SAVE_OVERWRITE_FILE, save_append_overwrite_keys)),
+				printascii(keyquit, func_to_key(GLOBAL_QUIT, save_append_overwrite_keys)));
 
-		switch (ch) {
-			case iKeySaveOverwriteFile:
+		switch (func) {
+			case SAVE_OVERWRITE_FILE:
 				strcpy(mode, "w");
 				break;
 
-			case iKeyAbort:
-			case iKeyQuit:
+			case GLOBAL_ABORT:
+			case GLOBAL_QUIT:
 				wait_message(1, _(txt_art_not_saved));
 				return NULL;
 
-			default:	/* iKeySaveAppendFile */
+			default:	/* SAVE_APPEND_FILE */
 				break;
 		}
-		tinrc.default_save_mode = ch;
+		if (func == SAVE_OVERWRITE_FILE)
+			tinrc.default_save_mode = 'o';
+		else
+			tinrc.default_save_mode = 'a';
 	}
 
 	if ((fp = fopen(path, mode)) == NULL) {
@@ -466,11 +466,11 @@ save_and_process_art(
 
 		/* Add on the archive name as a directory */
 		/* TODO: maybe a s!/!.! on archive-name would be better */
-		joinpath(archpath, path, artptr->archive->name);
+		joinpath(archpath, sizeof(archpath), path, artptr->archive->name);
 
 		/* Generate the filename part and append it */
 		snprintf(filename, sizeof(filename), "%s%s", partprefix, artptr->archive->partnum);
-		joinpath(path, archpath, filename);
+		joinpath(path, sizeof(path), archpath, filename);
 /*fprintf(stderr, "save_and_process_art archive-name mangled path=(%s)\n", path);*/
 		if (!create_path(path))
 			return FALSE;
@@ -481,11 +481,7 @@ save_and_process_art(
 		 * This is backward-contemptibility with older versions of tin
 		 */
 		if (!is_mailbox && max > 1) {
-#ifdef VMS
-			const char suffixsep = '-';		/* Suffix seperator for .001 type extensions */
-#else
 			const char suffixsep = '.';
-#endif /* VMS */
 
 			sprintf(&path[strlen(path)], "%c%03d", suffixsep, num_save + 1);
 		}
@@ -560,7 +556,6 @@ t_bool
 create_path(
 	const char *path)
 {
-#ifndef VMS
 	char buf[PATH_LEN];
 	int i, j, len;
 	struct stat st;
@@ -581,15 +576,6 @@ create_path(
 			}
 		}
 	}
-#else
-	if (my_mkdir(path, (mode_t) (S_IRWXU|S_IRUGO|S_IXUGO)) == -1) {
-		if (errno != EEXIST) {
-			perror_message(_(txt_cannot_create), buf);
-			return FALSE;
-		}
-	}
-#endif /* !VMS */
-
 	return TRUE;
 }
 
@@ -616,26 +602,32 @@ generate_filename(
  * Expand metacharacters and use defaults as needed.
  * Return TRUE if the path is a mailbox, or FALSE otherwise.
  */
-t_bool
+static t_bool
 expand_save_filename(
 	char *outpath,
+	size_t outpath_len,
 	const char *path)
 {
-	int ret = strfpath(path, outpath, PATH_LEN, curr_group);
+	char base_filename[PATH_LEN];
+	char buf[PATH_LEN];
+	char buf_path[PATH_LEN];
+	int ret;
 
 	/*
-	 * If no path exists or the above failed in some way, use sensible defaults
-	 * Put the generic path into 'outpath'
+	 * Make sure that externally supplied filename is a filename only and fits
+	 * into buffer
 	 */
-	if ((ret == 0) || !(strrchr(outpath, DIRSEP))) {
-		char buf[PATH_LEN];
+	STRCPY(buf_path, path);
+	base_name(buf_path, base_filename);
 
-		if (!strfpath(curr_group->attribute->savedir, buf, sizeof(buf), curr_group))
-			joinpath(buf, homedir, DEFAULT_SAVEDIR);
-		joinpath(outpath, buf, path);
-		return FALSE;
-	} else
-		return (ret == 1);
+	/* Build default path to save to */
+	if (!(ret = strfpath(curr_group->attribute->savedir, buf, sizeof(buf), curr_group)))
+		joinpath(buf, sizeof(buf), homedir, DEFAULT_SAVEDIR);
+
+	/* Join path and filename */
+	joinpath(outpath, outpath_len, buf, base_filename);
+
+	return (ret == 1);	/* should now always evaluate to FALSE */
 }
 
 
@@ -648,7 +640,7 @@ expand_save_filename(
  */
 t_bool
 post_process_files(
-	int proc_type_ch,
+	t_function proc_type_func,
 	t_bool auto_delete)
 {
 	if (num_save < 1)
@@ -660,13 +652,13 @@ post_process_files(
 #endif /* USE_CURSES */
 	my_printf("%s%s", _(txt_post_processing), cCRLF);
 
-	switch (proc_type_ch) {
-		case iKeyPProcShar:
+	switch (proc_type_func) {
+		case POSTPROCESS_SHAR:
 			post_process_sh();
 			break;
 
 		/* This is the default, eg, with AUTOSAVE */
-		case iKeyPProcYes:
+		case POSTPROCESS_YES:
 		default:
 			post_process_uud();
 			break;
@@ -763,7 +755,7 @@ post_process_uud(
 
 			/* item->mimetype seems not to be available for uudecoded files etc */
 			if (tinrc.post_process_view) {
-				joinpath(path, file_out_dir, item->filename);
+				joinpath(path, sizeof(path), file_out_dir, item->filename);
 				view_file(path, strrchr(path, DIRSEP) + 1);
 			}
 		} else {
@@ -854,7 +846,7 @@ post_process_uud(
 							generate_filename(name, sizeof(name), "uue");
 
 						filename = name;
-						expand_save_filename(path, filename);
+						expand_save_filename(path, sizeof(path), filename);
 						filename = strrchr(path, DIRSEP) + 1;	/* ptr to filename portion */
 						if ((fp_out = fopen(path, "w")) == NULL) {
 							perror_message(_(txt_cannot_open), path);
@@ -1075,12 +1067,7 @@ post_process_sh(
 	 * Grab the dirname portion
 	 */
 	my_strncpy(file_out_dir, save[0].path, save[0].file - save[0].path);
-
-#ifdef VMS
-	snprintf(file_out, sizeof(file_out), "%ssh.%05d", file_out_dir, (int) process_id);
-#else
 	snprintf(file_out, sizeof(file_out), "%ssh%05d", file_out_dir, (int) process_id);
-#endif /* VMS */
 
 	for (i = 0; i < num_save; i++) {
 		if ((fp_in = fopen(save[i].path, "r")) == NULL)
@@ -1091,7 +1078,7 @@ post_process_sh(
 		while (fgets(buf, (int) sizeof(buf), fp_in) != NULL) {
 			/* find #!/bin/sh style patterns */
 			if ((fp_out == NULL) && pcre_exec(shar_regex.re, shar_regex.extra, buf, strlen(buf), 0, 0, NULL, 0) >= 0)
-				fp_out = fopen(file_out, "w" FOPEN_OPTS);
+				fp_out = fopen(file_out, "w");
 
 			/* write to temp file */
 			if (fp_out != NULL)
@@ -1127,7 +1114,7 @@ print_art_seperator_line(
 	t_bool is_mailbox)
 {
 #ifdef DEBUG
-	if (debug == 2)
+	if (debug & DEBUG_MISC)
 		error_message("Mailbox=[%d], mailbox_format=[%s]", is_mailbox, txt_mailbox_formats[tinrc.mailbox_format]);
 #endif /* DEBUG */
 
@@ -1147,15 +1134,10 @@ start_viewer(
 	t_mailcap *foo;
 
 	if ((foo = get_mailcap_entry(part, path)) != NULL) {
-		char buff[LEN];
-
 		if (foo->nametemplate)	/* honor nametemplate */
 			rename_file(path, foo->nametemplate);
 
 		wait_message(0, _(txt_starting_command), foo->command);
-
-		/* are the () needed if foo->command holds more than one cmd? */
-		snprintf(buff, sizeof(buff), "(%s)", foo->command);
 		if (foo->needsterminal) {
 			set_xclick_off();
 			EndWin();
@@ -1215,9 +1197,9 @@ decode_save_one(
 
 		lookup_extension(extension, sizeof(extension), content_types[part->type], part->subtype);
 		generate_filename(buf, sizeof(buf), extension);
-		mbox = expand_save_filename(savepath, buf);
+		mbox = expand_save_filename(savepath, sizeof(savepath), buf);
 	} else
-		mbox = expand_save_filename(savepath, name);
+		mbox = expand_save_filename(savepath, sizeof(savepath), name);
 
 	/*
 	 * Not a good idea to dump attachments over a mailbox
@@ -1287,7 +1269,7 @@ decode_save_one(
 		}
 	} else {
 		snprintf(buf, sizeof(buf), _(txt_view_attachment), savepath, content_types[part->type], part->subtype);
-		if ((i = prompt_yn(cLINES, buf, TRUE)) == 1)
+		if ((i = prompt_yn(buf, TRUE)) == 1)
 			start_viewer(part, savepath);
 		else if (i == -1) {	/* Skip rest of attachments */
 			unlink(savepath);
@@ -1304,7 +1286,7 @@ decode_save_one(
 	}
 	if (!postproc) {
 		snprintf(buf, sizeof(buf), _(txt_save_attachment), savepath, content_types[part->type], part->subtype);
-		if ((i = prompt_yn(cLINES, buf, FALSE)) != 1) {
+		if ((i = prompt_yn(buf, FALSE)) != 1) {
 			unlink(savepath);
 			if (i == -1)	/* Skip rest of attachments */
 				return FALSE;
