@@ -3,10 +3,10 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2004-07-19
+ *  Updated   : 2008-03-11
  *  Notes     :
  *
- * Copyright (c) 1991-2004 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2008 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,20 +41,19 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-#ifndef MENUKEYS_H
-#	include "menukeys.h"
-#endif /* !MENUKEYS_H */
+
 
 /*
  * Local prototypes
  */
+static t_function select_left(void);
+static t_function select_right(void);
 static int active_comp(t_comptype p1, t_comptype p2);
 static int reposition_group(struct t_group *group, int default_num);
 static int save_restore_curr_group(t_bool saving);
-static int select_left(void);
-static int select_right(void);
 static t_bool pos_next_unread_group(t_bool redraw);
 static t_bool yanked_out = TRUE;
+static void build_gline(int i);
 static void catchup_group(struct t_group *group, t_bool goto_next_unread_group);
 static void draw_group_arrow(void);
 static void read_groups(void);
@@ -71,24 +70,26 @@ static void yank_active_file(void);
  * selmenu.curr = index (start at 0) of cursor position on menu,
  *                or -1 when no groups visible on screen
  * selmenu.max = Total # of groups in my_group[]
- * selmenu.first, selmenu.last are static here
+ * selmenu.first is static here
  */
-t_menu selmenu = { 1, 0, 0, 0, show_selection_page, draw_group_arrow };
+t_menu selmenu = { 1, 0, 0, show_selection_page, draw_group_arrow, build_gline };
+
+static int groupname_len;	/* max. group name length */
 
 
-static int
+static t_function
 select_left(
 	void)
 {
-	return iKeyQuit;
+	return GLOBAL_QUIT;
 }
 
 
-static int
+static t_function
 select_right(
 	void)
 {
-	return iKeySelectReadGrp;
+	return SELECT_ENTER_GROUP;
 }
 
 
@@ -99,7 +100,8 @@ selection_page(
 {
 	char buf[LEN];
 	char key[MAXKEYLEN];
-	int i, n, ch;
+	int i, n;
+	t_function func;
 
 	selmenu.curr = start_groupnum;
 
@@ -116,6 +118,7 @@ selection_page(
 	if (num_cmd_line_groups == 1)
 		select_read_group();
 
+	cursoroff();
 	show_selection_page();	/* display group selection page */
 
 	forever {
@@ -129,59 +132,68 @@ selection_page(
 
 		set_xclick_on();
 
-		switch ((ch = handle_keypad(select_left, select_right, &menukeymap.select_nav))) {
-			case iKeyAbort:		/* Abort */
+		switch ((func = handle_keypad(select_left, select_right, global_mouse_action, select_keys))) {
+			case GLOBAL_ABORT:		/* Abort */
 				break;
 
-			case '1': case '2': case '3': case '4': case '5':
-			case '6': case '7': case '8': case '9':
+			case DIGIT_1:
+			case DIGIT_2:
+			case DIGIT_3:
+			case DIGIT_4:
+			case DIGIT_5:
+			case DIGIT_6:
+			case DIGIT_7:
+			case DIGIT_8:
+			case DIGIT_9:
 				if (selmenu.max)
-					prompt_item_num(ch, _(txt_select_group));
+					prompt_item_num(func_to_key(func, select_keys), _(txt_select_group));
 				else
 					info_message(_(txt_no_groups));
 				break;
 
 #ifndef NO_SHELL_ESCAPE
-			case iKeyShellEscape:
+			case GLOBAL_SHELL_ESCAPE:
 				do_shell_escape();
 				break;
 #endif /* !NO_SHELL_ESCAPE */
 
-			case iKeyFirstPage:	/* show first page of groups */
+			case GLOBAL_FIRST_PAGE:		/* show first page of groups */
 				top_of_list();
 				break;
 
-			case iKeyLastPage:	/* show last page of groups */
+			case GLOBAL_LAST_PAGE:		/* show last page of groups */
 				end_of_list();
 				break;
 
-			case iKeyPageUp:		/* page up */
-			case iKeyPageUp2:
-			case iKeyPageUp3:
+			case GLOBAL_PAGE_UP:
 				page_up();
 				break;
 
-			case iKeyPageDown:		/* page down */
-			case iKeyPageDown2:
-			case iKeyPageDown3:
+			case GLOBAL_PAGE_DOWN:
 				page_down();
 				break;
 
-			case iKeyUp:		/* line up */
-			case iKeyUp2:
+			case GLOBAL_LINE_UP:
 				move_up();
 				break;
 
-			case iKeyDown:		/* line down */
-			case iKeyDown2:
+			case GLOBAL_LINE_DOWN:
 				move_down();
 				break;
 
-			case iKeySelectSortActive:	/* Sort active groups */
+			case GLOBAL_SCROLL_DOWN:
+				scroll_down();
+				break;
+
+			case GLOBAL_SCROLL_UP:
+				scroll_up();
+				break;
+
+			case SELECT_SORT_ACTIVE:	/* sort active groups */
 				sort_active_file();
 				break;
 
-			case iKeySetRange:	/* set range */
+			case GLOBAL_SET_RANGE:
 				if (selmenu.max) {
 					if (set_range(SELECT_LEVEL, 1, selmenu.max, selmenu.curr + 1))
 						show_selection_page();
@@ -189,42 +201,40 @@ selection_page(
 					info_message(_(txt_no_groups));
 				break;
 
-			case iKeySearchSubjF:	/* search forward */
-			case iKeySearchSubjB:	/* search backward */
-			case iKeySearchRepeat:
-				if (ch == iKeySearchRepeat && i_key_search_last != iKeySearchSubjF && i_key_search_last != iKeySearchSubjB)
-					info_message(_(txt_bad_command), printascii(key, map_to_local(iKeyHelp, &menukeymap.select_nav)));
+			case GLOBAL_SEARCH_SUBJECT_FORWARD:
+			case GLOBAL_SEARCH_SUBJECT_BACKWARD:
+			case GLOBAL_SEARCH_REPEAT:
+				if (func == GLOBAL_SEARCH_REPEAT && last_search != GLOBAL_SEARCH_SUBJECT_FORWARD && last_search != GLOBAL_SEARCH_SUBJECT_BACKWARD)
+					info_message(_(txt_bad_command), printascii(key, key_to_func(GLOBAL_HELP, select_keys)));
 				else {
-					if ((i = search_active((ch == iKeySearchSubjF), (ch == iKeySearchRepeat))) != -1) {
+					if ((i = search_active((func == GLOBAL_SEARCH_SUBJECT_FORWARD), (func == GLOBAL_SEARCH_REPEAT))) != -1) {
 						move_to_item(i);
 						clear_message();
 					}
 				}
 				break;
 
-			case iKeySelectReadGrp:	/* go into group */
-			case iKeySelectReadGrp2:
+			case SELECT_ENTER_GROUP:		/* go into group */
 				select_read_group();
 				break;
 
-			case iKeySelectEnterNextUnreadGrp:	/* enter next group containing unread articles */
-			case iKeySelectEnterNextUnreadGrp2:
+			case SELECT_ENTER_NEXT_UNREAD_GROUP:	/* enter next group containing unread articles */
 				if (pos_next_unread_group(FALSE))
 					read_groups();
 				break;							/* Nothing more to do at the moment */
 
-			case iKeyRedrawScr:		/* redraw */
+			case GLOBAL_REDRAW_SCREEN:
 				my_retouch();					/* TODO: not done elsewhere, maybe should be in show_selection_page */
 				set_xclick_off();
 				show_selection_page();
 				break;
 
-			case iKeySelectResetNewsrc:	/* reset .newsrc */
+			case SELECT_RESET_NEWSRC:		/* reset .newsrc */
 				if (no_write) {
 					info_message(_(txt_info_no_write));
 					break;
 				}
-				if (prompt_yn(cLINES, _(txt_reset_newsrc), FALSE) == 1) {
+				if (prompt_yn(_(txt_reset_newsrc), FALSE) == 1) {
 					reset_newsrc();
 					sync_active_file();
 					selmenu.curr = 0;
@@ -232,44 +242,61 @@ selection_page(
 				}
 				break;
 
-			case iKeySelectCatchup:	/* catchup - mark all articles as read */
-			case iKeySelectCatchupNextUnread:	/* and goto next unread group */
+			case CATCHUP:			/* catchup - mark all articles as read */
+			case CATCHUP_NEXT_UNREAD:	/* and goto next unread group */
 				if (selmenu.max)
-					catchup_group(&CURR_GROUP, (ch == iKeySelectCatchupNextUnread));
+					catchup_group(&CURR_GROUP, (func == CATCHUP_NEXT_UNREAD));
 				else
 					info_message(_(txt_no_groups));
 				break;
 
-			case iKeySelectToggleDescriptions:	/* toggle newsgroup descriptions */
+			case GLOBAL_EDIT_FILTER:
+				if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
+					break;
+				(void) read_filter_file(filter_file);
+				break;
+
+			case SELECT_TOGGLE_DESCRIPTIONS:	/* toggle newsgroup descriptions */
 				show_description = bool_not(show_description);
 				if (show_description)
 					read_descriptions(TRUE);
 				show_selection_page();
 				break;
 
-			case iKeySelectGoto:			/* prompt for a new group name */
-				if ((n = choose_new_group()) >= 0)
-					move_to_item(n);
+			case SELECT_GOTO:			/* prompt for a new group name */
+				{
+					int oldmax = selmenu.max;
+
+					if ((n = choose_new_group()) >= 0) {
+						/*
+						 * If a new group was added and it is on the actual screen
+						 * draw it. If it is off screen the redraw will handle it.
+						 */
+						if (oldmax != selmenu.max && n >= selmenu.first && n < selmenu.first + NOTESLINES)
+							build_gline(n);
+						move_to_item(n);
+					}
+				}
 				break;
 
-			case iKeyHelp:					/* help */
+			case GLOBAL_HELP:
 				show_help_page(SELECT_LEVEL, _(txt_group_select_com));
 				show_selection_page();
 				break;
 
-			case iKeyToggleHelpDisplay:		/* toggle mini help menu */
+			case GLOBAL_TOGGLE_HELP_DISPLAY:	/* toggle mini help menu */
 				toggle_mini_help(SELECT_LEVEL);
 				show_selection_page();
 				break;
 
-			case iKeyToggleInverseVideo:	/* toggle inverse video */
+			case GLOBAL_TOGGLE_INVERSE_VIDEO:
 				toggle_inverse_video();
 				show_selection_page();
 				show_inverse_video_status();
 				break;
 
 #ifdef HAVE_COLOR
-			case iKeyToggleColor:			/* toggle color */
+			case GLOBAL_TOGGLE_COLOR:
 				if (toggle_color()) {
 					show_selection_page();
 					show_color_status();
@@ -277,12 +304,12 @@ selection_page(
 				break;
 #endif /* HAVE_COLOR */
 
-			case iKeyToggleInfoLastLine:	/* display group description */
+			case GLOBAL_TOGGLE_INFO_LAST_LINE:	/* display group description */
 				tinrc.info_in_last_line = bool_not(tinrc.info_in_last_line);
 				show_selection_page();
 				break;
 
-			case iKeySelectMoveGrp:	/* reposition group within group list */
+			case SELECT_MOVE_GROUP:			/* reposition group within group list */
 				/* TODO: move all this to reposition_group() */
 				if (!selmenu.max) {
 					info_message(_(txt_no_groups));
@@ -302,7 +329,7 @@ selection_page(
 				n = selmenu.curr;
 				selmenu.curr = reposition_group(&active[my_group[n]], n);
 				HpGlitch(erase_arrow());
-				if (selmenu.curr < selmenu.first || selmenu.curr >= selmenu.last || selmenu.curr != n)
+				if (selmenu.curr < selmenu.first || selmenu.curr >= selmenu.first + NOTESLINES - 1 || selmenu.curr != n)
 					show_selection_page();
 				else {
 					i = selmenu.curr;
@@ -314,31 +341,31 @@ selection_page(
 				}
 				break;
 
-			case iKeyOptionMenu:	/* option menu */
-				(void) change_config_file(NULL);
+			case GLOBAL_OPTION_MENU:
+				change_config_file(NULL);
 				read_attributes_files();
 				show_selection_page();
 				break;
 
-			case iKeySelectNextUnreadGrp:	/* goto next unread group */
+			case SELECT_NEXT_UNREAD_GROUP:		/* goto next unread group */
 				pos_next_unread_group(TRUE);
 				break;
 
-			case iKeyQuit:	/* quit */
+			case GLOBAL_QUIT:			/* quit */
 				select_done();
 				break;
 
-			case iKeyQuitTin:	/* quit, no ask */
+			case GLOBAL_QUIT_TIN:			/* quit, no ask */
 				select_quit();
 				break;
 
-			case iKeySelectQuitNoWrite:	/* quit, but don't save configuration */
-				if (prompt_yn(cLINES, _(txt_quit_no_write), TRUE) == 1)
+			case SELECT_QUIT_NO_WRITE:		/* quit, but don't save configuration */
+				if (prompt_yn(_(txt_quit_no_write), TRUE) == 1)
 					tin_done(EXIT_SUCCESS);
 				show_selection_page();
 				break;
 
-			case iKeySelectToggleReadDisplay:
+			case SELECT_TOGGLE_READ_DISPLAY:
 				/*
 				 * If in tinrc.show_only_unread_groups mode toggle all
 				 * subscribed to groups and only groups that contain unread
@@ -357,11 +384,11 @@ selection_page(
 					info_message(_(txt_show_unread));
 				break;
 
-			case iKeySelectBugReport:
+			case GLOBAL_BUGREPORT:
 				bug_report();
 				break;
 
-			case iKeySelectSubscribe:	/* subscribe to current group */
+			case SELECT_SUBSCRIBE:			/* subscribe to current group */
 				if (!selmenu.max) {
 					info_message(_(txt_no_groups));
 					break;
@@ -378,7 +405,7 @@ selection_page(
 				}
 				break;
 
-			case iKeySelectSubscribePat:	/* subscribe to groups matching pattern */
+			case SELECT_SUBSCRIBE_PATTERN:		/* subscribe to groups matching pattern */
 				if (no_write) {
 					info_message(_(txt_info_no_write));
 					break;
@@ -386,7 +413,7 @@ selection_page(
 				subscribe_pattern(_(txt_subscribe_pattern), _(txt_subscribing), _(txt_subscribed_num_groups), TRUE);
 				break;
 
-			case iKeySelectUnsubscribe:	/* unsubscribe to current group */
+			case SELECT_UNSUBSCRIBE:		/* unsubscribe to current group */
 				if (!selmenu.max) {
 					info_message(_(txt_no_groups));
 					break;
@@ -413,7 +440,7 @@ selection_page(
 				}
 				break;
 
-			case iKeySelectUnsubscribePat:	/* unsubscribe to groups matching pattern */
+			case SELECT_UNSUBSCRIBE_PATTERN:	/* unsubscribe to groups matching pattern */
 				if (no_write) {
 					info_message(_(txt_info_no_write));
 					break;
@@ -422,16 +449,20 @@ selection_page(
 								_(txt_unsubscribing), _(txt_unsubscribed_num_groups), FALSE);
 				break;
 
-			case iKeyVersion:	/* show tin version */
+			case GLOBAL_VERSION:			/* show tin version */
 				info_message(cvers);
 				break;
 
-			case iKeyPost:	/* post a basenote */
+			case GLOBAL_POST:			/* post a basenote */
 				if (!selmenu.max) {
+					if (!can_post) {
+						info_message(_(txt_cannot_post));
+						break;
+					}
 					snprintf(buf, sizeof(buf), _(txt_post_newsgroups), tinrc.default_post_newsgroups);
 					if (!prompt_string_default(buf, tinrc.default_post_newsgroups, _(txt_no_newsgroups), HIST_POST_NEWSGROUPS))
 						break;
-					if (group_find(tinrc.default_post_newsgroups) == NULL) {
+					if (group_find(tinrc.default_post_newsgroups, FALSE) == NULL) {
 						error_message(_(txt_not_in_active_file), tinrc.default_post_newsgroups);
 						break;
 					} else {
@@ -442,7 +473,7 @@ selection_page(
 						 * CHARSET_CONVERSION conversion case in new_part()
 						 * which relies currently relies on CURR_GROUP
 						 */
-						selmenu.curr = my_group_add(buf);
+						selmenu.curr = my_group_add(buf, FALSE);
 						/*
 						 * and the next hack to avoid a grabbled selection
 						 * screen after the posting
@@ -461,8 +492,7 @@ selection_page(
 					show_selection_page();
 				break;
 
-			case iKeyPostponed:
-			case iKeyPostponed2:	/* post postponed article */
+			case GLOBAL_POSTPONED:			/* post postponed article */
 				if (can_post) {
 					if (pickup_postponed_articles(FALSE, FALSE))
 						show_selection_page();
@@ -470,23 +500,22 @@ selection_page(
 					info_message(_(txt_cannot_post));
 				break;
 
-			case iKeyDisplayPostHist:	/* display messages posted by user */
+			case GLOBAL_DISPLAY_POST_HISTORY:	/* display messages posted by user */
 				if (user_posted_messages())
 					show_selection_page();
 				break;
 
-			case iKeySelectYankActive:	/* yank in/out rest of groups from active */
+			case SELECT_YANK_ACTIVE:		/* yank in/out rest of groups from active */
 				yank_active_file();
 				break;
 
-			case iKeySelectSyncWithActive:	/* Re-read active file to see if any new news */
+			case SELECT_SYNC_WITH_ACTIVE:		/* Re-read active file to see if any new news */
 				sync_active_file();
 				if (!yanked_out)
 					yank_active_file();			/* yank out if yanked in */
 				break;
 
-			case iKeySelectMarkGrpUnread:
-			case iKeySelectMarkGrpUnread2:	/* mark group unread */
+			case SELECT_MARK_GROUP_UNREAD:
 				if (!selmenu.max) {
 					info_message(_(txt_no_groups));
 					break;
@@ -504,7 +533,7 @@ selection_page(
 				break;
 
 			default:
-				info_message(_(txt_bad_command), printascii(key, map_to_local(iKeyHelp, &menukeymap.select_nav)));
+				info_message(_(txt_bad_command), printascii(key, func_to_key(GLOBAL_HELP, select_keys)));
 		}
 	}
 }
@@ -515,13 +544,7 @@ show_selection_page(
 	void)
 {
 	char buf[LEN];
-	char tmp[10];
-	char active_name[255];
-	char group_descript[255];
-	char subs;
-	int i, j, n;
-	int blank_len;
-	int len, groupname_len = 0;
+	int i, len;
 
 	signal_context = cSelect;
 	currmenu = &selmenu;
@@ -531,17 +554,12 @@ show_selection_page(
 	else
 		snprintf(buf, sizeof(buf), "%s (%d%s)", _(txt_group_selection), selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
 
-	MoveCursor(0, 0);		/* top left corner */
-	CleartoEOLN();
-	show_title(buf);
-	MoveCursor(1, 0);
-	CleartoEOLN();
-	MoveCursor(INDEX_TOP, 0);
-
 	if (selmenu.curr < 0)
 		selmenu.curr = 0;
 
+	ClearScreen();
 	set_first_screen_item();
+	show_title(buf);
 
 	/*
 	 * calculate max length of groupname field
@@ -574,83 +592,90 @@ show_selection_page(
 	if (groupname_len < 0)
 		groupname_len = 0;
 
-	blank_len = (MIN(cCOLS, (int) sizeof(group_descript)) - (groupname_len + SELECT_MISC_COLS)) + (show_description ? 2 : 4);
+	for (i = selmenu.first; i < selmenu.first + NOTESLINES && i < selmenu.max; i++)
+		build_gline(i);
 
-	for (j = 0, i = selmenu.first; i < selmenu.last; i++, j++) {
-#ifdef USE_CURSES
-		char sptr[BUFSIZ];
-#else
-		char *sptr = screen[j].col;
-#endif /* USE_CURSES */
-		if (active[my_group[i]].inrange)
-			strcpy(tmp, "    #");
-		else if (active[my_group[i]].newsrc.num_unread) {
-			strcpy(tmp, tin_ltoa(active[my_group[i]].newsrc.num_unread, 5));
-		} else
-			strcpy(tmp, "     ");
-
-		n = my_group[i];
-
-		/*
-		 * Display a flag for this group if needed
-		 * . Bogus groups are dumped immediately
-		 * . Normal subscribed groups may be
-		 *   ' ' normal, 'X' not postable, 'M' moderated, '=' renamed
-		 * . Newgroups are 'N'
-		 * . Unsubscribed groups are 'u'
-		 */
-		if (active[n].bogus)					/* Group is not in active list */
-			subs = 'D';
-		else if (active[n].subscribed)			/* Important that this preceeds Newgroup */
-			subs = group_flag(active[n].moderated);
-		else
-			subs = ((active[n].newgroup) ? 'N' : 'u'); /* New (but unsubscribed) group or unsubscribed group */
-
-		strncpy(active_name, active[n].name, groupname_len);
-		active_name[groupname_len] = '\0';
-
-		if (blank_len > (int) (sizeof(group_descript) - 1))
-			blank_len = sizeof(group_descript) - 1;
-
-		if (show_description) {
-			if (active[n].description) {
-				strncpy(group_descript, active[n].description, blank_len);
-				group_descript[blank_len] = '\0';
-				sprintf(sptr, "  %c %s %s  %-*.*s  %-*.*s%s",
-				         subs, tin_ltoa(i + 1, 4), tmp,
-				         groupname_len, groupname_len, active_name,
-				         blank_len, blank_len, group_descript, cCRLF);
-			} else
-				sprintf(sptr, "  %c %s %s  %-*.*s  %s",
-				         subs, tin_ltoa(i + 1, 4), tmp,
-				         (groupname_len + blank_len),
-				         (groupname_len + blank_len), active[n].name, cCRLF);
-		} else {
-			if (tinrc.draw_arrow)
-				sprintf(sptr, "  %c %s %s  %-*.*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, cCRLF);
-			else
-				sprintf(sptr, "  %c %s %s  %-*.*s%*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, blank_len, " ", cCRLF);
-		}
-		if (tinrc.strip_blanks)
-			strcat(strip_line(sptr), cCRLF);
-
-		CleartoEOLN();
-		my_fputs(sptr, stdout);
-	}
-
-	CleartoEOS();
 	show_mini_help(SELECT_LEVEL);
 
 	if (selmenu.max <= 0) {
 		info_message(_(txt_no_groups));
 		return;
-	} else if (selmenu.last == selmenu.max)
-		info_message(_(txt_end_of_groups));
+	}
 
 	draw_group_arrow();
 }
 
 
+static void
+build_gline(
+	int i)
+{
+	char tmp[10];
+	char active_name[255];
+	char group_descript[255];
+	char subs;
+#ifdef USE_CURSES
+	char sptr[BUFSIZ];
+#else
+	char *sptr = screen[INDEX2SNUM(i)].col;
+#endif /* USE_CURSES */
+	int n, blank_len;
+
+	blank_len = (MIN(cCOLS, (int) sizeof(group_descript)) - (groupname_len + SELECT_MISC_COLS)) + (show_description ? 2 : 4);
+
+	if (active[my_group[i]].inrange)
+		strcpy(tmp, "    #");
+	else if (active[my_group[i]].newsrc.num_unread) {
+		strcpy(tmp, tin_ltoa(active[my_group[i]].newsrc.num_unread, 5));
+	} else
+		strcpy(tmp, "     ");
+
+	n = my_group[i];
+
+	/*
+	 * Display a flag for this group if needed
+	 * . Bogus groups are dumped immediately
+	 * . Normal subscribed groups may be
+	 *   ' ' normal, 'X' not postable, 'M' moderated, '=' renamed
+	 * . Newgroups are 'N'
+	 * . Unsubscribed groups are 'u'
+	 */
+	if (active[n].bogus)					/* Group is not in active list */
+		subs = 'D';
+	else if (active[n].subscribed)			/* Important that this preceeds Newgroup */
+		subs = group_flag(active[n].moderated);
+	else
+		subs = ((active[n].newgroup) ? 'N' : 'u'); /* New (but unsubscribed) group or unsubscribed group */
+
+	strncpy(active_name, active[n].name, groupname_len);
+	active_name[groupname_len] = '\0';
+
+	if (show_description) {
+		if (active[n].description) {
+			strncpy(group_descript, active[n].description, blank_len);
+			group_descript[blank_len] = '\0';
+			sprintf(sptr, "  %c %s %s  %-*.*s  %-*.*s%s",
+				 subs, tin_ltoa(i + 1, 4), tmp,
+				 groupname_len, groupname_len, active_name,
+				 blank_len, blank_len, group_descript, cCRLF);
+		} else
+			sprintf(sptr, "  %c %s %s  %-*.*s  %s",
+				 subs, tin_ltoa(i + 1, 4), tmp,
+				 (groupname_len + blank_len),
+				 (groupname_len + blank_len), active[n].name, cCRLF);
+	} else {
+		if (tinrc.draw_arrow)
+			sprintf(sptr, "  %c %s %s  %-*.*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, cCRLF);
+		else
+			sprintf(sptr, "  %c %s %s  %-*.*s%*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, blank_len, " ", cCRLF);
+	}
+	if (tinrc.strip_blanks)
+		strcat(strip_line(sptr), cCRLF);
+
+	WriteLine(INDEX2LNUM(i), sptr);
+}
+
+
 static void
 draw_group_arrow(
 	void)
@@ -663,6 +688,8 @@ draw_group_arrow(
 			info_message(_(txt_group_aliased), CURR_GROUP.aliasedto);
 		else if (tinrc.info_in_last_line)
 			info_message("%s", CURR_GROUP.description ? CURR_GROUP.description : _(txt_no_description));
+		else if (selmenu.curr == selmenu.max - 1)
+			info_message(_(txt_end_of_groups));
 	}
 }
 
@@ -789,8 +816,8 @@ int
 choose_new_group(
 	void)
 {
-	int idx;
 	char *prompt;
+	int idx;
 
 	prompt = fmt_string(_(txt_newsgroup), tinrc.default_goto_group);
 
@@ -807,7 +834,7 @@ choose_new_group(
 
 	clear_message();
 
-	if ((idx = my_group_add(tinrc.default_goto_group)) == -1)
+	if ((idx = my_group_add(tinrc.default_goto_group, TRUE)) == -1)
 		info_message(_(txt_not_in_active_file), tinrc.default_goto_group);
 
 	return idx;
@@ -844,11 +871,12 @@ skip_newgroups(
 int
 add_my_group(
 	const char *group,
-	t_bool add)
+	t_bool add,
+	t_bool ignore_case)
 {
 	int i, j;
 
-	if ((i = find_group_index(group)) < 0)
+	if ((i = find_group_index(group, ignore_case)) < 0)
 		return -1;
 
 	for (j = 0; j < selmenu.max; j++) {
@@ -936,7 +964,7 @@ catchup_group(
 {
 	char *smsg = NULL;
 
-	if ((!TINRC_CONFIRM_ACTION) || prompt_yn(cLINES, sized_message(&smsg, _(txt_mark_group_read), group->name), TRUE) == 1) {
+	if ((!TINRC_CONFIRM_ACTION) || prompt_yn(sized_message(&smsg, _(txt_mark_group_read), group->name), TRUE) == 1) {
 		grp_mark_read(group, NULL);
 		mark_screen(selmenu.curr, 9, "     ");
 
@@ -1106,14 +1134,14 @@ toggle_my_groups(
 		if ((ptr = strchr(buf, SUBSCRIBED)) != NULL) {
 			*ptr = '\0';
 
-			if ((i = find_group_index(buf)) < 0)
+			if ((i = find_group_index(buf, FALSE)) < 0)
 				continue;
 
 			if (tinrc.show_only_unread_groups) {
 				if (active[i].newsrc.num_unread || (active[i].bogus && tinrc.strip_bogus == BOGUS_SHOW))
-					my_group_add(buf);
+					my_group_add(buf, FALSE);
 			} else
-				my_group_add(buf);
+				my_group_add(buf, FALSE);
 		}
 	}
 	fclose(fp);
@@ -1123,8 +1151,8 @@ toggle_my_groups(
 
 
 /*
- * Subscribe or unsubscribe from a list of groups. List can be full list as supported
- * by match_group_list()
+ * Subscribe or unsubscribe from a list of groups. List can be full list as
+ * supported by match_group_list()
  */
 static void
 subscribe_pattern(
@@ -1146,37 +1174,21 @@ subscribe_pattern(
 
 	wait_message(0, message);
 
-	/*
-	 * TODO: why do we do a pass over my_group[] before another one
-	 *       over active[]? AFAICS the first loop can go away.
-	 */
-	for (i = 0; i < selmenu.max; i++) {
-		if (match_group_list(active[my_group[i]].name, buf)) {
-			if (active[my_group[i]].subscribed != (state != FALSE)) {
+	for_each_group(i) {
+		if (match_group_list(active[i].name, buf)) {
+			if (active[i].subscribed != (state != FALSE)) {
 				spin_cursor();
-				subscribe(&active[my_group[i]], SUB_CHAR(state), TRUE);
-				subscribe_num++;
-			}
-		}
-	}
-
-	if (num_active > selmenu.max) {			/* ie, there are groups yanked out */
-		for_each_group(i) {
-			if (match_group_list(active[i].name, buf)) {
-				if (active[i].subscribed != (state != FALSE)) {
-					spin_cursor();
-					/* If found and group is not subscribed add it to end of my_group[]. */
-					subscribe(&active[i], SUB_CHAR(state), TRUE);
-					if (state) {
-						my_group_add(active[i].name);
-						/*
-						 * TODO: grp_mark_unread() or something needs to do a
-						 *       group_get_art_info() to get initial count right
-						 */
-						grp_mark_unread(&active[i]);
-					}
-					subscribe_num++;
+				/* If found and group is not subscribed add it to end of my_group[]. */
+				subscribe(&active[i], SUB_CHAR(state), TRUE);
+				if (state) {
+					my_group_add(active[i].name, FALSE);
+					/*
+					 * TODO: grp_mark_unread() or something needs to do a
+					 *       group_get_art_info() to get initial count right
+					 */
+					grp_mark_unread(&active[i]);
 				}
+				subscribe_num++;
 			}
 		}
 	}
@@ -1206,9 +1218,9 @@ static void
 select_done(
 	void)
 {
-	if (!TINRC_CONFIRM_TO_QUIT || prompt_yn(cLINES, _(txt_quit), TRUE) == 1)
+	if (!TINRC_CONFIRM_TO_QUIT || prompt_yn(_(txt_quit), TRUE) == 1)
 		select_quit();
-	if (!no_write && prompt_yn(cLINES, _(txt_save_config), TRUE) == 1) {
+	if (!no_write && prompt_yn(_(txt_save_config), TRUE) == 1) {
 		write_config_file(local_config_file);
 		write_newsrc();
 	}
