@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2004-08-21
+ *  Updated   : 2008-03-26
  *  Notes     :
  *
- * Copyright (c) 1997-2004 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2008 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,27 +65,37 @@
 /*
  * local prototypes
  */
-#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static UChar *char2UChar(const char *str);
-	static char *UChar2char(const UChar *ustr);
-#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && !NO_LOCALE */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	static wchar_t *my_wcsdup(const wchar_t *wstr);
+#	if defined(HAVE_LIBICUUC)
+		static UChar *char2UChar(const char *str);
+		static char *UChar2char(const UChar *ustr);
+#	endif /* HAVE_LIBICUUC */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 
 /*
  * special ltoa()
- * converts value into a string with a len of digits
- * last char may be one of the following
- * Kilo, Mega, Giga, Terra
+ * converts value into a string with a maxlen of digits (usualy should be
+ * >=4), last char may be one of the following:
+ * 'K'ilo, 'M'ega, 'G'iga, 'T'erra, 'P'eta, 'E'xa, 'Z'etta, 'Y'otta,
+ * 'X'ona, 'W'eka, 'V'unda, 'U'da (these last 4 are no official SI-prefixes)
+ * or 'e' if an error occurs
  */
 char *
 tin_ltoa(
 	long value,
 	int digits)
 {
-	static char buffer[256];
-	static const char power[] = " KMGT";
+	static char buffer[64];
+	static const char power[] = { ' ', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'X', 'W', 'V', 'U', '\0' };
 	int len;
-	int i = 0;
+	size_t i = 0;
+
+	if (digits <= 0) {
+		buffer[0] = 'e';
+		return buffer;
+	}
 
 	snprintf(buffer, sizeof(buffer), "%ld", value);
 	len = (int) strlen(buffer);
@@ -95,11 +105,21 @@ tin_ltoa(
 		i++;
 	}
 
+	if (i >= strlen(power)) {	/* buffer is to small */
+		buffer[(digits & 0x7f) - 1] = 'e';
+		buffer[(digits & 0x7f)] = '\0';
+		return buffer;
+	}
+
 	if (i) {
 		while (len < (digits - 1))
 			buffer[len++] = ' ';
 
-		buffer[digits - 1] = power[i];
+		if (digits > len)
+			buffer[digits - 1] = power[i];
+		else /* overflow */
+			buffer[digits - 1] = 'e';
+
 		buffer[digits] = '\0';
 	} else
 		snprintf(buffer, sizeof(buffer), "%*ld", digits, value);
@@ -108,6 +128,7 @@ tin_ltoa(
 }
 
 
+#if !defined(USE_DMALLOC) || (defined(USE_DMALLOC) && !defined(HAVE_STRDUP))
 /*
  * Handrolled version of strdup(), presumably to take advantage of
  * the enhanced error detection in my_malloc
@@ -121,15 +142,15 @@ my_strdup(
 	size_t len = strlen(str) + 1;
 	void *ptr = my_malloc(len);
 
-#if 0 /* as my_malloc exits on error, ptr can't be NULL */
+#	if 0 /* as my_malloc exits on error, ptr can't be NULL */
 	if (ptr == NULL)
 		return NULL;
-#endif /* 0 */
+#	endif /* 0 */
 
 	memcpy(ptr, str, len);
 	return (char *) ptr;
 }
-
+#endif /* !USE_DMALLOC || (USE_DMALLOC && !HAVE_STRDUP) */
 
 /*
  * strtok that understands empty tokens
@@ -405,7 +426,7 @@ OUT:
 
 	return (sign * (-val));
 }
-#	undef DIGIT(x)
+#	undef DIGIT
 #	undef MBASE
 #endif /* !HAVE_STRTOL */
 
@@ -448,6 +469,28 @@ strncasecmp(
 }
 #endif /* !HAVE_STRNCASECMP */
 
+#ifndef HAVE_STRSEP
+/*
+ * strsep() is not mandatory in ANSI-C
+ */
+char *strsep(
+	char **stringp,
+	const char *delim)
+{
+	char *s = *stringp;
+	char *p;
+
+	if (!s)
+		return NULL;
+
+	if ((p = strpbrk(s, delim)) != NULL)
+		*p++ = '\0';
+
+	*stringp = p;
+	return s;
+}
+#endif /* !HAVE_STRSEP */
+
 
 /*
  * str_trim - leading and trailing whitespace
@@ -466,6 +509,9 @@ str_trim(
 	char *wp;		/* writing string pointer */
 	char *ls;		/* last space */
 
+	if (string == NULL)
+		return NULL;
+
 	for (rp = wp = ls = string; isspace((int) *rp); rp++)		/* Skip leading space */
 		;
 
@@ -539,11 +585,11 @@ sh_format(
 	const char *fmt,
 	...)
 {
-	int result = 0;
 	char *src;
 	char temp[20];
-	va_list ap;
+	int result = 0;
 	int quote = 0;
+	va_list ap;
 
 	va_start(ap, fmt);
 
@@ -699,8 +745,8 @@ char *
 wchar_t2char(
 	const wchar_t *wstr)
 {
-	size_t len;
 	char *str;
+	size_t len;
 
 	len = wcstombs(NULL, wstr, 0);
 	if (len == (size_t) (-1))
@@ -766,44 +812,37 @@ wcspart(
 char *
 strunc(
 	const char *message,
-	char *buf,
-	size_t buf_len,
 	int len)
 {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	char *tmp;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	wchar_t *wmessage, *wbuf;
-	size_t wbuf_len;
 
 	if ((wmessage = char2wchar_t(message)) != NULL) {
-		wbuf_len = wcslen(wmessage) + 1;
-		wbuf = my_malloc(sizeof(wchar_t) * wbuf_len);
-
-		wstrunc(wmessage, wbuf, wbuf_len, len);
+		wbuf = wstrunc(wmessage, len);
 		free(wmessage);
 
 		if ((tmp = wchar_t2char(wbuf)) != NULL) {
-			strncpy(buf, tmp, buf_len);
-			buf[buf_len - 1] = '\0';
-
 			free(wbuf);
-			free(tmp);
 
-			return buf;
+			return tmp;
 		}
 		free(wbuf);
 	}
 	/* something went wrong using wide-chars, default back to normal chars */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-	if ((int) strlen(message) <= len && len < (int) buf_len)
-		strcpy(buf, message);
-	else
-		snprintf(buf, buf_len, "%-.*s%s", len - 3, message, TRUNC_TAIL);
+	if ((int) strlen(message) <= len)
+		tmp = my_strdup(message);
+	else {
+		tmp = my_malloc(len + 1);
+		snprintf(tmp, len + 1, "%-.*s%s", len - 3, message, TRUNC_TAIL);
+	}
 
-	return buf;
+	return tmp;
 }
 
+
 /*
  * if you use UTF-8 as local charset and want to use
  * U+2026 (HORIZONTAL_ELLIPSIS) instead of "..." uncomment
@@ -816,8 +855,6 @@ strunc(
 wchar_t *
 wstrunc(
 	const wchar_t *wmessage,
-	wchar_t *wbuf,
-	size_t wbuf_len,
 	int len)
 {
 	wchar_t *wtmp;
@@ -826,11 +863,9 @@ wstrunc(
 	wtmp = my_wcsdup(wmessage);
 	wconvert_to_printable(wtmp);
 
-	if (wcswidth(wtmp, wcslen(wtmp)) <= len && wcslen(wtmp) < wbuf_len) /* wtmp doesn't need to be truncated */
-		wcscpy(wbuf, wtmp);
-	else {
+	if (wcswidth(wtmp, wcslen(wtmp)) > len) {
 		/* wtmp must be truncated */
-		wchar_t *wtmp2, *format;
+		wchar_t *wtmp2, *tail;
 
 #	ifdef USE_UTF8_HORIZONTAL_ELLIPSIS
 		if (IS_LOCAL_CHARSET("UTF-8")) {
@@ -838,35 +873,28 @@ wstrunc(
 			 * use U+2026 (HORIZONTAL ELLIPSIS) instead of "..."
 			 * we gain two additional screen positions
 			 */
-			format = char2wchar_t("%ls%lc");
-
-			wtmp2 = wcspart(wtmp, MIN(len - 1, (int) wbuf_len - 2), FALSE);
-			swprintf(wbuf, wbuf_len, format, wtmp2, 8230); /* U+2026 */
+			tail = my_malloc(sizeof(wchar_t) * 2);
+			tail[0] = 8230; /* U+2026 */
+			tail[1] = 0;	/* \0 */
 		} else
 #	endif /* USE_UTF8_HORIZONTAL_ELLIPSIS */
-		{
-			wchar_t *tail;
-
 			tail = char2wchar_t(TRUNC_TAIL);
-			format = char2wchar_t("%ls%ls");
 
-			wtmp2 = wcspart(wtmp, MIN(len - 3, (int) wbuf_len - 4), FALSE);
-			swprintf(wbuf, wbuf_len, format, wtmp2, tail);
-			free(tail);
-		}
-		free(format);
-		free(wtmp2);
+		wtmp2 = wcspart(wtmp, len - wcslen(tail), FALSE);
+		free(wtmp);
+		wtmp = my_realloc(wtmp2, sizeof(wchar_t) * (wcslen(wtmp2) + wcslen(tail) + 1));	/* wtmp2 isn't valid snymore and doesn't have to be free()ed */
+		wcscat(wtmp, tail);
+		free(tail);
 	}
-	free(wtmp);
 
-	return wbuf;
+	return wtmp;
 }
 
 
 /*
  * duplicates a wide-char string
  */
-wchar_t *
+static wchar_t *
 my_wcsdup(
 	const wchar_t *wstr)
 {
@@ -913,8 +941,8 @@ static char *
 UChar2char(
 	const UChar *ustr)
 {
-	int32_t needed;
 	char *str;
+	int32_t needed;
 	UErrorCode status = U_ZERO_ERROR;
 
 	u_strToUTF8(NULL, 0, &needed, ustr, -1, &status);
@@ -1063,8 +1091,8 @@ render_bidi(
 	const char *str,
 	t_bool *is_rtl)
 {
-	int32_t ustr_len;
 	char *tmp;
+	int32_t ustr_len;
 	UBiDi *bidi_data;
 	UChar *ustr, *ustr_reordered;
 	UErrorCode status = U_ZERO_ERROR;
